;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 10.	Dibujar con Visual LISP.

(defun cmd-circulo (centro radio /)
  (cmd-entrar)
  (command "._circle" centro radio)
  (cmd-salir)
) ;_ fin de defun
;;;Listado 10.1. Dibujo de un crculo utilizando COMMAND.

(defun cmd-circulo (centro radio /)
  (setq tmp (getvar "OSMODE"))
  (setvar "OSMODE" 0)
  (setvar "CMDECHO" 0)
  (vl-cmdf "._circle" centro radio)
  (setvar "CMDECHO" 1)
  (setvar "OSMODE" tmp)
) ;_ fin de defun
;;;Listado 10.2. Dibujo de un crculo utilizando VL-CMDF.

(defun cmd-ttr (pt1 pt2 radio /)
  (setvar "cmdecho" 0)
  (vl-cmdf "._circle" "_ttr" pt1 pt2 radio)
  (setvar "cmdecho" 1)
) ;_ fin de defun
;;;Listado 10.3. Dibujo de un crculo a partir de dos puntos de tangencia y el radio.

(defun cmd-circulo (centro radio /)
  (if (> (getvar "CMDACTIVE") 0)
    (vl-cmdf)
  ) ;_ fin de if
  (cmd-entrar)
  (vl-cmdf "._circle" centro radio)
  (cmd-salir)
  (if (> (getvar "cmdactive") 0)
    (progn (vl-cmdf) nil)
    t
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 10.4. Funcin CMD-CIRCULO con control de errores.

(defun cmd (comando /)
  (if (> (getvar "cmdactive") 0)
    (vl-cmdf)
  ) ;_ fin de if
  (apply 'vl-cmdf (list comando))
  (while (> (getvar "cmdactive") 0)
    (vl-cmdf pause)
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 10.5. Funcin universal para ejecutar comandos de manera interactiva.

(defun cmd-test ()
  (alert "Pulse Aceptar para elegir un Color")
  (initdia)
  (cmd "._COLOR")
  (alert "Pulse aceptar para dibujar un Crculo")
  (cmd "._CIRCLE")
) ;_ fin de defun
;;;Listado 10.6. Funcin de que lanza comandos desde un programa AutoLISP.

(defun cmd-poly (lista-puntos 2d cerrado /)
  (if (> (getvar "cmdactive") 0)
    (vl-cmdf)
  ) ;_ fin de if
  (cmd-entrar)
  (vl-cmdf (if 2d
             "._pline"
             "._3dpoly"
           ) ;_ fin de if
  ) ;_ fin de vl-cmdf
  (foreach punto lista-puntos (vl-cmdf punto))
  (vl-cmdf (if cerrado
             "_cl"
             ""
           ) ;_ fin de if
  ) ;_ fin de vl-cmdf
  (cmd-salir)
  (if (> (getvar "cmdactive") 0)
    (progn (vl-cmdf) nil)
    t
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 10.7. Funcin para dibujar una polilnea 2D o 3D.

(defun ent-pt (xyz)
  (entmake (list '(0 . "POINT") (cons 10 xyz)))
) ;_ fin de defun
;;;Listado 10.8. Funcin para dibujo de un punto.

(defun valor-cod (clave ename)
  (cdr (assoc clave (entget ename)))
) ;_ fin de defun
;;;Listado 10.9. Extraccin del valor asociado a un cdigo DXF.

(defun ent-copia-obj (list-ent / ctr)
  (if (apply 'or
             (mapcar '(lambda (x) (= x (cdr (assoc 0 list-ent))))
                     '("CIRCLE" "ELLIPSE" "ARC" "INSERT" "POINT"
                       "SHAPE" "TEXT" "MTEXT"
                      )
             ) ;_ fin de mapcar
      ) ;_ fin de apply
    (while (setq ctr (getpoint "\nNuevo centro: "))
      (entmake
        (subst (cons 10 ctr) (assoc 10 list-ent) list-ent)
      ) ;_ fin de entmake
    ) ;_ fin de while
    (prompt "\nObjeto no admitido")
  ) ;_ fin de if
  (princ)
) ;_ fin de defun
;;;Listado 10.10. Funcin que copia objetos utilizando ENTMAKE.

(defun ent-circ (centro radio capa vec-normal)
  (entmake
    (list
      '(0 . "CIRCLE")
      '(100 . "AcDbEntity")
      '(100 . "AcDbCircle")
      (cons 8 capa)
      (cons 10 centro)
      (cons 40 radio)
      (cons 210 vec-normal)
    ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.11. Funcin para dibujar crculos en distintas capas y planos.

(defun valores (clave lista / sublista resultado)
  (while (setq sublista (assoc clave lista))
    (setq resultado (cons (cdr sublista) resultado)
          lista     (cdr (member sublista lista))
    ) ;_ fin de setq
  ) ;_ fin de while
  (reverse resultado)
) ;_ fin de defun
;;;Listado 10.12. Extraccin de valores mltiples de una lista de asociacin.

(defun vert-poly (lista / coord-z)
  (setq coord-z (cdr (assoc 38 lista)))
  (mapcar '(lambda (2d)
             (reverse (cons coord-z (reverse 2d)))
           ) ;_ fin de lambda
          (valores 10 lista)
  ) ;_ fin de mapcar
) ;_ fin de defun
;;;Listado 10.13. Extraccin de vrtices de una LWPOLYLINE.

(defun ent-poly (vertices cerrado)
  (entmake
    (append (list '(0 . "LWPOLYLINE")
                  '(100 . "AcDbEntity")
                  '(100 . "AcDbPolyline")
                  (cons 38
                        (if (> (length (car vertices)) 2)
                          (nth 2 (car vertices))
                          (getvar "elevation")
                        ) ;_ fin de if
                  ) ;_ fin de cons
                  (cons 90 (length vertices))
                  (cons 70
                        (if cerrado
                          1
                          0
                        ) ;_ fin de if
                  ) ;_ fin de cons
            ) ;_ fin de list
            (mapcar '(lambda (x) (cons 10 x)) vertices)
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.14. Creacin de polilnea mediante ENTMAKE.

(defun ent-poly-2 (vertices cerrado capa vec-normal)
  (entmake
    (append
      (list '(0 . "LWPOLYLINE")
            '(100 . "AcDbEntity")
            '(100 . "AcDbPolyline")
            (cons 8 capa)
            (cons 38
                  (if (> (length (car vertices)) 2)
                    (nth 2 (car vertices))
                    (getvar "elevation")
                  ) ;_ fin de if
            ) ;_ fin de cons
            (cons 90 (length vertices))
            (cons 70
                  (if cerrado
                    1
                    0
                  ) ;_ fin de if
            ) ;_ fin de cons
            (cons 210 vec-normal)
      ) ;_ fin de list
      (mapcar '(lambda (x) (cons 10 x)) vertices)
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.15. Dibujo de polilnea especificando capa y sistema de coordenadas.

(defun ent-cabecera (capa cerrado)
  (entmake (list '(0 . "POLYLINE")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDb3dPolyline")
                 (cons 8 capa)
                 '(10 0.0 0.0 0.0)
                 (cons 70
                       (+ 8
                          (if cerrado
                            1
                            0
                          ) ;_ fin de if
                       ) ;_ fin de +
                 ) ;_ fin de cons
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.16. Funcin para crear la cabecera de la polilnea 3D.


(defun ent-vertice (xyz capa)
  (entmake (list '(0 . "VERTEX")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbVertex")
                 '(100 . "AcDb3dPolylineVertex")
                 (cons 8 capa)
                 (cons 10 xyz)
                 '(70 . 32)
                 '(50 . 0)
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.17. Funcin para crear la entidad vrtice.


(defun ent-seqend (capa)
  (entmake (list '(0 . "SEQEND")
                 '(100 . "AcDbEntity")
                 (cons 8 capa)
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.18. Funcin para crear la entidad fin-de-secuencia.

(defun ent-3dpol (vertices capa cerrar)
  (ent-cabecera capa cerrar)
  (foreach xyz vertices (ent-vertice xyz capa))
  (ent-seqend capa)
) ;_ fin de defun
;;;Listado 10.19. Funcin que crea una polilnea 3D utilizando entmake.

(defun enames-bloque (id-entidad / tmp)
  (while id-entidad
    (setq tmp        (cons (cdr (assoc -1 (entget id-entidad))) tmp)
          id-entidad (entnext id-entidad)
    ) ;_ fin de setq
  ) ;_ fin de while
  (reverse tmp)
) ;_ fin de defun
;;;Listado 10.20. Obtencin de los componentes de un bloque.

(defun ent-attdef (identificador solicitud valor pt-ins altura
                   visible
                  )
  (entmake (list '(0 . "ATTDEF")
                 '(8 . "0")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbText")
                 (cons 10 pt-ins)
                 (cons 40 altura)
                 (cons 1 valor)
                 '(100 . "AcDbAttributeDefinition")
                 (cons 3 solicitud)
                 (cons 2 identificador)
                 (cons 70
                       (if visible
                         0
                         1
                       ) ;_ fin de if
                 ) ;_ fin de cons
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.21. Entidad ATTDEF creada con ENTMAKE.

(defun ent-block (nombre pto-ins atrib-var)
  (entmake (list '(0 . "BLOCK")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbBlockBegin")
                 '(8 . "0")
                 (cons 2 nombre)
                 (cons 10 pto-ins)
                 (cons 70
                       (if atrib-var
                         2
                         0
                       ) ;_ fin de if
                 ) ;_ fin de cons
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.22. Creacin de la cabecera del bloque.

(defun ent-endblk ()
  (entmake (list '(0 . "ENDBLK")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbBlockEnd")
                 '(8 . "0")
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.23. Creacin de la entidad fin de secuencia.

(defun ent-bloque-torre ()
  (ent-block "TORRE" '(0.0 0.0 0.0) t)
  (ent-poly-2
    '((-0.5 -0.5 0.0)
      (0.5 -0.5 0.0)
      (0.5 0.5 0.0)
      (-0.5 0.5 0.0)
     )
    t
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-poly-2
    '((-0.5 -0.5 0.0) (0.5 0.5 0.0))
    nil
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-poly-2
    '((0.5 -0.5 0.0) (-0.5 0.5 0.0))
    nil
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-attdef
    "ID-TORRE" "Torre nmero" "00" '(0.75 -0.5 0.0) 0.5 nil
   ) ;_ fin de ent-attdef
  (ent-endblk)
) ;_ fin de defun
;;;Listado 10.24. Funcin para la creacin de un bloque.

(defun ed-bloque-torre ()
  (vl-cmdf "_-BEDIT" "TORRE")
  (ent-poly-2
    '((-0.5 -0.5 0.0)
      (0.5 -0.5 0.0)
      (0.5 0.5 0.0)
      (-0.5 0.5 0.0)
     )
    t
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-poly-2
    '((-0.5 -0.5 0.0) (0.5 0.5 0.0))
    nil
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-poly-2
    '((0.5 -0.5 0.0) (-0.5 0.5 0.0))
    nil
    "0"
    '(0.0 0.0 1.0)
  ) ;_ fin de ent-poly-2
  (ent-attdef
    "ID-TORRE" "Torre nmero" "00" '(0.75 -0.5 0.0) 0.5 nil
   ) ;_ fin de ent-attdef
  (vl-cmdf "_BCLOSE" "_Save")
) ;_ fin de defun
;;;Listado 10.25. Creacin del bloque Torre en el Editor de Bloques.

(defun espacio-actual (dibujo /)
  (vla-get-block (vla-get-ActiveLayout dibujo))
) ;_ fin de defun
;;;Listado 10.26. Funcin que obtiene el espacio actual para la creacin de nueva geometra.

(defun 3d->2d (punto)
  (list (car punto) (cadr punto))
) ;_ fin de defun
;;;Listado 10.27. Funcin auxiliar 3d->2d.

(defun ax-poly (vertices cerrado / obj)
  (setq obj
         (vla-AddLightWeightPolyline
           (espacio-actual *aesl:dibujo*)
           (vlax-make-variant
             (ax-lista->matriz
               (apply 'append (mapcar '3d->2d vertices))
             ) ;_ fin de ax-lista->matriz
           ) ;_ fin de vlax-make-variant
         ) ;_ fin de vla-AddLightWeightPolyline
  ) ;_ fin de setq
  (if cerrado
    (vlax-put-property obj 'closed :vlax-true)
  ) ;_ fin de if
  (if (nth 2 (car vertices))
    (vlax-put-property
      obj
      'elevation
      (nth 2 (car vertices))
    ) ;_ fin de vlax-put-property
  ) ;_ fin de if
  obj
) ;_ fin de defun
;;;Listado 10.28. Creacin de una LWPOLYLINE mediante ActiveX.

(defun ax-3dpol (vertices cerrado / obj)
  (setq obj
         (vla-add3dpoly
           (espacio-actual *aesl:dibujo*)
           (vlax-make-variant
             (ax-lista->matriz
               (apply 'append vertices)
             ) ;_ fin de ax-lista->matriz
           ) ;_ fin de vlax-make-variant
         ) ;_ fin de vla-add3dpoly
  ) ;_ fin de setq
  (if cerrado
    (vlax-put-property obj 'closed :vlax-true)
  ) ;_ fin de if
  obj
) ;_ fin de defun
;;;Listado 10.29. Polilnea 3D con mtodos ActiveX.

(defun ax-col-bloques (/)
  (if (null *aesl:bloques*)
    (progn (setq *aesl:bloques* (vla-get-blocks *aesl:dibujo*))
           (pragma '((protect-assign *aesl:bloques*)))
           *aesl:bloques*
    ) ;_ fin de progn
    *aesl:bloques*
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 10.30. Hacer referencia a la coleccin BLOCKS.

(defun ax-bloque-punto-cota (/ tmp atrib)
  (setq tmp (vla-add (ax-col-bloques)
                     (vlax-3d-point '(0.0 0.0 0.0))
                     "PUNTO-COTA"
            ) ;_ fin de vla-add
  ) ;_ fin de setq
  (vla-addline
    tmp
    (vlax-3d-point '(-0.5 0.0 0.0))
    (vlax-3d-point '(0.5 0.0 0.0))
  ) ;_ fin de vla-addline
  (vla-addline
    tmp
    (vlax-3d-point '(0.0 -0.5 0.0))
    (vlax-3d-point '(0.0 0.5 0.0))
  ) ;_ fin de vla-addline
  (setq atrib (vla-addattribute
                tmp
                0.5
                0
                "Cota del punto"
                (vlax-3d-point '(0.0 -1.0 0.0))
                "COTA"
                "0.0"
              ) ;_ fin de vla-addattribute
  ) ;_ fin de setq
  (vlax-put-property
    atrib
    "Alignment"
    acAlignmentTopCenter
  ) ;_ fin de vlax-put-property
  (vlax-put-property
    atrib
    "TextAlignmentPoint"
    (vlax-3d-point '(0.0 -1.0 0.0))
  ) ;_ fin de vlax-put-property
  tmp
) ;_ fin de defun
;;; Listado 10.31. Creacin de un bloque con mtodos ActiveX.

(defun cmd-capa (nombre color tipolinea)
  (vl-cmdf "capa" "e" nombre "co" color nombre "t" tipolinea nombre
           ""
          ) ;_ fin de vl-cmdf
) ;_ fin de defun
;;;Listado 10.32. Funcin que crea una capa empleando opciones de la versin en espaol.

(defun cmd-layer (nombre color tipolinea)
  (vl-cmdf "._layer" "_m" nombre "_c" color nombre "_l" tipolinea
           nombre ""
          ) ;_ fin de vl-cmdf
) ;_ fin de defun
;;;Listado 10.33. Funcin que crea una capa empleando opciones de la versin en ingls.

(defun cmd-cargalinea (nombre)
  (if (not (tblsearch "LTYPE" nombre))
    (vl-cmdf "._linetype"
             "_l"
             nombre
             (if (= (getvar "measurement") 1)
               (findfile "acadiso.lin")
               (findfile "acad.lin")
             ) ;_ fin de if
             ""
    ) ;_ fin de vl-cmdf
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 10.34. Carga de un tipo de lnea.

(defun local-ltyp (tlin / func)
  (if (wcmatch (ver) (strcat "*(en)"))
    (setq func 'car)
    (setq func 'cdr)
  ) ;_ fin de if
  (apply
    func
    (list
      (assoc (strcase tlin)
             '(("MORSE_G" . "BORDER")
               ("MORSE_G2" . "BORDER2")
               ("MORSE_Gx2" . "BORDERX2")
               ("CENTRO" . "CENTER")
               ("CENTRO2" . "CENTER2")
               ("CENTROx2" . "CENTERX2")
               ("TRAZO_Y_PUNTO" . "DASHDOT")
               ("TRAZO_Y_PUNTO2" . "DASHDOT2")
               ("TRAZO_Y_PUNTOX2" . "DASHDOTX2")
               ("TRAZOS" . "DASHED")
               ("TRAZOS2" . "DASHED2")
               ("TRAZOSX2" . "DASHEDX2")
               ("MORSE_D" . "DIVIDE")
               ("MORSE_D2" . "DIVIDE2")
               ("MORSE_DX2" . "DIVIDEX2")
               ("PUNTOS" . "DOT")
               ("PUNTOS2" . "DOT2")
               ("PUNTOSX2" . "DOTX2")
               ("LNEAS_OCULTAS" . "HIDDEN")
               ("LNEAS_OCULTAS2" . "HIDDEN2")
               ("LNEAS_OCULTASX2" . "HIDDENX2")
               ("VALS" . "PHANTOM")
               ("VALS2" . "PHANTOM2")
               ("VALSX2" . "PHANTOMX2")
               ("ACAD_ISO02W100" . "ACAD_ISO02W100")
               ("ACAD_ISO03W100" . "ACAD_ISO03W100")
               ("ACAD_ISO04W100" . "ACAD_ISO04W100")
               ("ACAD_ISO05W100" . "ACAD_ISO05W100")
               ("ACAD_ISO06W100" . "ACAD_ISO06W100")
               ("ACAD_ISO07W100" . "ACAD_ISO07W100")
               ("ACAD_ISO08W100" . "ACAD_ISO08W100")
               ("ACAD_ISO09W100" . "ACAD_ISO09W100")
               ("ACAD_ISO10W100" . "ACAD_ISO10W100")
               ("ACAD_ISO11W100" . "ACAD_ISO11W100")
               ("ACAD_ISO12W100" . "ACAD_ISO12W100")
               ("ACAD_ISO13W100" . "ACAD_ISO13W100")
               ("ACAD_ISO14W100" . "ACAD_ISO14W100")
               ("ACAD_ISO15W100" . "ACAD_ISO15W100")
               ("LMITE1" . "FENCELINE1")
               ("LMITE2" . "FENCELINE2")
               ("VAS" . "TRACKS")
               ("AISLAMIENTO" . "BATTING")
               ("AGUA_CALIENTE" . "HOT_WATER_SUPPLY")
               ("GAS" . "GAS_LINE")
               ("ZIGZAG" . "ZIGZAG")
              )
      ) ;_ fin de assoc
    ) ;_ fin de list
  ) ;_ fin de apply
) ;_ fin de defun
;;;Listado 10.35. Funcin traductora espaol->ingls de los nombres de tipos de lnea.

(defun usar? (capa)
  (zerop
    (logand (cdr (assoc 70 (tblsearch "layer" capa)))
            (+ 1 4)
    ) ;_ fin de logand
  ) ;_ fin de zerop
) ;_ fin de defun
;;;Listado 10.36. Comprueba si una capa no est inutilizada ni bloqueada.

(defun ent-capa (nombre color tipolinea)
  (entmake
    (list
      '(0 . "LAYER")
      '(100 . "AcDbSymbolTableRecord")
      '(100 . "AcDbLayerTableRecord")
      (cons 2 nombre)
      '(70 . 0)
      (cons 62 color)
      (cons 6 tipolinea)
      '(290 . 1)
      '(370 . -3)
    ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.37. Creacin de una capa mediante entmake.

(defun ent-tipolinea
                     (nombre descripcion lista-parametros)
  (entmake
    (append
      (list
        '(0 . "LTYPE")
        '(100 . "AcDbSymbolTableRecord")
        '(100 . "AcDbLinetypeTableRecord")
        (cons 2 nombre)
        '(70 . 0)
        (cons 3 descripcion)
        (cons 72 (ascii (nth 0 lista-parametros)))
        (cons 73 (- (length lista-parametros) 1))
        (cons
          40
          (apply '+
                 (mapcar 'abs (cdr lista-parametros))
          ) ;_ fin de apply
        ) ;_ fin de cons
      ) ;_ fin de list
      (apply 'append
             (mapcar
               '(lambda (x) (list (cons 49 x) '(74 . 0)))
               (cdr lista-parametros)
             ) ;_ fin de mapcar
      ) ;_ fin de apply
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 10.38. Funcin que crea un nuevo tipo de lnea en el dibujo.

(defun ax-capa (col-capas nombre color tipolinea / capa)
  (setq capa (vla-add col-capas nombre))
  (vlax-put-property capa "color" color)
  (vlax-put-property capa "linetype" tipolinea)
  capa
) ;_ fin de defun
;;;Listado 10.39. Creacin de capa con ActiveX.

(defun ax-cargalinea (nombre / tmp)
  (setq nombre (local-ltyp nombre))
  (if
    (not (ax-existe?
           nombre
           (setq tmp (vla-get-Linetypes *aesl:dibujo*))
         ) ;_ fin de ax-existe?
    ) ;_ fin de not
     (progn (vla-load tmp
                      nombre
                      (if (= (getvar "measurement") 1)
                        (findfile "acadiso.lin")
                        (findfile "acad.lin")
                      ) ;_ fin de if
            ) ;_ fin de vla-load
            (vla-put-ActiveLinetype
              *aesl:dibujo*
              (vla-item tmp nombre)
            ) ;_ fin de vla-put-ActiveLinetype
     ) ;_ fin de progn
     (vla-put-ActiveLinetype
       *aesl:dibujo*
       (vla-item tmp nombre)
     ) ;_ fin de vla-put-ActiveLinetype
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 10.40. Carga de un tipo de lnea mediante ActiveX.

(defun ax-define-y-cargalinea (nombre descripcion definicion /
                               archivo id-archivo
                              )
  (setq archivo    (vl-filename-mktemp nil nil ".lin")
        id-archivo (open archivo "w")
  ) ;_ fin de setq
  (write-line
    (strcat "*" nombre "," descripcion)
    id-archivo
  ) ;_ fin de write-line
  (write-line definicion id-archivo)
  (close id-archivo)
  (vla-load (vla-get-linetypes *aesl:dibujo*)
            nombre
            archivo
  ) ;_ fin de vla-load
) ;_ fin de defun
;;;Listado 10.41. Definicin y carga de un tipo de lnea con ActiveX.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
