;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 12.	Modificar entidades.

(defun cambia-color (seleccin ndice)
  (vl-cmdf "._chprop" seleccin "" "_C" ndice "")
) ;_ fin de defun
;;;Listado 12.1. Cambiar el color mediante CAMBPROP.

(defun circulo-en-extremo
       (radio / lineas i linea pin pfi dibujados)
  (if (setq lineas (ssget "X" '((0 . "LINE"))))
    (progn
      (setq i 0)
      (while (setq linea (ssname lineas i))
        (setq pin (valor-cod 10 linea)
              pfi (valor-cod 11 linea)
        ) ;_ fin de setq
        (if (not (member pin dibujados))
          (progn
            (vl-cmdf "._circle" pin radio)
            (setq dibujados (cons pin dibujados))
          ) ;_ fin de progn
        ) ;_ fin de if
        (if (not (member pfi dibujados))
          (progn
            (vl-cmdf "._circle" pfi radio)
            (setq dibujados (cons pfi dibujados))
          ) ;_ fin de progn
        ) ;_ fin de if
        (setq i (1+ i))
      ) ;_ fin de while
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 12.2. Funcin que dibuja crculos en los extremos de las lneas.

(defun recorte-linea-circulo
                             (/ circulos lineas obj extremos)
  (if
    (and (setq circulos (ssget "X" '((0 . "CIRCLE"))))
         (setq lineas (ssget "X" '((0 . "LINE"))))
    ) ;_ fin de and
     (progn (setq i 0)
            (while (setq obj (ssname lineas i))
              (setq extremos (cons (valor-cod 11 obj)
                                   (cons (valor-cod 10 obj)
                                         extremos
                                   ) ;_ fin de cons
                             ) ;_ fin de cons
                    i        (1+ i)
              ) ;_ fin de setq
            ) ;_ fin de while
            (foreach pt extremos
              (vl-cmdf "._trim" circulos "" pt "")
            ) ;_ fin de foreach
            (if (> (getvar "cmdactive") 0)
              (vl-cmdf)
            ) ;_ fin de if
     ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 12.3. Funcin que demuestra el uso de RECORTA desde una funcin VLISP.

(defun C:GRAFO (/ radio)
  (cmd-entrar)
  (vla-StartUndoMark *aesl:dibujo*)
  (setq radio (valor-por-defecto
                'getreal
                "\nIndique radio del crculo "
                10.0
              ) ;_ fin de valor-por-defecto
  ) ;_ fin de setq
  (circulo-en-extremo radio)
  (recorte-linea-circulo)
  (vla-EndUndoMark *aesl:dibujo*)
  (cmd-salir)
  (princ)
) ;_ fin de defun
;;;Listado 12.4. Nuevo comando que automatiza la creacin de diagramas de red.

(defun ent-mod-capa (list-ent capa)
  (entmod
    (subst (cons 8 capa) (assoc 8 list-ent) list-ent)
  ) ;_ fin de entmod
) ;_ fin de defun
;;;Listado 12.5. Funcin que cambia de capa una entidad.

(defun ent-desplaza-obj (list-ent)
  (if
    (apply
      'or
      (mapcar '(lambda (x) (= x (cdr (assoc 0 list-ent))))
              '("CIRCLE" "ELLIPSE" "ARC" "INSERT" "POINT" "SHAPE"
                "TEXT" "MTEXT"
               )
      ) ;_ fin de mapcar
    ) ;_ fin de apply
     (while (setq ctr (getpoint "\nNuevo centro: "))
       (entmod (subst (cons 10 ctr)
                      (assoc 10 list-ent)
                      list-ent
               ) ;_ fin de subst
       ) ;_ fin de entmod
     ) ;_ fin de while
     (prompt "\nObjeto no admitido")
  ) ;_ fin de if
  (princ)
) ;_ fin de defun
;;;Listado 12.6. Funcin que desplaza objetos utilizando ENTMOD.

(defun 3d? (nombre-entidad)
  (and (= (valor-cod 0 nombre-entidad) "POLYLINE")
       (not (zerop (logand (valor-cod 70 nombre-entidad))))
  ) ;_ fin de and
) ;_ fin de defun
;;;Listado 12.7. Predicado que identifica una polilnea 3D.

(defun cierra-polilinea (nombre-entidad / lista)
  (if (wcmatch (valor-cod 0 nombre-entidad) "*POLYLINE")
    (progn
      (setq lista (entget nombre-entidad))
      (entmod
        (subst
          (cons 70
                (logior (valor-cod 70 nombre-entidad) 1)
          ) ;_ fin de cons
          (assoc 70 lista)
          lista
        ) ;_ fin de subst
      ) ;_ fin de entmod
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 12.8. Funcin que cierra cualquier tipo de polilnea.

(defun abre-polilinea (nombre-entidad / lista)
  (if (wcmatch (valor-cod 0 nombre-entidad) "*POLYLINE")
    (progn
      (setq lista (entget nombre-entidad))
      (entmod
        (subst
          (cons
            70
            (logand (valor-cod 70 nombre-entidad) 0)
          ) ;_ fin de cons
          (assoc 70 lista)
          lista
        ) ;_ fin de subst
      ) ;_ fin de entmod
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 12.9. Funcin que abre polilneas utilizando LOGAND.

(defun cerrar-poly (nombre-entidad cerrar / lista)
  (if (wcmatch (valor-cod 0 nombre-entidad) "*POLYLINE")
    (progn
      (setq lista (entget nombre-entidad))
      (entmod
        (subst
          (cons 70
                (boole (if cerrar
                         7
                         1
                       ) ;_ fin de if
                       (valor-cod 70 nombre-entidad)
                       (if cerrar
                         1
                         (~ 1)
                       ) ;_ fin de if
                ) ;_ fin de boole
          ) ;_ fin de cons
          (assoc 70 lista)
          lista
        ) ;_ fin de subst
      ) ;_ fin de entmod
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 12.10. Funcin para cerrar o abrir polilneas usando BOOLE.

(defun ax-link (obj nombre url descripcion ubicacion /)
  (vl-load-com)
  (setq links (vla-get-Hyperlinks obj)
        link  (vla-Add links nombre)
  ) ;_ fin de setq
  (vla-put-URL link url)
  (vla-put-URLDescription link descripcion)
  (vla-put-URLNamedLocation link ubicacion)
) ;_ fin de defun
;;;Listado 12.11. Funcin que aade un hipervnculo a una entidad.

(defun RGB->TrueColor (rgb)
  (apply '+
         (mapcar '(lambda (c i) (lsh c i))
                 rgb
                 '(16 8 0)
         ) ;_ fin de mapcar
  ) ;_ fin de apply
) ;_ fin de defun
;;;Listado 12.12. Funcin que convierte los valores RGB en TrueColor.

(defun TrueColor->RGB (tcol)
  (mapcar '(lambda (i) (lsh (lsh tcol i) -24))
          '(8 16 24)
  ) ;_ fin de mapcar
) ;_ fin de defun
;;;Listado 12.13. Funcin para convertir el color TrueColor en valores RGB.

(defun escala-color (rgb-ini rgb-fin cantidad prefijo / inc-rgb
                     lista-colores n
                    )
  (setq inc-rgb (mapcar
                  '(lambda (i)
                     (/ (- (nth i rgb-ini) (nth i rgb-fin))
                        cantidad
                     ) ;_ fin de /
                   ) ;_ fin de lambda
                  (list 0 1 2)
                ) ;_ fin de mapcar
        n       0
  ) ;_ fin de setq
  (repeat (1- cantidad)
    (setq lista-colores
           (cons
             (mapcar
               '-
               rgb-ini
               (mapcar '(lambda (i) (* i n))
                       inc-rgb
               ) ;_ fin de mapcar
             ) ;_ fin de mapcar
             lista-colores
           ) ;_ fin de cons
          n (1+ n)
    ) ;_ fin de setq
  ) ;_ fin de repeat
  (reverse (cons rgb-fin lista-colores))
) ;_ fin de defun
;;;Listado 12.14. Funcin que calcula los valores RGB para la escala de colores.


(defun crea-colores
       (lista-colores prefijo / apl n objcolor objetos-color)
  (setq apl (vlax-get-acad-object)
        n   1
  ) ;_ fin de setq
  (foreach rgb lista-colores
    (setq objcolor
           (vla-GetInterfaceObject apl "AutoCAD.AcCmColor.18")
    ) ;_ fin de setq
    (vla-SetNames
      objcolor
      (strcat prefijo "_" (itoa n))
      (strcat "Escala_" prefijo)
    ) ;_ fin de vla-SetNames
    (apply 'vla-SetRGB (cons objcolor rgb))
    (setq objetos-color
           (cons objcolor objetos-color)
          n (1+ n)
    ) ;_ fin de setq
  ) ;_ fin de foreach
  (reverse objetos-color)
) ;_ fin de defun
;;;Listado 12.15. Funcin que crea los objetos AcCmColor de la escala.

(defun datos-escala
                    (/ tcl-ini tcl-fin pasos prefijo ops sel)
  (prompt "\nDatos para la escala de colores.")
  (while (or (not tcl-ini)
             (not tcl-fin)
             (not pasos)
             (not prefijo)
         ) ;_ fin de or
    (setq ops ""
          ops (if (not tcl-ini)
                (strcat ops "colorInicial ")
                ops
              ) ;_ fin de if
          ops (if (not tcl-fin)
                (strcat ops "colorFinal ")
                ops
              ) ;_ fin de if
          ops (if (not pasos)
                (strcat ops "Pasos ")
                ops
              ) ;_ fin de if
          ops (if (not prefijo)
                (strcat ops "Nombre")
                ops
              ) ;_ fin de if
    ) ;_ fin de setq
    (if (not (setq msj (sustituye "/" " " ops)))
      (setq msj ops)
    ) ;_ fin de if
    (apply 'initget (list 1 ops))
    (setq sel
           (getkword (strcat "\nSeleccione opcin ["
                             msj
                             "]: "
                     ) ;_ fin de strcat
           ) ;_ fin de getkword
    ) ;_ fin de setq
    (cond
      ((= sel "colorInicial")
       (while (not (setq tcl-ini
                          (assoc 420
                                 (acad_truecolordlg
                                   '(420 . 16391690)
                                   nil
                                 ) ;_ fin de acad_truecolordlg
                          ) ;_ fin de assoc
                   ) ;_ fin de setq
              ) ;_ fin de not
         (alert
           "Debe seleccionar \nen la ficha Color Verdadero."
         ) ;_ fin de alert
       ) ;_ fin de while
      )
      ((= sel "colorFinal")
       (while (not (setq tcl-fin
                          (assoc 420
                                 (acad_truecolordlg
                                   '(420 . 410356)
                                   nil
                                 ) ;_ fin de acad_truecolordlg
                          ) ;_ fin de assoc
                   ) ;_ fin de setq
              ) ;_ fin de not
         (alert
           "Debe seleccionar \nen la ficha Color Verdadero."
         ) ;_ fin de alert
       ) ;_ fin de while
      )
      ((= sel "Pasos")
       (initget (+ 1 2 4))
       (setq pasos
              (getint
                "\nNmero de colores en la escala: "
              ) ;_ fin de getint
       ) ;_ fin de setq
      )
      ((= sel "Nombre")
       (setq prefijo
              (getstring
                "\nPrefijo para nombrar los colores: "
              ) ;_ fin de getstring
       ) ;_ fin de setq
      )
    ) ;_ fin de cond
  ) ;_ fin de while
  (list (TrueColor->RGB (cdr tcl-ini))
        (TrueColor->RGB (cdr tcl-fin))
        pasos
        prefijo
  ) ;_ fin de list
) ;_ fin de defun
;;;Listado 12.16. Entrada de datos para la creacin de escalas de color.

(defun sel-sombreados (/ lis-somb sel-somb)
  (setq sel-somb (ssget "X" '((0 . "HATCH")))
        i        0
  ) ;_ fin de setq
  (while (setq ent (ssname sel-somb i))
    (setq lis-somb (cons (vlax-ename->vla-object ent) lis-somb)
          i        (1+ i)
    ) ;_ fin de setq
  ) ;_ fin de while
  lis-somb
) ;_ fin de defun
;;;Listado 12.17. Funcin para la obtener los objetos HATCH.

(defun colorea (lis-somb pasos colores / i indice indices obj area
                areas objetos
               )
  (setq i 0)
  (while (setq obj (nth i lis-somb))
    (setq area (vl-catch-all-apply 'vla-get-Area (list obj)))
    (if (not (vl-catch-all-error-p area))
      (setq areas   (cons area areas)
            objetos (cons obj objetos)
      ) ;_ fin de setq
    ) ;_ fin de if
    (setq i (1+ i))
  ) ;_ fin de while
  (setq d-area (/ (- (apply 'max areas) (apply 'min areas))
                  (1- pasos)
               ) ;_ fin de /
  ) ;_ fin de setq
  (setq indices (mapcar '(lambda (a) (read (rtos (/ a d-area) 2 0)))
                        areas
                ) ;_ fin de mapcar
        i       0
  ) ;_ fin de setq
  (while (setq obj (nth i objetos))
    (setq indice (nth i indices))
    (cond
      ((< indice 0) (setq indice 0))
      ((>= indice (length colores))
       (setq indice (1- (length colores)))
      )
    ) ;_ fin de cond
    (vla-put-TrueColor obj (nth indice colores))
    (setq i (1+ i))
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 12.18. Funcin empleada para aplicar el color a las reas sombreadas.

(defun C:COLOR-AREA (/ datos escala colores sombreados)
  (if (setq sombreados (sel-sombreados))
    (progn (setq datos (datos-escala))
           (setq escala (apply 'escala-color datos))
           (setq colores (crea-colores escala (last datos)))
           (colorea sombreados (nth 2 datos) colores)
    ) ;_ fin de progn
    (prompt "\nNo se han encontrado sombreados")
  ) ;_ fin de if
  (princ)
) ;_ fin de defun
;;;Listado 12.19. Funcin principal que aplica sombreados TrueColor a objetos HATCH.

(defun ax-matriz-rectangular (ename num-fila num-col num-nivel
                              dist-fila dist-col dist-nivel
                             )
  (vla-ArrayRectangular
    (vlax-ename->vla-object ename)
    num-fila
    num-col
    num-nivel
    dist-fila
    dist-col
    dist-nivel
  ) ;_ fin de vla-ArrayRectangular
) ;_ fin de defun
;;;Listado 12.20. Creacin de una matriz rectangular con ActiveX.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
