;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 13.	Objetos 3D.-------------------------------------------------------------

(defun ent-plin  (pt-i pt-f norm /)
  (entmake (list '(0 . "LWPOLYLINE")       ;Tipo de entidad
                 '(100 . "AcDbEntity")     ;Subclase
                 '(100 . "AcDbPolyline")   ;Subclase
                 '(90 . 2)                 ;Nmero de vrtices
                 (cons 10 pt-i)            ;Coordenadas vrtice 1
                 '(91 . 1)                 ;Id del vrtice
                 (cons 10 pt-f)            ;Coordenadas vrtice 2
                 '(91 . 2)                 ;Id del vrtice
                 (cons 210 norm))))        ;Vector normal
;;;Listado 13.1. Funcin que dibuja una polilnea en el plano frontal.

(defun ent-lin  (pt-i pt-f norm /)
  (entmake (list '(0 . "LINE")                ;Tipo entidad
                 '(100 . "AcDbEntity")        ;Subclase
                 '(100 . "AcDbLine")          ;Subclase
                 (cons 10 (trans pt-i 0 norm));Vrtice 1
                 (cons 11 (trans pt-f 0 norm));Vrtice 2
                 (cons 210 norm))))           ;Vector normal
;;;Listado 13.2 Funcin de prueba que crea una lnea especificando el vector normal.

;;; Vector A->B
;;; Argumentos: A, B, listas de tres nmeros reales. 
(defun vec (A B) (mapcar '- B A))

;;; Suma de vectores
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun v+v (v1 v2) (mapcar '+ v1 v2))

;;; Producto ESCALAR (dot product)
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun x-esc (v1 v2) (apply '+ (mapcar '* v1 v2)))

;;; Longitud del vector (mdulo)
;;; Argumento: v, lista de tres nmeros reales.
(defun m-vec (v) (sqrt (apply '+ (mapcar '* v v))))

;;; Vector unitario
;;; Argumento: v, lista de tres nmeros reales.
(defun v-unit (v / m)
  (cond ((zerop (setq m (m-vec v))) nil)
        (t (mapcar '(lambda (n) (/ n m)) v))
  ) ;_ fin de cond
) ;_ fin de defun

;;; Producto VECTORIAL (cross product)
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun prod-vec (v1 v2)
  (list (- (* (cadr v1) (caddr v2)) (* (cadr v2) (caddr v1)))
        (- (* (car v2) (caddr v1)) (* (car v1) (caddr v2)))
        (- (* (car v1) (cadr v2)) (* (car v2) (cadr v1)))
  ) ;_ fin de list
) ;_ fin de defun
;;;Listado 13.3. Operaciones con vectores.

(defun ax-traslacion (obj vector)
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list 1.0 0.0 0.0 (nth 0 vector))
            (list 0.0 1.0 0.0 (nth 1 vector))
            (list 0.0 0.0 1.0 (nth 2 vector))
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.4. Funcin para desplazamiento.

;;; Grados a Radianes
(defun gar (g) (/ (* g pi) 180.0))
;;; Radianes a Grados
(defun rag (r) (* (/ r pi) 180.0))
;;;Listado 13.5. Conversin entre Grados y Radianes.

(defun ax-rot-x (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list 1.0 0.0 0.0 0.0)
            (list 0.0 (cos a) (sin a) 0.0)
            (list 0.0 (- (sin a)) (cos a) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.6. Rotacin en torno a X.

(defun ax-rot-y (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos a) 0.0 (sin a) 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list (- (sin a)) 0.0 (cos a) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.7. Rotacin en torno a Y.

(defun ax-rot-z (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos a) (- (sin a)) 0.0 0.0)
            (list (sin a) (cos a) 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.8. Rotacin en torno a Z.

(defun ax-escala (obj vector / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (list (list (nth 0 vector) 0.0 0.0 0.0)
                            (list 0.0 (nth 1 vector) 0.0 0.0)
                            (list 0.0 0.0 (nth 2 vector) 0.0)
                            (list 0.0 0.0 0.0 1.0)
                      ) ;_ fin de list
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.9. Funcin para transformacin de escala en XYZ.

(defun ax-ciz-x (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (list (list 1.0 factor 0.0 0.0)
                            (list 0.0 1.0 0.0 0.0)
                            (list 0.0 0.0 1.0 0.0)
                            (list 0.0 0.0 0.0 1.0)
                      ) ;_ fin de list
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.10. Cizalladura en X.

(defun ax-ciz-y (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                   (vlax-tmatrix
                     (list (list 1.0 0.0 0.0 0.0)
                           (list factor 1.0 0.0 0.0)
                           (list 0.0 0.0 1.0 0.0)
                           (list 0.0 0.0 0.0 1.0)
                        ) ;_ fin de list
                      ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.11. Cizalladura en Y.

(defun ax-ciz-z (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                   (vlax-tmatrix
                     (list (list 1.0 0.0 0.0 0.0)
                           (list 0.0 1.0 0.0 0.0)
                           (list factor factor 1.0 0.0)
                           (list 0.0 0.0 0.0 1.0)
                        ) ;_ fin de list
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.12. Cizalladura en Z.

(defun C:TRANSFORMA
                    (/ obj base factor-x factor-y factor-z)
  (if (setq obj
             (vlax-ename->vla-object
               (car
                 (entsel
                   "\nDesigne objeto a transformar: "
                 ) ;_ fin de entsel
               ) ;_ fin de car
             ) ;_ fin de vlax-ename->vla-object
      ) ;_ fin de setq
    (progn
      (setq base
             (getpoint
               "\nPunto base para la transformacin: "
             ) ;_ fin de getpoint
      ) ;_ fin de setq
      (initget (+ 1 2))
      (setq factor-x
             (getreal "\nFactor de escala X: ")
      ) ;_ fin de setq
      (initget (+ 1 2))
      (setq factor-y
             (getreal "\nFactor de escala Y: ")
      ) ;_ fin de setq
      (initget (+ 1 2))
      (setq factor-z
             (getreal "\nFactor de escala Z: ")
      ) ;_ fin de setq
      (ax-traslacion obj (mapcar '- base))
      (ax-escala obj
                 (list factor-x factor-y factor-z)
      ) ;_ fin de ax-escala
      (ax-traslacion obj base)
    ) ;_ fin de progn
    (prompt "\nNingn objeto designado.")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.13. Comando para transformar con distintas escalas en XYZ.

(defun ax-scp (nombre origen dirx diry / tmp)
  (setq
    tmp (vla-Add (vla-get-UserCoordinateSystems *aesl:dibujo*)
                 (vlax-3d-point '(0 0 0))
                 (vlax-3d-point dirx)
                 (vlax-3d-point diry)
                 nombre
        ) ;_ fin de vla-Add
  ) ;_ fin de setq
  (vla-put-Origin tmp (vlax-3d-point origen))
  tmp
) ;_ fin de defun
;;;Listado 13.14. Funcin que aade un nuevo SCP al documento actual.

(defun ax-matriz-scp (/ nombre num-scp nuevo-scp)
  (setq nombre (getvar "UCSNAME"))
  (cond
    ((or (equal nombre "")
         (and (vl-string-search "*" nombre 0)
              (vl-string-search "*" nombre (1- (strlen nombre)))
         ) ;_ fin de and
     ) ;_ fin de or
     (setq num-scp (vla-get-Count
                     (vla-get-UserCoordinateSystems *aesl:dibujo*)
                   ) ;_ fin de vla-get-Count
           nombre  (strcat "SCP_" (itoa num-scp))
     ) ;_ fin de setq
     (setq nuevo-scp (ax-scp nombre
                             (getvar "UCSORG")
                             (getvar "UCSXDIR")
                             (getvar "UCSYDIR")
                     ) ;_ fin de ax-scp
     ) ;_ fin de setq
     (vla-put-ActiveUCS *aesl:dibujo* nuevo-scp)
     (list nombre (vla-GetUCSMatrix nuevo-scp))
    )
    (t
     (list (vla-get-name (vla-get-ActiveUCS *aesl:dibujo*))
           (vla-GetUCSMatrix (vla-get-ActiveUCS *aesl:dibujo*))
     ) ;_ fin de list
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 13.15. Funcin que devuelve la Matriz de Transformacin del SCP actual.

(defun var-vis ()
  (if (= (getvar "BLOCKEDITOR") 0)
    (PROGN (setvar "VSMONOCOLOR" " RGB:211,76,3")
           (setvar "VSFACECOLORMODE" 1)
           (setvar "VSEDGES" 1)
           (setvar "VSEDGECOLOR" "RGB:255,212,82")
           (setvar "VSISOONTOP" 0)
           (setvar "VSFACESTYLE" 2)
           (setvar "VSOBSCUREDEDGES" 0)
           (setvar "VSOCCLUDEDEDGES" 0)
           (setvar "PERSPECTIVE" 1)
    ) ;_ fin de PROGN
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.16. Funcin que establece un estilo visual personalizado.

(defun ax-vista (direccion zoom / vport)
  (setq vport (vla-get-ActiveViewport *aesl:dibujo*))
  (vla-put-Direction vport (vlax-3d-point direccion))
  (vla-put-ActiveViewport *aesl:dibujo* vport)
  (vlax-release-object vport)
  (if zoom
    (vla-ZoomExtents *aesl:acad*)
  ) ;_ fin de if
  (var-vis)
  (princ)
) ;_ fin de defun
;;;Listado 13.17. Funcin que establece la direccin de la vista y el estilo visual.

(defun ax-sup ()
  (ax-vista '(0 0 1) t)
) ;_ fin de defun

(defun ax-der ()
  (ax-vista '(1 0 0) t)
) ;_ fin de defun

(defun ax-fre ()
  (ax-vista '(0 -1 0) t)
) ;_ fin de defun

(defun ax-NEs ()
  (ax-vista '(1 1 1) t)
) ;_ fin de defun

(defun ax-NWs ()
  (ax-vista '(-1 1 1) t)
) ;_ fin de defun

(defun ax-SWs ()
  (ax-vista '(-1 -1 1) t)
) ;_ fin de defun

(defun ax-SEs ()
  (ax-vista '(1 -1 1) t)
) ;_ fin de defun

(defun ax-inf ()
  (ax-vista '(0 0 -1) t)
) ;_ fin de defun

(defun ax-izq ()
  (ax-vista '(-1 0 0) t)
) ;_ fin de defun

(defun ax-tra ()
  (ax-vista '(0 1 0) t)
) ;_ fin de defun

(defun ax-NEi ()
  (ax-vista '(1 1 -1) t)
) ;_ fin de defun

(defun ax-NWi ()
  (ax-vista '(-1 1 -1) t)
) ;_ fin de defun

(defun ax-SWi ()
  (ax-vista '(-1 -1 -1) t)
) ;_ fin de defun

(defun ax-SEi ()
  (ax-vista '(1 -1 -1) t)
) ;_ fin de defun

;;;Listado 13.18. Funciones para ajustar la vista isomtrica 3D.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
