;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 14.	Curvas NURBS: La entidad Spline.------------------------------

(defun ax-spline (lista-puntos / tg-ini tg-fin 
                  puntos arr-pts
                 )
  (setq tg-ini  (vlax-3d-point '(0 0 0))
        tg-fin  (vlax-3d-point '(0 0 0))
        puntos  (apply 'append lista-puntos)
        arr-pts (vlax-make-safearray
                  vlax-vbDouble
                  (cons 0 (1- (length puntos)))
                ) ;_ fin de vlax-make-safearray
        arr-pts (vlax-safearray-fill arr-pts puntos)
  ) ;_ fin de setq
  (vla-AddSpline
    (espacio-actual *aesl:dibujo*)
    arr-pts
    tg-ini
    tg-fin
  ) ;_ fin de vla-AddSpline
) ;_ fin de defun
;;;Listado 14.1. Funcin que crea una entidad SPLINE aplicando el mtodo AddSpline.

(defun ent-spline-FP (lista-puntos)
  (entmake
    (append (list '(0 . "SPLINE")
                  '(100 . "AcDbEntity")
                  '(100 . "AcDbSpline")
                  '(71 . 3)
                  (cons 74 (length lista-puntos))
            ) ;_ fin de list
            (mapcar '(lambda (x) (cons 11 x)) lista-puntos)
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 14.2. Funcin que crea una Spline por Puntos de Ajuste.

(defun vector-nudos
                    (vc grado sujeta / numero centro vector i)
  (setq numero (+ grado vc 1)
        grado  (if sujeta
                 (1+ grado)
                 grado
               ) ;_ fin de if
        centro (- numero (* grado 2))
        i      0
  ) ;_ fin de setq
  (repeat grado (setq vector (cons 0 vector)))
  (repeat centro
    (setq i (1+ i))
    (setq vector (cons i vector))
  ) ;_ fin de repeat
  (repeat grado (setq vector (cons (1+ i) vector)))
  (reverse vector)
) ;_ fin de defun
;;;Listado 14.3. Funcin que crea el vector de nudos.

(defun ent-spline-VC
                     (lista-vertices grado sujeta / vc k-vec)
  (setq vc    (length lista-vertices)
        k-vec (vector-nudos vc grado sujeta)
  ) ;_ fin de setq
  (entmake
    (append (list '(0 . "SPLINE")
                  '(100 . "AcDbEntity")
                  '(100 . "AcDbSpline")
                  (cons 71 grado)
                  (cons 72 (length k-vec))
                  (cons 73 vc)
                  '(74 . 0)
                  '(42 . 0.0000001)
                  '(43 . 0.0000001)
            ) ;_ fin de list
            (mapcar '(lambda (k) (cons 40 k)) k-vec)
            (mapcar '(lambda (x) (cons 10 x)) lista-vertices)
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 14.4. Funcin que crea una Spline por el mtodo de vrtices de control.

(defun vc-helice (centro radio-base radio-sup altura resolucion
                  giros torsion / paso n-ver ang incang incrad val-z rad-v
                  f-rad vertice vertices
                 )
  (setq paso   (/ (float altura) (* giros resolucion))
        n-ver  (* giros resolucion)
        incang (/ pi (/ resolucion 2.0))
        incrad (/ (float (- radio-sup radio-base))
                  (* giros resolucion)
               ) ;_ fin de /
        ang    (- 0.0 incang)
        val-z  (- (nth 2 centro) (/ paso 2))
        f-rad  (cos (/ pi resolucion))
        rad-v  (- (/ radio-base f-rad) incrad)
  ) ;_ fin de setq
  (repeat (+ n-ver 2)
    (setq vertice  (polar centro ang rad-v)
          vertice  (list (nth 0 vertice) (nth 1 vertice) val-z)
          vertices (cons vertice vertices)
          ang      (if (= torsion 1)
                     (+ ang incang)
                     (- ang incang)
                   ) ;_ fin de if
          rad-v    (+ rad-v incrad)
          val-z    (+ val-z paso)
    ) ;_ fin de setq
  ) ;_ fin de repeat
  (reverse vertices)
) ;_ fin de defun
;;;Listado 14.5. Clculo de los vrtices de control para la hlice.

(defun ent-helix-VC (lista-puntos grado sujeta centro-base
                     radio-base radio-sup giros paso torsion / vc
                     k-vec
                    )
  (setq vc    (length lista-puntos)
        k-vec (vector-nudos vc grado sujeta)
  ) ;_ fin de setq
  (entmake
    (append (list '(0 . "HELIX")
                  '(100 . "AcDbEntity")
                  '(100 . "AcDbSpline")
                  (cons 71 grado)
                  (cons 72 (length k-vec))
                  (cons 73 vc)
                  '(74 . 0)
                  '(42 . 0.0000001)
                  '(43 . 0.0000001)
            ) ;_ fin de list
            (mapcar '(lambda (k) (cons 40 k)) k-vec)
            (mapcar '(lambda (x) (cons 10 x)) lista-puntos)
            (list '(100 . "AcDbHelix")
                  (cons 10 centro-base)
                  (cons 11 (polar centro-base 0.0 radio-base))
                  '(12 0.0 0.0 1.0)
                  (cons 40 radio-sup)
                  (cons 41 giros)
                  (cons 42 paso)
                  (cons 290 torsion)
                  (cons 280 1)
            ) ;_ fin de list
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 14.6. Funcin que crea una entidad HELIX mediante entmake.

(defun datos-spline-helice (/)
  (initget 1)
  (setq centro-base
         (getpoint "\nPrecise centro de base: ")
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq radio-base
         (getdist centro-base
                  "\nPrecise radio de la base: "
         ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq radio-sup (getdist centro-base
                           "\nPrecise radio superior: "
                  ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq
    altura (getdist centro-base
                    "\nPrecise altura de la hlice: "
           ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq giros (getint "\nIndique nmero de giros: "))
  (initget "Horaria Antihoraria")
  (setq torsion
         (getkword
           "\nIndique direccin de torsin de hlice [Horaria/Antihoraria] <Antihoraria>:"
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (if (or (null torsion) (= torsion "Antihoraria"))
    (setq torsion 1)
    (setq torsion 0)
  ) ;_ fin de if
  (initget (+ 1 2 4))
  (setq resolucion
         (getint
           "\nNmero de vrtices para cada vuelta: "
         ) ;_ fin de getint
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 14.7. Solicitud de datos para la hlice.

(defun C:ENT-HELICE (/ centro-base radio-base radio-sup altura
                     resolucion giros torsion lista-vertices
                    )
  (datos-spline-helice)
  (setq lista-vertices
         (vc-helice centro-base radio-base radio-sup altura
                    resolucion giros torsion
                   ) ;_ fin de vc-helice
  ) ;_ fin de setq
  (ent-helix-vc
    lista-vertices 3 nil centro-base radio-base radio-sup giros paso
    torsion
   ) ;_ fin de ent-helix-vc
) ;_ fin de defun
;;;Listado 14.8. Funcin principal C:ENT-HELICE.

(defun helix->spline (ename /)
  (cond
    ((entmake
       (subst
         '(0 . "SPLINE")
         '(0 . "HELIX")
         (reverse (cdr (member '(100 . "AcDbHelix")
                               (reverse (entget ename))
                       ) ;_ fin de member
                  ) ;_ fin de cdr
         ) ;_ fin de reverse
       ) ;_ fin de subst
     ) ;_ fin de entmake
     (entdel ename)
     (entlast)
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 14.9. Conversin de una entidad HELIX en una SPLINE.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
