;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 15.	VLAX-CURVE... medir curvas y algo ms.---------------------------------

(defun ax-longtotal (objcurva)
  (vlax-curve-GetDistAtParam
    objcurva
    (vlax-curve-GetEndParam objcurva)
  ) ;_ fin de vlax-curve-GetDistAtParam
) ;_ fin de defun
;;;Listado 15.1. Determinacin de la longitud total de una curva.

(defun C:LEE-LONGITUD (/ obj)
  (if
    (setq obj (car (entsel "\nDesigne Curva a medir: ")))
     (alert
       (strcat
         "El objeto "
         (cdr (assoc 0 (entget obj)))
         " mide\n"
         (vl-princ-to-string
           (ax-longtotal (vlax-ename->vla-object obj))
         ) ;_ fin de vl-princ-to-string
         " unidades de longitud."
       ) ;_ fin de strcat
     ) ;_ fin de alert
     (alert "Ningn objeto designado")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.2. Funcin de prueba para AX-LONGTOTAL.

(defun ax-dist-punto (objcurva punto / parametro)
  (if (setq parametro (vlax-curve-GetParamAtPoint
                        objcurva
                        (trans punto 1 0)
                      ) ;_ fin de vlax-curve-GetParamAtPoint
      ) ;_ fin de setq
    (vlax-curve-GetDistAtParam objcurva parametro)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.3. Longitud desde el inicio a un punto cualquiera.

(defun C:DIST-A-PUNTO (/ obj pt dist)
  (prompt "\nDesigne Curva a medir: ")
  (if (and (setq obj (car (entsel)))
           (setq pt (getpoint "\nSeleccione Punto: "))
           (setq dist (ax-dist-punto
                        (vlax-ename->vla-object obj)
                        pt
                      ) ;_ fin de ax-dist-punto
           ) ;_ fin de setq
      ) ;_ fin de and
    (alert (strcat "\nLa distancia desde el inicio"
                   "\nhasta el punto designado es\n"
                   (rtos dist 2 2)
           ) ;_ fin de strcat
    ) ;_ fin de alert
    (alert
      (if (null obj)
        "Ningn objeto designado"
        "El punto no se encuentra sobre el objeto"
      ) ;_ fin de if
    ) ;_ fin de alert
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.4. Medicin desde el inicio de la curva a un punto.

(defun ax-dist-entrepuntos (objcurva pto1 pto2 / dist1 dist2)
  (if (and (setq dist1 (ax-dist-punto objcurva (trans pto1 1 0)))
           (setq dist2 (ax-dist-punto objcurva (trans pto2 1 0)))
      ) ;_ fin de and
    (abs (- dist1 dist2))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.5. Determinar la distancia entre dos puntos de la curva.

(defun C:DIST-ENTRE-PUNTOS (/ obj pt1 pt2 dist)
  (prompt "\nDesigne Curva a medir: ")
  (if
    (and (setq obj (car (entsel)))
         (setq pt1 (getpoint "\nSeleccione Punto 1: "))
         (setq pt2 (getpoint "\nSeleccione Punto 2: "))
         (setq dist (ax-dist-entrepuntos
                      (vlax-ename->vla-object obj)
                      pt1
                      pt2
                    ) ;_ fin de ax-dist-entrepuntos
         ) ;_ fin de setq
    ) ;_ fin de and
     (alert
       (strcat "\nLa distancia entre los puntos designados"
               "\nes igual a \n"
               (rtos dist 2 2)
       ) ;_ fin de strcat
     ) ;_ fin de alert
     (alert
       (if (null obj)
         "Ningn objeto designado"
         "El punto no se encuentra sobre el objeto"
       ) ;_ fin de if
     ) ;_ fin de alert
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.6. Medicin de la distancia entre dos puntos de una curva.

(defun C:AREA-CURVA (/ obj)
  (while (setq obj (car (entsel)))
    (setq obj (vlax-ename->vla-object obj))
    (cond
      ((not (vlax-property-available-p obj "Area"))
       (alert
         (strcat "El objeto "
                 (vlax-get-property obj "ObjectName")
                 " seleccionado\nNO tiene rea."
         ) ;_ fin de strcat
       ) ;_ fin de alert
      )
      ((not (vlax-curve-isPlanar obj))
       (alert
         (strcat "El objeto "
                 (vlax-get-property obj "ObjectName")
                 " seleccionado\nNO es plano."
         ) ;_ fin de strcat
       ) ;_ fin de alert
      )
      ((not (vlax-curve-isClosed obj))
       (alert
         (strcat "El objeto "
                 (vlax-get-property obj "ObjectName")
                 " seleccionado\nNO est cerrado."
         ) ;_ fin de strcat
       ) ;_ fin de alert
      )
      (t
       (alert
         (strcat "El objeto "
                 (vlax-get-property obj "ObjectName")
                 " seleccionado\ntiene un rea de "
                 (rtos (vlax-curve-GetArea obj))
                 " unidades."
         ) ;_ fin de strcat
       ) ;_ fin de alert
      )
    ) ;_ fin de cond
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 15.7. Determinacin del rea encerrada por la curva.

(defun ent-rayo (punto vector)
  (entmake (list '(0 . "RAY")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbRay")
                 (cons 10 punto)
                 (cons 11 vector)
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 15.8. Funcin para dibujar un RAYO.

(defun calc-tangente (curva punto / ax-curva dir punto param)
  (setq dir   (trans (getvar "viewdir") 1 0 t)
        punto (trans punto 1 0)
  ) ;_ fin de setq
  (if (and (setq ax-curva (vlax-ename->vla-object curva))
           (setq punto (vl-catch-all-apply
                         'vlax-curve-GetClosestPointToProjection
                         (list ax-curva punto dir)
                       ) ;_ fin de vl-catch-all-apply
           ) ;_ fin de setq
      ) ;_ fin de and
    (if (vl-catch-all-error-p punto)
      nil
      (progn
        (setq param
               (vlax-curve-GetParamAtPoint ax-curva punto)
        ) ;_ fin de setq
        (list punto (vlax-curve-GetFirstDeriv ax-curva param))
      ) ;_ fin de progn
    ) ;_ fin de if
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.9. Funcin que calcula la tangente a una curva.

(defun C:TANGENTE (/ seleccion datos)
  (cmd-entrar)
  (if (setq seleccion
             (entsel
               "\nDesigne punto de tangencia: "
             ) ;_ fin de entsel
      ) ;_ fin de setq
    (if (setq
          datos (calc-tangente
                  (car seleccion)
                  (cadr seleccion)
                ) ;_ fin de calc-tangente
        ) ;_ fin de setq
      (ent-rayo (car datos) (cadr datos))
      (prompt "\nError en la operacin.")
    ) ;_ fin de if
  ) ;_ fin de if
  (cmd-salir)
  (princ)
) ;_ fin de defun
;;;Listado 15.10. Comando para dibujar un RAYO tangente a una curva.

(defun cmd-vectorz (punto vector / )
  (setvar "CMDECHO" 0)
  (vl-cmdf
    "._ucs"
    "_zaxis"
    "_none"
    (trans punto 0 1)
    "_none"
    (trans (mapcar '+ punto vector) 0 1)
  ) ;_ fin de vl-cmdf
  (setvar "CMDECHO" 1)
) ;_ fin de defun
;;;Listado 15.11. Funcin para establecer un sistema de coordenadas a partir de un punto y un vector.

(defun ax-scp-normal (origen vec-z / ang vec-x vec-y)
  (setq ang   (- (angle '(0 0 0) vec-z) (/ pi 2))
        vec-X (vec '(0 0 0) (polar '(0 0 0) ang 1.0))
        vec-Y (prod-vec vec-z vec-x)
  ) ;_ fin de setq
  (vla-put-ActiveUCS
    *aesl:dibujo*
    (ax-scp "TMP" origen vec-x vec-y)
  ) ;_ fin de vla-put-ActiveUCS
) ;_ fin de defun
;;;Listado 15.12. SCP mediante ActiveX a partir del vector de direccin del eje Z.

(defun C:SCP-NORMAL (/ seleccin datos)
  (if (setq seleccin
             (entsel
               "\nDesigne origen sobre una entidad: "
             ) ;_ fin de entsel
      ) ;_ fin de setq
    (if (setq
          datos (calc-tangente
                  (car seleccin)
                  (cadr seleccin)
                ) ;_ fin de calc-tangente
        ) ;_ fin de setq
      (ax-scp-normal (car datos) (cadr datos))
      (prompt "\nError en la operacin.")
    ) ;_ fin de if
  ) ;_ fin de if
  (princ)
) ;_ fin de defun
;;;Listado 15.13. Comando para establecer un SCP perpendicular al objeto.

(defun ax-gradua
                 (obj distancia / longitud avance tmp)
  (setq longitud (ax-longtotal obj)
        avance   distancia
  ) ;_ fin de setq
  (while (<= avance longitud)
    (setq tmp    (cons
                   (vlax-curve-GetPointAtDist obj avance)
                   tmp
                 ) ;_ fin de cons
          avance (+ avance distancia)
    ) ;_ fin de setq
  ) ;_ fin de while
  (reverse tmp)
) ;_ fin de defun
;;;Listado 15.14. Calcular puntos a distancias fijas.

(defun C:PARTE-DIST (/ csel ent obj dist puntos)
  (vla-StartUndoMark *aesl:dibujo*)
  (prompt "\nDesigne entidad a partir: ")
  (while
    (not
      (and (setq csel
                  (ssget "_:S" '((0 . "*LINE,ARC,HELIX")))
           ) ;_ fin de setq
           (not (vlax-curve-isClosed (ssname csel 0)))
      ) ;_ fin de and
    ) ;_ fin de not
     (prompt "\nDesigne una entidad Lineal abierta: ")
  ) ;_ fin de while
  (setq ent (ssname csel 0)
        obj (vlax-ename->vla-object ent)
  ) ;_ fin de setq
  (initget (+ 1 2 4 128) "Distancia")
  (setq dist
         (getint
           "\nIndique el nmero de segmentos o [Distancia]:"
         ) ;_ fin de getint
  ) ;_ fin de setq
  (cond ((= (type dist) 'STR)
         (initget (+ 1 2 4))
         (setq
           dist (getreal "\nPrecise longitud de segmento:")
         ) ;_ fin de setq
        )
        (t
         (setq dist (/ (vlax-curve-GetDistAtParam
                         obj
                         (vlax-curve-GetEndParam obj)
                       ) ;_ fin de vlax-curve-GetDistAtParam
                       dist
                    ) ;_ fin de /
         ) ;_ fin de setq
        )
  ) ;_ fin de cond
  (setq puntos (ax-gradua obj dist))
  (cmd-entrar)
  (foreach pt puntos
    (vl-cmdf "_BREAK" 
             (list ent (trans pt 0 1)) 
             "_F" 
             (list ent (trans pt 0 1)) 
             "@")
    (setq ent (entlast))
  ) ;_ fin de foreach
  (cmd-salir)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 15.15. Partir una entidad en segmentos iguales.

(defun coord->puntos (lista ext / i puntos)
  (setq i 0)
  (while (< i ext)
    (setq puntos (cons (list (nth (- ext (+ i 2)) lista)
                             (nth (- ext (+ i 1)) lista)
                             (nth (- ext i) lista)
                       ) ;_ fin de list
                       puntos
                 ) ;_ fin de cons
          i      (+ i 3)
    ) ;_ fin de setq
  ) ;_ fin de while
  puntos
) ;_ fin de defun
;;;Listado 15.16. Creacin de una lista de puntos a partir de una lista de coordenadas.

(defun modo-ext (modo)
  (cond ((null modo) acExtendNone)
        ((eq modo T) acExtendBoth)
        (t modo)
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 15.17. Determinacin del modo de extender las entidades.

(defun intersecciones (obj1 obj2 modo / cruces limite pti)
  (setq cruces
         (vl-catch-all-apply
           'vla-IntersectWith
           (list (vlax-ename->vla-object obj1)
                 (vlax-ename->vla-object obj2)
                 (modo-ext modo)
           ) ;_ fin de list
         ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (not (vl-catch-all-error-p cruces))
    (if (< (setq limite
                  (vlax-safearray-get-u-bound
                    (setq pti
                           (vlax-variant-value
                             cruces
                           ) ;_ fin de vlax-variant-value
                    ) ;_ fin de setq
                    1
                  ) ;_ fin de vlax-safearray-get-u-bound
           ) ;_ fin de setq
           0
        ) ;_ fin de <
      nil
      (coord->puntos
        (vlax-safearray->list pti)
        limite
      ) ;_ fin de coord->puntos
    ) ;_ fin de if
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 15.18. Funcin INTERSECCIONES.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
