;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 16.	Programacin de Mallas de Caras 3D.---------------------------------

(defun cmd-dib-malla (m n lista-coordenadas /)
  (cmd-entrar)
  (apply 'vl-cmdf
         (append (list "_3dmesh" m n) lista-coordenadas)
  ) ;_ fin de apply
  (cmd-salir)
  (entlast)
) ;_ fin de defun
;;;Listado 16.1. Dibujo de la malla poligonal mediante la funcin command.

(defun PolygonMesh-cabecera (m n)
  (entmake (list '(0 . "POLYLINE")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbPolygonMesh")
                 '(70 . 16)
                 (cons 71 m)
                 (cons 72 n)
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 16.2. Funcin para generar la cabecera de la Malla Poligonal.

(defun PolygonMesh-vertice (xyz)
  (entmake (list '(0 . "VERTEX")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbVertex")
                 '(100 . "AcDbPolygonMeshVertex")
                 (cons 10 xyz)
                 '(70 . 64)
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 16.3. Funcin para generar cada entidad vrtice de la malla.

(defun ent-seqend ()
  (entmake (list '(0 . "SEQEND") '(100 . "AcDbEntity")))
) ;_ fin de defun
;;;Listado 16.4. Funcin invocada para crear la entidad fin de secuencia.

(defun ent-dib-malla (m n lista-coordenadas /)
  (PolygonMesh-cabecera m n)
  (foreach punto lista-coordenadas
    (PolygonMesh-vertice punto)
  ) ;_ fin de foreach
  (ent-seqend)
  (entlast)
) ;_ fin de defun
;;;Listado 16.5. Dibujo de la malla con entmake.

(defun ax-dib-malla
                    (m n lista-coordenadas / matriz-puntos)
  (setq lista-coordenadas (apply 'append
                                 lista-coordenadas
                          ) ;_ fin de apply
        matriz-puntos     (vlax-make-safearray
                            vlax-vbDouble
                            (cons
                              0
                              (- (length lista-coordenadas)
                                 1
                              ) ;_ fin de -
                            ) ;_ fin de cons
                          ) ;_ fin de vlax-make-safearray
  ) ;_ fin de setq
  (vlax-safearray-fill
    matriz-puntos
    lista-coordenadas
  ) ;_ fin de vlax-safearray-fill
  (vla-Add3dMesh
    (espacio-actual *aesl:dibujo*)
    m
    n
    matriz-puntos
  ) ;_ fin de vla-Add3dMesh
) ;_ fin de defun
;;;Listado 16.6. Creacin de la malla poligonal con el mtodo Add3dMesh.

(defun datos-malla (/)
  (initget 1 "Command Entmake Activex")
  (setq metodo
         (getkword
           "\nMtodo [Command/Entmake/Activex]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (initget 1 "1 2 3")
  (setq opcion (getkword "\nTipo de Superficie [1/2/3]: ")
        dimX   (getreal "\nDimensin en X: ")
        dimY   (getreal "\nDimensin en Y: ")
        dimZ   (getreal "\nDimensin en Z: ")
        res    (getint "\nResolucin de la malla: ")
  ) ;_ fin de setq
  (while (not (< 1 res 257))
    (prompt
      "\nEl valor de resolucin debe estar entre 2 y 256"
    ) ;_ fin de prompt
    (setq res (getint "\nResolucin de la malla: "))
  ) ;_ fin de while
  (initget 1 "Ninguno cUadrtico Cbico Bzier")
  (setq ajuste
         (getkword
           "\nAjuste [Ninguno/cUadrtico/Cbico/Bzier]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (if (/= ajuste "Ninguno")
    (progn (initget (+ 1 2 4))
           (setq densidad
                  (getint
                    "\nDensidad de suavizado (2 a 200):"
                  ) ;_ fin de getint
           ) ;_ fin de setq
    ) ;_ fin de progn
  ) ;_ fin de if
  (cond ((= ajuste "cUadrtico") (setq ajuste 5))
        ((= ajuste "Cbico") (setq ajuste 6))
        ((= ajuste "Bzier") (setq ajuste 8))
        (t (setq ajuste nil))
  ) ;_ fin de cond
  (initget 1)
  (setq origen (getpoint "\nCentro de la malla: ")
        pasoX  (/ dimX res)
        pasoY  (/ dimY res)
        Xmin   (- (/ dimX 2))
        Ymin   (- (/ dimY 2))
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 16.7. Funcin que solicita los datos para definicin de la malla.

;;; funcin f1
(defun f1 (x y /) (cos (sqrt (+ (* x x 2) (* y y)))))

;;; funcin f2
(defun f2 (x y /) (sqrt (abs (* x y))))

;;; funcin f3
(defun f3 (x y /) (/ (* x y) 10))
;;;Listado 16.8. Funciones para el clculo de distintas formas de superficie.

(defun op-formula (opcion /)
  (cond ((= opcion "1") 'f1)
        ((= opcion "2") 'f2)
        ((= opcion "3") 'f3)
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 16.9. Funcin que decide la frmula a emplear.

(defun calculo (formula Xmin Ymin dimz pasoX pasoY res / i j y lst
                f-alt
               )
  (setq i 0)
  (while (< i res)
    (setq j 0
          y Ymin
    ) ;_ fin de setq
    (while (< j res)
      (setq lst
             (cons
               (list Xmin
                     y
                     (apply formula (list Xmin y))
               ) ;_ fin de list
               lst
             ) ;_ fin de cons
      ) ;_ fin de setq
      (setq j (1+ j)
            y (+ y pasoY)
      ) ;_ fin de setq
    ) ;_ fin de while
    (setq i    (1+ i)
          Xmin (+ Xmin pasoX)
    ) ;_ fin de setq
  ) ;_ fin de while
  (setq f-alt (/ dimz
                 (- (apply
                      'max
                      (mapcar '(lambda (pt) (nth 2 pt))
                              lst
                      ) ;_ fin de mapcar
                    ) ;_ fin de apply
                    (apply
                      'min
                      (mapcar '(lambda (pt) (nth 2 pt))
                              lst
                      ) ;_ fin de mapcar
                    ) ;_ fin de apply
                 ) ;_ fin de -
              ) ;_ fin de /
        lst   (mapcar '(lambda (pt)
                         (list (nth 0 pt)
                               (nth 1 pt)
                               (* f-alt (nth 2 pt))
                         ) ;_ fin de list
                       ) ;_ fin de lambda
                      lst
              ) ;_ fin de mapcar
  ) ;_ fin de setq
  (reverse lst)
) ;_ fin de defun
;;;Listado 16.10. Funcin que calcula las coordenadas de los vrtices de la malla.

(defun C:POLYMALLA (/ tiempo metodo dimX dimY res origen pasoX
                    pasoY Xmin Ymin ajuste densidad
                    lista-coordenadas obj *error*
                   )
  (setq tiempo (getvar "millisecs"))
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-malla)
  (setq lista-coordenadas
         (calculo (op-formula opcion)
                  Xmin
                  Ymin
                  dimZ
                  pasoX
                  pasoY
                  res
         ) ;_ fin de calculo
  ) ;_ fin de setq
  (cond ((= metodo "Command")
         (cmd-entrar)
         (setq mtrans nil
               obj    (vlax-ename->vla-object
                        (cmd-dib-malla
                          res
                          res
                          lista-coordenadas
                        ) ;_ fin de cmd-dib-malla
                      ) ;_ fin de vlax-ename->vla-object
         ) ;_ fin de setq
         (cmd-salir)
        )
        ((= metodo "Entmake")
         (setq obj (vlax-ename->vla-object
                     (ent-dib-malla
                       res
                       res
                       lista-coordenadas
                     ) ;_ fin de ent-dib-malla
                   ) ;_ fin de vlax-ename->vla-object
         ) ;_ fin de setq
        )
        ((= metodo "Activex")
         (setq obj
                (ax-dib-malla res res lista-coordenadas)
         ) ;_ fin de setq
        )
  ) ;_ fin de cond
  (if ajuste
    (progn (vla-put-Type obj ajuste)
           (vla-put-MDensity obj densidad)
           (vla-put-NDensity obj densidad)
    ) ;_ fin de progn
  ) ;_ fin de if
  (if mtrans
    (vla-TransformBy obj mtrans)
  ) ;_ fin de if
  (ax-traslacion obj (trans origen 1 0 t))
  (vla-update obj)
  (ax-SWs)
  (prompt
    (strcat "\nTiempo de ejecucin: "
            (rtos (- (getvar "millisecs") tiempo) 2 0)
            " milisegundos"
    ) ;_ fin de strcat
  ) ;_ fin de prompt
  (vla-EndUndoMark *aesl:dibujo*)
  (princ)
) ;_ fin de defun
;;;Listado 16.11. Funcin principal C:POLYMALLA.

(defun cmd-dib-pcara (lista-vertices lista-caras /)
  (vl-cmdf "._pface")
  (foreach vert lista-vertices (vl-cmdf vert))
  (vl-cmdf "")
  (foreach cara lista-caras
    (foreach id cara (vl-cmdf id))
    (vl-cmdf "")
  ) ;_ fin de foreach
  (vl-cmdf "")
  (entlast)
) ;_ fin de defun
;;;Listado 16.12 Funcin que crea la malla Policara mediante el comando PCARA.

(defun def-cara (lista-caras / tmp res)
  (foreach cara lista-caras
    (setq vini  (nth 0 cara)
          nvert (length cara)
          i     1
    ) ;_ fin de setq
    (cond
      ((= nvert 3)
       (setq res (cons (append cara (list (nth 2 cara))) res))
      )
      ((= nvert 4) (setq res (cons cara res)))
      ((> nvert 4)
       (repeat (- nvert 2)
         (setq tmp nil)
         (cond ((= i 1)
                (setq
                  tmp (cons (list vini
                                  (nth i cara)
                                  (nth (setq i (1+ i)) cara)
                                  (- (nth i cara))
                            ) ;_ fin de list
                            tmp
                      ) ;_ fin de cons
                ) ;_ fin de setq
               )
               ((= i (- nvert 2))
                (setq tmp
                       (cons (list (- vini)
                                   (nth i cara)
                                   (+ (nth (setq i (1+ i)) cara))
                                   (nth i cara)
                             ) ;_ fin de list
                             tmp
                       ) ;_ fin de cons
                ) ;_ fin de setq
               )
               (t
                (setq tmp
                       (cons (list (- vini)
                                   (nth i cara)
                                   (- (nth (setq i (1+ i)) cara))
                                   (- (nth i cara))
                             ) ;_ fin de list
                             tmp
                       ) ;_ fin de cons
                ) ;_ fin de setq
               )
         ) ;_ fin de cond
         (setq res (append tmp res))
       ) ;_ fin de repeat
      )
      (t
       (prompt "ERROR: La cara debe tener tres o ms vrtices"
       ) ;_ fin de prompt
       (exit)
      )
    ) ;_ fin de cond
  ) ;_ fin de foreach
  (reverse res)
) ;_ fin de defun
;;;Listado 16.13. Discretizacin de las caras.

(defun policara-cabecera (lista-vertices lista-caras /)
  (entmake (list '(0 . "POLYLINE")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbPolyFaceMesh")
                 '(70 . 64)
                 (cons 71 (length lista-vertices))
                 (cons 72 (length lista-caras))
           ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 16.14. Creacin de la entidad cabecera de la malla.

(defun policara-vertices (lista-vertices /)
  (foreach vert lista-vertices
    (entmake (list '(0 . "VERTEX")
                   '(100 . "AcDbEntity")
                   '(100 . "AcDbVertex")
                   '(100 . "AcDbPolyFaceMeshVertex")
                   (cons 10 vert)
                   '(70 . 192)
             ) ;_ fin de list
    ) ;_ fin de entmake
  ) ;_ fin de foreach
) ;_ fin de defun
;;;Listado 16.15. Creacin de las entidades VERTEX.

(defun policara-caras (lista-caras /)
  (foreach cara lista-caras
    (entmake (list '(0 . "VERTEX")
                   '(100 . "AcDbEntity")
                   '(100 . "AcDbFaceRecord")
                   '(10 0.0 0.0 0.0)
                   '(70 . 128)
                   (cons 71 (nth 0 cara))
                   (cons 72 (nth 1 cara))
                   (cons 73 (nth 2 cara))
                   (cons 74 (nth 3 cara))
             ) ;_ fin de list
    ) ;_ fin de entmake
  ) ;_ fin de foreach
) ;_ fin de defun
;;;Listado 16.16. Creacin de las caras (entidades FaceRecord).

(defun ent-dib-pcara (lista-vertices lista-caras /)
  (setq lista-caras (def-cara lista-caras))
  (policara-cabecera lista-vertices lista-caras)
  (policara-vertices lista-vertices)
  (policara-caras lista-caras)
  (ent-seqend)
  (entlast)
) ;_ fin de defun
;;;Listado 16.17. Funcin que dibuja la malla Policara mediante entmake.

(defun ax-dib-pcara (lista-coordenadas lista-caras /
                     matriz-vertices matriz-caras
                    )
  (setq lista-coordenadas (apply 'append
                                 lista-coordenadas
                          ) ;_ fin de apply
        matriz-vertices   (vlax-make-safearray
                            vlax-vbDouble
                            (cons
                              0
                              (-
                                (length lista-coordenadas)
                                1
                              ) ;_ fin de -
                            ) ;_ fin de cons
                          ) ;_ fin de vlax-make-safearray
        matriz-vertices   (vlax-safearray-fill
                            matriz-vertices
                            lista-coordenadas
                          ) ;_ fin de vlax-safearray-fill
        lista-caras       (apply 'append
                                 (def-cara lista-caras)
                          ) ;_ fin de apply
        matriz-caras      (vlax-make-safearray
                            vlax-vbInteger
                            (cons 0
                                  (- (length lista-caras) 1)
                            ) ;_ fin de cons
                          ) ;_ fin de vlax-make-safearray
        matriz-caras      (vlax-safearray-fill
                            matriz-caras
                            lista-caras
                          ) ;_ fin de vlax-safearray-fill
  ) ;_ fin de setq
  (vla-AddPolyfaceMesh
    (espacio-actual *aesl:dibujo*)
    matriz-vertices
    matriz-caras
  ) ;_ fin de vla-AddPolyfaceMesh
) ;_ fin de defun
;;;Listado 16.18. Creacin de la malla mediante vla-AddPolyfaceMesh.

(defun datos-poliedro (/)
  (initget 1 "Command Entmake Activex")
  (setq metodo
         (getkword
           "\nMtodo [Command/Entmake/Activex]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (initget 1 "Tetraedro Hexaedro Dodecaedro")
  (setq clase  (getkword
                 "\nPoliedro [Tetraedro/Hexaedro/Dodecaedro]: "
               ) ;_ fin de getkword
        centro (getpoint "\nCentro del poliedro: ")
        radio  (getdist
                 centro
                 "\Radio de la esfera circunscrita: "
               ) ;_ fin de getdist
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 16.19. Solicitud de datos al usuario.

(defun op-poliedro (clase /)
  (cond ((= clase "Tetraedro")
         (setq vertices '((0 0 1)
                          (0 0.94280904 -0.33333333)
                          (-0.81649658 -0.47140452 -0.33333333)
                          (0.81649658 -0.47140452 -0.33333333)
                         )
               caras    '((1 2 3) (1 3 4) (1 4 2) (2 4 3))
         ) ;_ fin de setq
        )
        ((= clase "Hexaedro")
         (setq vertices '((-0.57735 -0.57735 -0.57735)
                          (-0.57735 0.57735 -0.57735)
                          (0.57735 0.57735 -0.57735)
                          (0.57735 -0.57735 -0.57735)
                          (-0.57735 -0.57735 0.57735)
                          (-0.57735 0.57735 0.57735)
                          (0.57735 0.57735 0.57735)
                          (0.57735 -0.57735 0.57735)
                         )
               caras    '((1 2 3 4)
                          (5 6 2 1)
                          (6 7 3 2)
                          (7 8 4 3)
                          (8 5 1 4)
                          (8 7 6 5)
                         )
         ) ;_ fin de setq
        )
        ((= clase "Dodecaedro")
         (setq vertices '((0.57735 -0.187592 0.794654)
                          (0.356822 0.491123 0.794654)
                          (-0.356822 0.491123 0.794654)
                          (-0.57735 -0.187592 0.794654)
                          (-1.11022e-016 -0.607062 0.794654)
                          (0.934172 -0.303531 0.187592)
                          (0.934172 0.303531 -0.187592)
                          (0.57735 0.794654 0.187592)
                          (-5.55112e-017 0.982247 -0.187592)
                          (-0.57735 0.794654 0.187592)
                          (-0.934172 0.303531 -0.187592)
                          (-0.934172 -0.303531 0.187592)
                          (-0.57735 -0.794654 -0.187592)
                          (5.55112e-017 -0.982247 0.187592)
                          (0.57735 -0.794654 -0.187592)
                          (0.356822 -0.491123 -0.794654)
                          (0.57735 0.187592 -0.794654)
                          (-1.33234e-016 0.607062 -0.794654)
                          (-0.57735 0.187592 -0.794654)
                          (-0.356822 -0.491123 -0.794654)
                         )
               caras    '((1 2 3 4 5)
                          (1 6 7 8 2)
                          (2 8 9 10 3)
                          (3 10 11 12 4)
                          (4 12 13 14 5)
                          (5 14 15 6 1)
                          (6 15 16 17 7)
                          (8 7 17 18 9)
                          (10 9 18 19 11)
                          (12 11 19 20 13)
                          (14 13 20 16 15)
                          (16 20 19 18 17)
                         )
         ) ;_ fin de setq
        )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 16.20. Carga de datos de vrtices y caras para el poliedro.

(defun C:POLIEDRO-PCARA (/ *error* tiempo mtrans metodo clase
                         centro radio obj
                        )
  (setq tiempo (getvar "millisecs"))
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-poliedro)
  (op-poliedro clase)
  (cond ((= metodo "Command")
         (cmd-entrar)
         (setq mtrans nil
               obj    (vlax-ename->vla-object
                        (cmd-dib-pcara vertices caras)
                      ) ;_ fin de vlax-ename->vla-object
         ) ;_ fin de setq
         (cmd-salir)
        )
        ((= metodo "Entmake")
         (setq obj (vlax-ename->vla-object
                     (ent-dib-pcara vertices caras)
                   ) ;_ fin de vlax-ename->vla-object
         ) ;_ fin de setq
        )
        ((= metodo "Activex")
         (setq obj (ax-dib-pcara vertices caras))
        )
  ) ;_ fin de cond
  ;; Transformaciones:
  (ax-escala obj (list radio radio radio))
  (if mtrans
    (vla-TransformBy obj mtrans)
  ) ;_ fin de if
  (ax-traslacion obj (trans centro 1 0 t))
  (ax-SWs)
  (prompt
    (strcat "\nTiempo de ejecucin: "
            (rtos (- (getvar "millisecs") tiempo) 2 0)
            " milisegundos"
    ) ;_ fin de strcat
  ) ;_ fin de prompt
  (vla-EndUndoMark *aesl:dibujo*)
  (princ)
) ;_ fin de defun
;;;Listado 16.21. Funcin Principal C:POLIEDRO-PCARA.

(defun lista-vertices-malla
                            (ent-malla suavizado / lista-vertices)
  (while
    (and
      (setq ent-malla (entnext ent-malla))
      (/=
        (cdr (assoc 0 (setq dxf (entget ent-malla))))
        "SEQEND"
      ) ;_ fin de /=
    ) ;_ fin de and
     (cond
       (suavizado
        (if (/= (logand 8 (cdr (assoc 70 dxf))) 0)
          (setq lista-vertices
                 (cons (cdr (assoc 10 dxf))
                       lista-vertices
                 ) ;_ fin de cons
          ) ;_ fin de setq
        ) ;_ fin de if
       )
       (t
        (if (/= (logand 16 (cdr (assoc 70 dxf))) 0)
          (setq lista-vertices
                 (cons (cdr (assoc 10 dxf))
                       lista-vertices
                 ) ;_ fin de cons
          ) ;_ fin de setq
        ) ;_ fin de if
       )
     ) ;_ fin de cond
  ) ;_ fin de while
  (reverse lista-vertices)
) ;_ fin de defun
;;;Listado 16.22. Funcin que devuelve los vrtices producidos por suavizado de una malla poligonal.

(defun cal-z (xyz ecuacion dim-z / z h f-esc)
  (while xyz
    (setq z   (cons (apply ecuacion
                           (list (nth 0 xyz) (nth 1 xyz))
                    ) ;_ fin de apply
                    z
              ) ;_ fin de cons
          xyz (cdddr xyz)
    ) ;_ fin de setq
  ) ;_ fin de while
  (setq h     (- (apply 'max z) (apply 'min z))
        f-esc (/ dim-z h)
  ) ;_ fin de setq
  (reverse (mapcar '(lambda (n) (* n f-esc)) z))
) ;_ fin de defun
;;;Listado 16.23. Funcin para el clculo del valor de la coordenada Z.

(defun ax-mod-malla (obj ecuacion dim-z / xyz lst-z i
                     nombre vertices pt
                    )
  (setq xyz    (vlax-safearray->list
                 (vlax-variant-value
                   (vla-get-coordinates obj)
                 ) ;_ fin de vlax-variant-value
               ) ;_ fin de vlax-safearray->list
        lst-z  (cal-z xyz ecuacion dim-z)
        i      0
        nombre (vla-get-ObjectName obj)
  ) ;_ fin de setq
  (cond
    ((= nombre "AcDbPolygonMesh")
     (setq vertices (* (vla-get-MVertexCount obj)
                       (vla-get-NVertexCount obj)
                    ) ;_ fin de *
     ) ;_ fin de setq
    )
    ((= (vla-get-ObjectName obj) "AcDbPolyFaceMesh")
     (setq vertices (vla-get-NumberOfVertices obj))
    )
    (t
     (prompt
       "\nEl objeto debe ser una Malla Poligonal o Policara."
     ) ;_ fin de prompt
     (exit)
    )
  ) ;_ fin de cond
  (repeat vertices
    (setq pt (vlax-safearray->list
               (vlax-variant-value
                 (vla-get-coordinate obj i)
               ) ;_ fin de vlax-variant-value
             ) ;_ fin de vlax-safearray->list
    ) ;_ fin de setq
    (vla-put-coordinate
      obj
      i
      (vlax-3d-point
        (list (nth 0 pt)
              (nth 1 pt)
              (+ (nth 2 pt) (nth i lst-z))
        ) ;_ fin de list
      ) ;_ fin de vlax-3d-point
    ) ;_ fin de vla-put-coordinate
    (setq i (1+ i))
  ) ;_ fin de repeat
  (vla-Update obj)
) ;_ fin de defun
;;;Listado 16.24. Modificacin de la posicin de los vrtices de una malla Poligonal o Policara.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
