;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 17.	Modelado de Slidos.

(defun ax-cono (control radio dim-z /)
  (vla-AddCone
    (espacio-actual *aesl:dibujo*)
    (apply 'vlax-3d-point control)
    radio
    dim-z
  ) ;_ fin de vla-AddCone
) ;_ fin de defun
;;;Listado 17.1.Creacin de una primitiva del tipo Cono.

(defun cmd-cono (ctrbase radinf radsup dim-z /)
  (cmd-entrar)
  (vl-cmdf "._CONE" ctrbase radinf)
  (if (/= radsup 0)
    (vl-cmdf "_t" radsup dim-z)
    (vl-cmdf dim-z)
  ) ;_ fin de if
  (cmd-salir)
) ;_ fin de defun
;;;Listado 17.2. Creacin de un cono mediante command/vl-cmdf.

(defun ax-region
                 (espacio contornos / perfiles region)
  (setq perfiles (vlax-make-safearray
                   vlax-vbobject
                   (cons 0 (1- (length contornos)))
                 ) ;_ fin de vlax-make-safearray
        perfiles (vlax-make-variant
                   (vlax-safearray-fill
                     perfiles
                     contornos
                   ) ;_ fin de vlax-safearray-fill
                 ) ;_ fin de vlax-make-variant
        region   (vl-catch-all-apply
                   'vla-AddRegion
                   (list espacio perfiles)
                 ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p region)
     (prompt (vl-catch-all-error-message region))
    )
    (t
     (if (= (getvar "DELOBJ") 1)
       (foreach contorno contornos
         (vla-Delete contorno)
       ) ;_ fin de foreach
     ) ;_ fin de if
     (vlax-safearray->list
       (vlax-variant-value region)
     ) ;_ fin de vlax-safearray->list
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 17.3. Funcin que crea regiones segn los contornos recibidos.

(defun ax-boolean (obj1 operacion obj2 / tmp)
  (setq tmp (vl-catch-all-apply
              'vla-Boolean
              (list obj1 operacion obj2)
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p tmp)
    (prompt (vl-catch-all-error-message tmp))
    obj1
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.4. Funcin que realiza una operacin Booleana sobre dos objetos controlando errores.

(defun datos-region (/ radmax)
  (initget 1)
  (setq centro (getpoint "\nCentro de la regin: "))
  (initget (+ 1 2 4))
  (setq
    radioext (getdist centro "\nRadio exterior: ")
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq numag      (getint "\nCantidad de agujeros: ")
        distcentro (* 2 (/ radioext 3.0))
        radmax     (apply
                     'min
                     (list
                       (/ (* pi distcentro) (* numag 1.1))
                       (/ radioext 3.1)
                     ) ;_ fin de list
                   ) ;_ fin de apply
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq radioag (getdist centro "\nRadio agujeros: "))
  (while (>= radioag radmax)
    (prompt
      (strcat
        "\nEl radio de agujeros debe ser menor que "
        (rtos radmax 2 2)
      ) ;_ fin de strcat
    ) ;_ fin de prompt
    (setq radioag
           (getdist centro "\nRadio agujeros: ")
    ) ;_ fin de setq
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 17.5. Funcin que solicita los datos para la regin compuesta.

(defun C:REG-COMP (/ espacio mtrans centro radioext numag
                   distcentro radioag origen normal cir-base base
                   ang incang cir-agujeros agujeros
                  )
  (setq espacio (espacio-actual *aesl:dibujo*))
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-region)
  (setq origen   (vlax-3d-point '(0 0 0))
        normal   (vlax-3d-point '(0.0 0.0 1.0))
        cir-base (vla-AddCircle espacio origen radioext)
  ) ;_ fin de setq
  (vla-put-Normal cir-base normal)
  (setq base (car (ax-region espacio (list cir-base))))
  (setq ang    0
        incang (/ (* 2 pi) numag)
  ) ;_ fin de setq
  (repeat numag
    (setq cir-agujeros (cons (vla-AddCircle
                               espacio
                               (vlax-3d-point
                                 (polar '(0 0 0)
                                        ang
                                        distcentro
                                 ) ;_ fin de polar
                               ) ;_ fin de vlax-3d-point
                               radioag
                             ) ;_ fin de vla-AddCircle
                             cir-agujeros
                       ) ;_ fin de cons
          ang          (+ ang incang)
    ) ;_ fin de setq
    (vla-put-Normal (car cir-agujeros) normal)
  ) ;_ fin de repeat
  (setq agujeros (ax-region espacio cir-agujeros))
  (foreach agujero agujeros
    (ax-boolean base acSubtraction agujero)
  ) ;_ fin de foreach
  (if (> (getvar "DELOBJ") 0)
    (progn
      (vla-Delete cir-base)
      (foreach cir-agujero cir-agujeros
        (vla-Delete cir-agujero)
      ) ;_ fin de foreach
    ) ;_ fin de progn
  ) ;_ fin de if
  (if mtrans
    (vla-TransformBy base mtrans)
  ) ;_ fin de if
  (ax-traslacion base (trans centro 1 0 t))
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 17.6. Programa ejemplo para creacin de regiones compuestas.

(defun reg-base (radioext numag distcentro radioag / espacio origen
                 normal cir-base base ang incang circulos regiones
                )
  (setq espacio  (espacio-actual *aesl:dibujo*)
        origen   (vlax-3d-point '(0 0 0))
        normal   (vlax-3d-point '(0.0 0.0 1.0))
        cir-base (vla-AddCircle espacio origen radioext)
  ) ;_ fin de setq
  (vla-put-Normal cir-base normal)
  (setq
    base (car (ax-region espacio (list cir-base)))
  ) ;_ fin de setq
  (setq ang    0
        incang (/ (* 2 pi) numag)
  ) ;_ fin de setq
  (repeat numag
    (setq circulos (cons (vla-AddCircle
                           espacio
                           (vlax-3d-point
                             (polar '(0 0 0)
                                    ang
                                    distcentro
                             ) ;_ fin de polar
                           ) ;_ fin de vlax-3d-point
                           radioag
                         ) ;_ fin de vla-AddCircle
                         circulos
                   ) ;_ fin de cons
          ang      (+ ang incang)
    ) ;_ fin de setq
    (vla-put-Normal (car circulos) normal)
  ) ;_ fin de repeat
  (setq regiones (ax-region espacio circulos))
  (foreach region regiones
    (ax-boolean base acSubtraction region)
  ) ;_ fin de foreach
  (if (> (getvar "DELOBJ") 0)
    (progn
      (vla-Delete cir-base)
      (foreach circulo circulos (vla-Delete circulo))
    ) ;_ fin de progn
  ) ;_ fin de if
  base
) ;_ fin de defun
;;;Listado 17.7. Funcin que crea la regin compuesta usada como base.

(defun sol-hist (solido)
  (if (= (getvar "SOLIDHIST") 1)
    (vla-put-History solido :vlax-true)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.8. Activar la propiedad de Registro en un Slido 3D.

(defun C:SOL-EXT (/ centro radioext numag radioag distcentro
                  radioag dim-z amax angc circ perfiles region
                  extrusion
                 )
  (vla-startundomark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-region)
  (initget 1)
  (setq dimZ (getdist centro
                      "\nIndique altura de extrusin: "
             ) ;_ fin de getdist
        amax (- 90.0 (rag (atan (/ dimZ radioext))))
  ) ;_ fin de setq
  (initget 1)
  (setq anguloConicidad
         (getreal "\nngulo de conicidad:")
  ) ;_ fin de setq
  (while (not (<= -90.0 anguloConicidad amax))
    (setq anguloConicidad
           (getreal
             (strcat
               "\nEl ngulo debe ser mayor que -90 Y menor que "
               (rtos amax)
               ""
             ) ;_ fin de strcat
           ) ;_ fin de getreal
    ) ;_ fin de setq
  ) ;_ fin de while
  (setq region (reg-base radioext
                         numag
                         distcentro
                         radioag
               ) ;_ fin de reg-base
  ) ;_ fin de setq
  (setq anguloConicidad (gar anguloConicidad)
        extrusion       (vl-catch-all-apply
                          'vla-addextrudedsolid
                          (list (espacio-actual *aesl:dibujo*)
                                region
                                dimZ
                                anguloConicidad
                          ) ;_ fin de list
                        ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p extrusion)
     (prompt (vl-catch-all-error-message extrusion))
    )
    (t
     (sol-hist extrusion)
     (if (> (getvar "DELOBJ") 0)
       (vla-Delete region)
     ) ;_ fin de if
     (if mtrans
       (vla-TransformBy extrusion mtrans)
     ) ;_ fin de if
     (ax-traslacion extrusion (trans centro 1 0 t))
     (ax-SWs)
    )
  ) ;_ fin de cond
  (vla-endundomark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 17.9. Creacin de un Slido de Extrusin.

(defun ax-ext-tray (espacio perfil tray / res)
  (if (vl-catch-all-error-p
        (setq res (vl-catch-all-apply
                    'vla-AddExtrudedSolidAlongPath
                    (list espacio perfil tray)
                  ) ;_ fin de vl-catch-all-apply
        ) ;_ fin de setq
      ) ;_ fin de vl-catch-all-error-p
    (princ
      (strcat "\nERROR: " (vl-catch-all-error-message res))
    ) ;_ fin de princ
    (progn (sol-hist res) (vla-update res))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.10. Funcin auxiliar ax-ext-tray.

(defun C:SOL-TRAY
                  (/ espacio tray perf inicio normal ptref desp)
  (setq espacio (espacio-actual *aesl:dibujo*))
  (vla-StartUndoMark *aesl:dibujo*)
  (prompt
    "\nSeleccione una spline plana como trayectoria. "
  ) ;_ fin de prompt
  (while (not (setq tray (ssget "_:S"
                                '((-4 . "<AND")
                                  (0 . "SPLINE")
                                  (-4 . "&")
                                  (70 . 8)
                                  (-4 . "AND>")
                                 )
                         ) ;_ fin de ssget
              ) ;_ fin de setq
         ) ;_ fin de not
    (prompt "\nDebe seleccionar una spline plana, ")
  ) ;_ fin de while
  (setq tray (vlax-ename->vla-object (ssname tray 0)))
  (prompt
    "\nSeleccione una polilnea 2D como perfil. "
  ) ;_ fin de prompt
  (while
    (not (setq
           perf (ssget "_:S" '((0 . "LWPOLYLINE")))
         ) ;_ fin de setq
    ) ;_ fin de not
     (prompt "\nDebe seleccionar una polilnea 2D.")
  ) ;_ fin de while
  (setq perf (vlax-ename->vla-object (ssname perf 0)))
  (if (= (vla-get-closed perf) :vlax-false)
    (vla-put-closed perf :vlax-true)
  ) ;_ fin de if
  (setq inicio (vlax-curve-GetPointAtParam
                 tray
                 (vlax-curve-GetStartParam tray)
               ) ;_ fin de vlax-curve-GetPointAtParam
        normal (vlax-3d-point
                 (vlax-curve-GetFirstDeriv
                   tray
                   (vlax-curve-GetStartParam tray)
                 ) ;_ fin de vlax-curve-GetFirstDeriv
               ) ;_ fin de vlax-3d-point
  ) ;_ fin de setq
  (vla-put-normal perf normal)
  (setq ptref (vlax-safearray->list
                (vlax-variant-value
                  (vla-get-coordinate perf 0)
                ) ;_ fin de vlax-variant-value
              ) ;_ fin de vlax-safearray->list
        ptref (trans ptref
                     (vlax-safearray->list
                       (vlax-variant-value normal)
                     ) ;_ fin de vlax-safearray->list
                     0
              ) ;_ fin de trans
        desp  (mapcar '- inicio ptref)
  ) ;_ fin de setq
  (ax-traslacion perf desp)
  (setq perf (car (ax-region espacio (list perf))))
  (ax-ext-tray espacio perf tray)
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 17.11. Creacin de un Slido por barrido a lo largo de una trayectoria.

(defun cmd-helice-base ()
  (cmd-entrar)
  (vl-cmdf "_HELIX"
           (trans '(0.0 0.0 0.0) 0 1)
           (trans '(1.0 0.0 0.0) 0 1)
           1.0
           "_Turns"
           1
           "_Axis"
           (trans '(0.0 0.0 1.0) 0 1)
  ) ;_ fin de vl-cmdf
  (cmd-salir)
  (vlax-ename->vla-object (entlast))
) ;_ fin de defun
;;;Listado 17.12. Funcin que crea un objeto HLICE mediante command/vl-cmdf.

(defun ent-helice-base ()
  (if (entmake '((0 . "HELIX")
                 (100 . "AcDbEntity")
                 (100 . "AcDbSpline")
                 (70 . 0)
                 (71 . 3)
                 (72 . 11)
                 (73 . 7)
                 (74 . 0)
                 (42 . 1.0e-010)
                 (43 . 1.0e-010)
                 (40 . 0.0)
                 (40 . 0.0)
                 (40 . 0.0)
                 (40 . 0.0)
                 (40 . 1.16642)
                 (40 . 1.16642)
                 (40 . 1.16642)
                 (40 . 1.5708)
                 (40 . 1.5708)
                 (40 . 1.5708)
                 (40 . 1.5708)
                 (10 1.0 0.0 0.0)
                 (10 1.0 0.388808 0.0618807)
                 (10 0.750892 0.766376 0.123761)
                 (10 0.393442 0.919349 0.185642)
                 (10 0.269522 0.972382 0.207095)
                 (10 0.134791 1.0 0.228547)
                 (10 6.12323e-017 1.0 0.25)
                 (100 . "AcDbHelix")
                 (90 . 29)
                 (91 . 65)
                 (10 0.0 0.0 0.0)
                 (11 1.0 0.0 0.0)
                 (12 0.0 0.0 1.0)
                 (40 . 1.0)
                 (41 . 0.25)
                 (42 . 1.0)
                 (290 . 1)
                 (280 . 1)
                )
      ) ;_ fin de entmake
    (vlax-ename->vla-object (entlast))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.13. Funcin que crea una HLICE mediante entmake.

(defun ax-helice
       (posicion altura vueltas radiobase radiosup / obj)
  (if (setq obj (ent-helice-base))
    (progn (vla-put-Height obj altura)
           (vla-put-Turns obj vueltas)
           (vla-put-BaseRadius obj radiobase)
           (vla-put-TopRadius obj radiosup)
           (vla-put-Position obj (vlax-3d-point posicion))
           obj
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.14. Funcin que crea la hlice ajustando sus propiedades.

(defun datos-muelle (/ diamext diamhilo)
  (initget 1)
  (setq centro (getpoint "\nCentro de la base"))
  (initget (+ 1 2 4))
  (setq diamext (getdist centro "\nDimetro exterior: "))
  (initget (+ 1 2 4))
  (setq altura (getdist centro "\nLongitud del muelle: "))
  (initget (+ 1 2 4))
  (while
    (> (setq diamhilo (getdist centro "\nDimetro de hilo: "))
       (/ diamext 5.0)
    ) ;_ fin de >
     (prompt
       (strcat "\nEl dimetro del hilo debe ser menor que "
               (rtos (/ diamext 5.0) 2 2)
       ) ;_ fin de strcat
     ) ;_ fin de prompt
     (initget (+ 1 2 4))
  ) ;_ fin de while
  (initget (+ 1 2 4))
  (while (> 1
            (setq vueltas (getint "\nNmero de vueltas: "))
            (fix (/ altura (* diamhilo 2)))
         ) ;_ fin de >
    (prompt
      (strcat "\nEl Nmero de vueltas no puede ser mayor que "
              (itoa (fix (/ altura (* diamhilo 2))))
      ) ;_ fin de strcat
    ) ;_ fin de prompt
    (initget (+ 1 2 4))
  ) ;_ fin de while
  (setq radiohilo (/ diamhilo 2.0)
        radiobase (- (/ diamext 2.0) radiohilo)
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 17.15. Solicitud al usuario de los datos para construir el muelle.

(defun cmd-barrido (perfil tray / csperfil cstray res)
  (cmd-entrar)
  (setq csperfil (ssadd)
        cstray   (ssadd)
  ) ;_ fin de setq
  (ssadd perfil csperfil)
  (ssadd tray cstray)
  (vl-cmdf "._SWEEP" csperfil "" cstray)
  (cond ((> (getvar "DELOBJ") 0) (entdel tray)))
  (cmd-salir)
  (setq res (vlax-ename->vla-object (entlast)))
  (sol-hist res)
  res
) ;_ fin de defun
;;;Listado 17.16. Funcin que crea un slido 3D utilizando el comando BARRIDO (_SWEEP).

(defun C:MUELLE (/ *error* espacio mtrans centro altura vueltas
                 radiohilo radiobase trayectoria ptref normal
                 perfil reg-perfil obj-muelle
                )
  (setq espacio (espacio-actual *aesl:dibujo*))
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-muelle)
  (setq trayectoria (ax-helice
                      '(0 0 0) altura vueltas radiobase radiobase
                     ) ;_ fin de ax-helice
        ptref       (vlax-curve-GetPointAtParam
                      trayectoria
                      (vlax-curve-GetStartParam trayectoria)
                    ) ;_ fin de vlax-curve-GetPointAtParam
        normal      (vlax-curve-GetFirstDeriv
                      trayectoria
                      (vlax-curve-GetStartParam trayectoria)
                    ) ;_ fin de vlax-curve-GetFirstDeriv
        perfil      (vl-catch-all-apply
                      'vla-AddCircle
                      (list espacio
                            (vlax-3d-point ptref)
                            radiohilo
                      ) ;_ fin de list
                    ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p perfil)
     (prompt (vl-catch-all-error-message perfil))
    )
    (t
     (vla-put-normal perfil (vlax-3d-point normal))
     (setq reg-perfil  (vlax-vla-object->ename
                         (car (ax-region espacio (list perfil)))
                       ) ;_ fin de vlax-vla-object->ename
           trayectoria (vlax-vla-object->ename trayectoria)
     ) ;_ fin de setq
     (setq obj-muelle (cmd-barrido reg-perfil trayectoria))
     (cond
       ((= (getvar "DELOBJ") 0)
        (vla-TransformBy perfil mtrans)
        (vla-TransformBy trayectoria mtrans)
        (ax-traslacion perfil (trans centro 1 0 t))
        (ax-traslacion trayectoria (trans centro 1 0 t))
       )
       (t
        (vla-Delete perfil)
        (entdel reg-perfil)
        (entdel trayectoria)
        (entdel reg-perfil)
       )
     ) ;_ fin de cond
     (if mtrans
       (vla-TransformBy obj-muelle mtrans)
     ) ;_ fin de if
     (ax-traslacion obj-muelle (trans centro 1 0 t))
     (vla-Update obj-muelle)
     (ax-SWs)
     (vla-EndUndoMark *aesl:dibujo*)
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 17.17. Funcin principal C:MUELLE.

(defun ax-sol-rev (espacio perfil punto vector angulo / res)
  (if (vl-catch-all-error-p
        (setq res (vl-catch-all-apply
                    'vla-AddRevolvedSolid
                    (list espacio
                          perfil
                          (vlax-3d-point punto)
                          (vlax-3d-point vector)
                          angulo
                    ) ;_ fin de list
                  ) ;_ fin de vl-catch-all-apply
        ) ;_ fin de setq
      ) ;_ fin de vl-catch-all-error-p
    (prompt
      (strcat "\nERROR: " (vl-catch-all-error-message res))
    ) ;_ fin de prompt
    (progn (sol-hist res) (vla-Update res))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.18. Funcin que crea un slido por revolucin.

(defun C:SOL-REV
                 (/ espacio perf eje ang centro vec)
  (setq espacio (espacio-actual *aesl:dibujo*))
  (vla-StartUndoMark *aesl:dibujo*)
  (prompt "\nSeleccione el perfil:")
  (while
    (not
      (setq perf
             (ssget
               "_:S"
               '((0 . "REGION,LWPOLYLINE,SPLINE,CIRCLE,ELLIPSE"))
             ) ;_ fin de ssget
      ) ;_ fin de setq
    ) ;_ fin de not
     (prompt
       "\nSeleccione una Regin, Polilnea 2D, Crculo o Elipse:"
     ) ;_ fin de prompt
  ) ;_ fin de while
  (prompt "\nSeleccione el eje de revolucin: ")
  (while (not (setq eje (ssget "_:S"
                               '((-4 . "<AND")
                                 (0 . "*LINE,ARC")
                                 (-4 . "<NOT")
                                 (-4 . "&")
                                 (70 . 1)
                                 (-4 . "NOT>")
                                 (-4 . "AND>")
                                )
                        ) ;_ fin de ssget
              ) ;_ fin de setq
         ) ;_ fin de not
    (prompt "\nSeleccione una entidad LINEAL NO CERRADA:")
  ) ;_ fin de while
  (initget (+ 2 4))
  (if (not (setq ang (getreal "\nngulo a barrer <360>: ")))
    (setq ang 360.0)
    (while (> ang 360.0)
      (initget (+ 2 4))
      (setq ang
             (getreal
               "\nDebe ser igual o menor que 360 <360>: "
             ) ;_ fin de getreal
      ) ;_ fin de setq
    ) ;_ fin de while
  ) ;_ fin de if
  (setq perf (vlax-ename->vla-object (ssname perf 0)))
  (if (/= (vla-get-ObjectName perf) "AcDbRegion")
    (cond
      ((and (vlax-property-available-p perf "Closed")
            (= (vla-get-Closed perf) :vlax-false)
       ) ;_ fin de and
       (if (vl-catch-all-error-p
             (vl-catch-all-apply
               'vla-put-Closed
               (list perf :vlax-true)
             ) ;_ fin de vl-catch-all-apply
           ) ;_ fin de vl-catch-all-error-p
         (vla-put-Closed2 perf :vlax-true)
       ) ;_ fin de if
      )
    ) ;_ fin de cond
  ) ;_ fin de if
  (setq perfiles (vlax-make-safearray
                   vlax-vbObject
                   '(0 . 0)
                 ) ;_ fin de vlax-make-safearray
        perfiles (vlax-make-variant
                   (vlax-safearray-fill
                     perfiles
                     (list perf)
                   ) ;_ fin de vlax-safearray-fill
                 ) ;_ fin de vlax-make-variant
        perf     (vl-catch-all-apply
                   'vla-AddRegion
                   (list espacio perfiles)
                 ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p perf)
     (prompt (strcat "ERROR:\t"
                     (vl-catch-all-error-message perf)
             ) ;_ fin de strcat
     ) ;_ fin de prompt
    )
    (t
     (setq perf (vlax-safearray-get-element
                  (vlax-variant-value perf)
                  0
                ) ;_ fin de vlax-safearray-get-element
     ) ;_ fin de setq
     (setq ang (gar ang)
           eje (vlax-ename->vla-object (ssname eje 0))
     ) ;_ fin de setq
     (if (= (vla-get-ObjectName eje) "AcDbXline")
       (setq centro (vlax-safearray->list
                      (vlax-variant-value
                        (vla-get-BasePoint eje)
                      ) ;_ fin de vlax-variant-value
                    ) ;_ fin de vlax-safearray->list
             vec    (vlax-safearray->list
                      (vlax-variant-value
                        (vla-get-DirectionVector eje)
                      ) ;_ fin de vlax-variant-value
                    ) ;_ fin de vlax-safearray->list
       ) ;_ fin de setq
       (setq centro (vlax-curve-GetStartPoint eje)
             vec    (mapcar '-
                            (vlax-curve-GetEndPoint eje)
                            centro
                    ) ;_ fin de mapcar
       ) ;_ fin de setq
     ) ;_ fin de if
     (ax-sol-rev espacio perf centro vec ang)
     (ax-SWs)
    )
  ) ;_ fin de cond
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 17.19. Programa de ejemplo que crea un slido de revolucin.

(defun ax-props (obj / props valor pmin pmax bbox)
  (setq props
         (vl-remove-if
           'null
           (mapcar
             '(lambda (prop)
                (if (vlax-property-available-p obj prop)
                  (progn
                    (setq valor (vl-catch-all-apply
                                  'vlax-get-property
                                  (list obj prop)
                                ) ;_ fin de vl-catch-all-apply
                    ) ;_ fin de setq
                    (if (not (vl-catch-all-error-p valor))
                      (if (= (type valor) 'variant)
                        (cond
                          ((>= (vlax-variant-type valor)
                               8192
                           ) ;_ fin de >=
                           (cons prop
                                 (vlax-safearray->list
                                   (vlax-variant-value valor)
                                 ) ;_ fin de vlax-safearray->list
                           ) ;_ fin de cons
                          )
                          ((t
                             (cons prop
                                   (vlax-variant-value valor)
                             ) ;_ fin de cons
                           ) ;_ fin de t
                          )
                        ) ;_ fin de cond
                        (cons prop valor)
                      ) ;_ fin de if
                    ) ;_ fin de if
                  ) ;_ fin de progn
                ) ;_ fin de if
              ) ;_ fin de lambda
             '("Centroid" "MomentOfInertia" "PrincipalDirections"
               "PrincipalMoments" "ProductOfInertia"
               "RadiiOfGyration" "Volume" "Area" "Circumference"
               "Radius" "Center" "Normal" "Perimeter" "Coordinates"
               "FaceCount" "VertexCount" "Smoothness" "Elevation"
               "ArcLength" "EndAngle" "EndPoint" "StartAngle"
               "StartPoint" "TotalAngle" "Angle" "Delta"
               "Thickness" "BasePoint" "DirectionVector"
               "SecondPoint" "BaseRadius" "Height" "Position"
               "TopRadius" "TotalLength" "TurnHeight" "Turns"
               "TurnSlope" "Twist" "Direction" "TaperAngle"
               "EndDraftAngle" "EndDraftMagnitude"
               "NumCrossSections" "NumGuidePaths" "StartDraftAngle"
               "StartDraftMagnitude" "SurfaceNormals" "SurfaceType"
               "RevolutionAngle" "AxisPosition" "AxisDirection"
               "ProfileRotation" "Bank" "Length" "ProfileRotation"
               "scale"
              )
           ) ;_ fin de mapcar
         ) ;_ fin de vl-remove-if
  ) ;_ fin de setq
  (setq bbox (vl-catch-all-apply
               'vla-GetBoundingBox
               (list obj 'pmin 'pmax)
             ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p bbox)
    props
    (setq props (cons (list "BoundingBox"
                            (vlax-safearray->list pmin)
                            (vlax-safearray->list pmax)
                      ) ;_ fin de list
                      props
                ) ;_ fin de cons
    ) ;_ fin de setq
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.20. Funcin que extrae propiedades fsicas y geomtricas de los objetos.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
