;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 18.	Edicin de Slidos.

(defun ax-corte (obj pt1 pt2 pt3 negativo /)
  (setq res (vl-catch-all-apply
              'vla-SliceSolid
              (list obj
                    (vlax-3d-point pt1)
                    (vlax-3d-point pt2)
                    (vlax-3d-point pt3)
                    negativo
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt (vl-catch-all-error-message res))
    res
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 18.1. Funcin que realiza el corte del slido.

(defun datos-sol-p (/)
  (initget 1 "Tetraedro Hexaedro Dodecaedro")
  (setq clase  (getkword
                 "\nPoliedro [Tetraedro/Hexaedro/Dodecaedro]: "
               ) ;_ fin de getkword
        centro (getpoint "\nCentro del poliedro: ")
        radio  (getdist
                 centro
                 "\Radio de la esfera circunscrita: "
               ) ;_ fin de getdist
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 18.2. Funcin datos-sol-p que solicita la entrada de datos al usuario.

(defun C:SOL-POLIEDRO (/ clase centro radio esfera)
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-sol-p)
  (op-poliedro clase)
  (setq esfera
         (vl-catch-all-apply
           'vla-AddSphere
           (list (espacio-actual *aesl:dibujo*)
                 (vlax-3d-point '(0 0 0))
                 1.0
           ) ;_ fin de list
         ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p esfera)
     (prompt (vl-catch-all-error-message esfera))
    )
    (t
     (sol-hist esfera)
     (foreach cara caras
       (ax-corte esfera
                 (nth (1- (car cara)) vertices)
                 (nth (1- (cadr cara)) vertices)
                 (nth (1- (caddr cara)) vertices)
                 :vlax-false
       ) ;_ fin de ax-corte
     ) ;_ fin de foreach
     ;; Transformaciones:
     (ax-escala esfera (list radio radio radio))
     (if mtrans
       (vla-TransformBy esfera mtrans)
     ) ;_ fin de if
     (ax-traslacion esfera (trans centro 1 0 t))
     (vla-Update esfera)
     (ax-SWs)
    )
  ) ;_ fin de cond
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 18.3. Funcin principal C:SOL-POLIEDRO.

(defun ax-seccion (obj pt1 pt2 pt3 /)
  (setq res (vl-catch-all-apply
              'vla-SectionSolid
              (list obj
                    (vlax-3d-point pt1)
                    (vlax-3d-point pt2)
                    (vlax-3d-point pt3)
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt (vl-catch-all-error-message res))
    res
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 18.4. Funcin que crea la seccin del slido como regin.

(defun C:SECC-POLIEDRO (/ clase centro radio esfera regiones)
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-sol-p)
  (op-poliedro clase)
  (setq esfera (vl-catch-all-apply
                 'vla-AddSphere
                 (list (espacio-actual *aesl:dibujo*)
                       (vlax-3d-point '(0 0 0))
                       1.0
                 ) ;_ fin de list
               ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond ((vl-catch-all-error-p esfera)
         (prompt (vl-catch-all-error-message esfera))
        )
        (t
         (sol-hist esfera)
         (foreach cara caras
           (setq regiones (cons (ax-seccion
                                  esfera
                                  (nth (1- (car cara)) vertices)
                                  (nth (1- (cadr cara)) vertices)
                                  (nth (1- (caddr cara)) vertices)
                                ) ;_ fin de ax-seccion
                                regiones
                          ) ;_ fin de cons
           ) ;_ fin de setq
         ) ;_ fin de foreach
         (if (> (getvar "DELOBJ") 0)
           (vla-Delete esfera)
         ) ;_ fin de if
         (foreach region regiones       ; Transformaciones
           (ax-escala region (list radio radio radio))
           (if mtrans
             (vla-TransformBy region mtrans)
           ) ;_ fin de if
           (ax-traslacion region (trans centro 1 0 t))
         ) ;_ fin de foreach
         (ax-SWs)
        )
  ) ;_ fin de cond
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 18.5. Funcin principal C:SECC-POLIEDRO.

(defun datos-conector (/ maxdiam)
  (setq origen  '(0.0 0.0 0.0)
        centro  (getpoint "\nDesigne centro del conector: ")
        grosor  (getdist centro "\nDesigne grosor del conector: ")
        maxdiam (* grosor 2.5)
  ) ;_ fin de setq
  (initget (+ 2 4))
  (while (or (not diam) (> diam maxdiam))
    (setq diam (getreal (strcat "\nDimetro de los agujeros <"
                                (rtos (* grosor 2))
                                ">: "
                        ) ;_ fin de strcat
               ) ;_ fin de getreal
    ) ;_ fin de setq
    (cond
      ((not diam) (setq diam (* grosor 2)))
      (t
       (if (> diam maxdiam)
         (prompt
           (strcat
             "\nEl dimetro del agujero no debe ser mayor que "
             (rtos maxdiam 2 2)
           ) ;_ fin de strcat
         ) ;_ fin de prompt
       ) ;_ fin de if
       (initget (+ 2 4))
      )
    ) ;_ fin de cond
  ) ;_ fin de while
  (cond
    ((= (getvar "SOLIDHIST") 0)
     (initget 1 "Si No")
     (if
       (equal
         (getkword
           "\nActivar el Historial del slido? [Si/No]: "
         ) ;_ fin de getkword
         "Si"
       ) ;_ fin de equal
        (setvar "SOLIDHIST" 1)
     ) ;_ fin de if
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 18.6. Entrada de datos para el conector.

(defun ax-cubo (centro lado / res)
  (setq res (vl-catch-all-apply
              'vla-AddBox
              (list (espacio-actual *aesl:dibujo*)
                    (vlax-3d-point centro)
                    lado
                    lado
                    lado
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond ((vl-catch-all-error-p res)
         (prompt
           (strcat "\nERROR: " (vl-catch-all-error-message res))
         ) ;_ fin de prompt
        )
        (t
         (if (= (getvar "SOLIDHIST") 1)
           (vla-put-History res :vlax-true)
         ) ;_ fin de if
         res
        )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 18.7 Funcin utilizada para crear los cubos como Slidos 3D.

(defun ax-cilindro (centro radio dim-z / res)
  (setq res
         (vl-catch-all-apply
           'vla-AddCylinder
           (list
             (espacio-actual *aesl:dibujo*)
             (vlax-3d-point centro)
             radio
             dim-z
           ) ;_ fin de list
         ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond ((vl-catch-all-error-p res)
         (prompt
           (strcat "\nERROR: " (vl-catch-all-error-message res))
         ) ;_ fin de prompt
        )
        (t
         (if (= (getvar "SOLIDHIST") 1)
           (vla-put-History res :vlax-true)
         ) ;_ fin de if
         res
        )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 18.8. Funcin utilizada para crear los cilindros como Slidos 3D.

(defun rot-90-x (obj / ang)
  (setq ang (/ pi 2))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list 1.0 0.0 0.0 0.0)
            (list 0.0 (cos ang) (sin ang) 0.0)
            (list 0.0 (- (sin ang)) (cos ang) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 18.9. Funcin que gira un objeto 90 en torno al eje X.

(defun rot-90-y (obj / ang)
  (setq ang (/ pi 2))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos ang) 0.0 (sin ang) 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list (- (sin ang)) 0.0 (cos ang) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 18.10. Funcin que gira un objeto 90 en torno al eje Y.

(defun elimina-duplicados (lista / tmp)
  (while lista
    (setq tmp   (cons (car lista) tmp)
          lista (vl-remove-if
                  '(lambda (a) (equal a (car tmp) 0.0001))
                  lista
                ) ;_ fin de vl-remove-if
    ) ;_ fin de setq
  ) ;_ fin de while
  (reverse tmp)
) ;_ fin de defun
;;;Listado 18.11. Funcin que elimina duplicados en una lista.

(defun C:CONECTOR (/ mtrans origen centro grosor lado diam desp
                   posiciones centros base dif
                  )
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-conector)
  (setq lado       (* grosor 7)
        base       (ax-cubo origen lado)
        desp       (list (/ lado 2.0) (/ lado 2.0) (/ lado 2.0))
        posiciones '((1 1 1)
                     (1 1 -1)
                     (1 -1 1)
                     (1 -1 -1)
                     (-1 1 1)
                     (-1 1 -1)
                     (-1 -1 1)
                     (-1 -1 -1)
                    )
        centros    (mapcar
                     '(lambda (pos) (mapcar '* desp pos))
                     posiciones
                   ) ;_ fin de mapcar
        lado       (* grosor 6)
  ) ;_ fin de setq
  (mapcar '(lambda (ctr)
             (setq dif (ax-cubo ctr lado))
             (vla-Boolean base acSubtraction dif)
           ) ;_ fin de lambda
          centros
  ) ;_ fin de mapcar
  (setq lado    (* grosor 2.0)
        desp    (list lado lado 0.0)
        centros (elimina-duplicados
                  (mapcar '(lambda (pos)
                             (mapcar '* desp pos)
                           ) ;_ fin de lambda
                          posiciones
                  ) ;_ fin de mapcar
                ) ;_ fin de elimina-duplicados
  ) ;_ fin de setq
  (mapcar '(lambda (ctr)
             (setq dif (ax-cilindro ctr (/ diam 2.0) lado))
             (vla-Boolean base acSubtraction dif)
           ) ;_ fin de lambda
          centros
  ) ;_ fin de mapcar
  (mapcar '(lambda (ctr)
             (setq dif (ax-cilindro ctr (/ diam 2.0) lado))
             (rot-90-y dif)
             (vla-Boolean base acSubtraction dif)
           ) ;_ fin de lambda
          centros
  ) ;_ fin de mapcar
  (mapcar '(lambda (ctr)
             (setq dif (ax-cilindro ctr (/ diam 2.0) lado))
             (rot-90-x dif)
             (vla-Boolean base acSubtraction dif)
           ) ;_ fin de lambda
          centros
  ) ;_ fin de mapcar
  (if mtrans
    (vla-TransformBy base mtrans)
  ) ;_ fin de if
  (ax-traslacion base (trans centro 1 0 t))
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 18.12. Funcin principal C:CONECTOR.

(defun datos-eslabon (/)
  (initget 1)
  (setq centro (getpoint "\nDesigne centro del eslabn: "))
  (initget (+ 1 2 4))
  (setq dim-x  (getdist centro "\nIndique longitud del eslabn: ")
        origen '(0.0 0.0 0.0)
        r      (/ dim-x 2.0)
  ) ;_ fin de setq
  (cond
    ((= (getvar "SOLIDHIST") 0)
     (initget 1 "Si No")
     (if
       (equal
         (getkword
           "\nActivar el Historial del slido? [Si/No]: "
         ) ;_ fin de getkword
         "Si"
       ) ;_ fin de equal
        (setvar "SOLIDHIST" 1)
     ) ;_ fin de if
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 18.13. Funcin que solicita los datos para el eslabn.

(defun ax-caja (centro dim-x dim-y dim-z / res)
  (setq res (vl-catch-all-apply
              'vla-AddBox
              (list (espacio-actual *aesl:dibujo*)
                    (vlax-3d-point centro)
                    dim-x
                    dim-y
                    dim-z
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond ((vl-catch-all-error-p res)
         (prompt
           (strcat "\nERROR: " (vl-catch-all-error-message res))
         ) ;_ fin de prompt
        )
        (t
         (if (= (getvar "SOLIDHIST") 1)
           (vla-put-History res :vlax-true)
         ) ;_ fin de if
         res
        )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 18.14. Funcin ax-caja que crea un Slido 3D en forma de prisma rectangular.

(defun rot-180-z (obj / ang)
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos pi) (- (sin pi)) 0.0 0.0)
            (list (sin pi) (cos pi) 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 18.15. Funcin que gira un objeto 180 en torno al eje Z.

(defun C:ESLABON (/ origen centro dim-x r base hueco caja1 caja2)
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-eslabon)
  (setq base  (ax-cilindro (list r 0.0 0.0) r (* r 2))
        hueco (ax-cilindro (list r 0.0 0.0) (/ r 2.0) (* r 2))
        caja1 (ax-caja (list (- (* 0.5 r)) 0.0 0.0)
                       (* 3 r)
                       (* 2 r)
                       (* 2 r)
              ) ;_ fin de ax-caja
        caja2 (ax-caja (list (- r) 0.0 0.0) (* 2 r) r (* r 2))
  ) ;_ fin de setq
  (vla-Boolean base acUnion caja1)
  (vla-Boolean base acSubtraction caja2)
  (vla-Boolean base acSubtraction hueco)
  (setq copia (vla-Copy base))
  (rot-90-x copia)
  (rot-180-z copia)
  (vla-Boolean base acIntersection copia)
  (if mtrans
    (vla-TransformBy base mtrans)
  ) ;_ fin de if
  (ax-traslacion base (trans centro 1 0 t))
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 18.16. Funcin principal C:ESLABON.

(defun C:SOL-INTERFERENCIA
       (/ solido1 solido2 resultado propiedades)
  (vl-load-com)
  (setq solido1 (ssname (ssget "_:S" '((0 . "3DSOLID")))
                        0
                ) ;_ fin de ssname
        solido2 (ssname (ssget "_:S" '((0 . "3DSOLID")))
                        0
                ) ;_ fin de ssname
  ) ;_ fin de setq
  (cond
    ((and solido1 solido2)
     (setq solido1   (vlax-ename->vla-object solido1)
           solido2   (vlax-ename->vla-object solido2)
           resultado (vl-catch-all-apply
                       'vla-CheckInterference
                       (list solido1 solido2 :vlax-true)
                     ) ;_ fin de vl-catch-all-apply
     ) ;_ fin de setq
     (cond
       ((not resultado)
        (alert
          "Los objetos seleccionados no se intersecan"
        ) ;_ fin de alert
       )
       ((vl-catch-all-error-p resultado)
        (prompt (vl-catch-all-error-message resultado))
       )
       (t
        (vla-put-Color resultado acRed)
        (vla-Highlight resultado :vlax-true)
        (setq propiedades (ax-props resultado))
        (initget 1 "Si No")
        (if
          (=
            (getkword
              "\nConservar el slido de interferencia? [Si/No]: "
            ) ;_ fin de getkword
            "No"
          ) ;_ fin de =
           (vla-Delete resultado)
           (progn
             (vla-Highlight resultado :vlax-false)
             (vla-put-EntityTransparency solido1 75)
             (vla-put-EntityTransparency solido2 75)
             (setvar "VSFACEHIGHLIGHT" 50)
             (vla-update *aesl:acad*)
           ) ;_ fin de progn
        ) ;_ fin de if
        (alert
          (strcat
            "Propiedades de la interferencia:"
            (apply
              'strcat
              (mapcar
                '(lambda (p)
                   (strcat "\n"
                           (car p)
                           ": "
                           (vl-princ-to-string
                             (cdr p)
                           ) ;_ fin de vl-princ-to-string
                   ) ;_ fin de strcat
                 ) ;_ fin de lambda
                propiedades
              ) ;_ fin de mapcar
            ) ;_ fin de apply
          ) ;_ fin de strcat
        ) ;_ fin de alert
       )
     ) ;_ fin de cond
    )
    (t (prompt "\nDebe seleccionar dos Slidos 3D."))
  ) ;_ fin de cond
  (princ)
) ;_ fin de defun
;;;Listado 18.17. Programa de ejemplo para el mtodo CheckInterference.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
