;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 19.	Superficies de Subdivisin.

(defun datos-malla-poliedro (/)
  (initget 1 "Tetraedro Hexaedro Dodecaedro")
  (setq clase
         (getkword
           "\nPoliedro [Tetraedro/Hexaedro/Dodecaedro]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (initget 1)
  (setq centro (getpoint "\nCentro del poliedro: "))
  (initget (+ 1 2 4))
  (setq
    radio (getdist
            centro
            "\Radio de la esfera circunscrita: "
          ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 4))
  (while
    (> (setq nivel (getint "\nNivel de suavizado: "))
       (getvar "SMOOTHMESHMAXLEV")
    ) ;_ fin de >
     (prompt
       (strcat
         "\nEl nivel de suavizado no debe ser mayor que "
         (itoa (getvar "SMOOTHMESHMAXLEV"))
       ) ;_ fin de strcat
     ) ;_ fin de prompt
     (initget (+ 1 2 4))
  ) ;_ fin de while
  (initget 1 "Siempre Nunca 1 2 3")
  (setq pliegue
         (getkword
           "\nNivel pliegue de aristas [Siempre/Nunca/1/2/3]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (cond ((= pliegue "Siempre") (setq pliegue -1))
        ((= pliegue "Nunca") (setq pliegue 0))
        (t (setq pliegue (atof pliegue)))
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 19.1. Solicitud de datos para la Malla (MESH) en forma de Poliedro.

(defun aristas-cara (indices-caras / i v0 v1 v2 indices-arista)
  (foreach cara indices-caras
    (setq i  0
          v0 (nth i cara)
    ) ;_ fin de setq
    (repeat (1- (length cara))
      (setq v1 (nth i cara)
            i  (1+ i)
            v2 (nth i cara)
      ) ;_ fin de setq
      (if
        (not
          (or (member (list v1 v2) indices-arista)
              (member (list v2 v1) indices-arista)
          ) ;_ fin de or
        ) ;_ fin de not
         (setq indices-arista
                (cons (list v1 v2)
                      indices-arista
                ) ;_ fin de cons
         ) ;_ fin de setq
      ) ;_ fin de if
    ) ;_ fin de repeat
    (if
      (not (or (member (list v2 v0) indices-arista)
               (member (list v0 v2) indices-arista)
           ) ;_ fin de or
      ) ;_ fin de not
       (setq indices-arista
              (cons (list v2 v0)
                    indices-arista
              ) ;_ fin de cons
       ) ;_ fin de setq
    ) ;_ fin de if
  ) ;_ fin de foreach
  (apply 'append (reverse indices-arista))
) ;_ fin de defun
;;;Listado 19.2. Funcin que crea la lista de aristas para la malla.

(defun ent-mesh
       (vertices caras aristas nivel pliegue / res lista-ent)
  (setq res (cons (cons 91 nivel) res)
        res (cons (cons 92 (length vertices)) res)
  ) ;_ fin de setq
  (foreach vertice vertices
    (setq res (cons (cons 10 vertice) res))
  ) ;_ fin de foreach
  (setq res
         (cons
           (cons 93
                 (+ (length caras)
                    (length (apply 'append caras))
                 ) ;_ fin de +
           ) ;_ fin de cons
           res
         ) ;_ fin de cons
  ) ;_ fin de setq
  (foreach cara caras
    (setq res (cons (cons 90 (length cara)) res))
    (foreach dato cara
      (setq res (cons (cons 90 dato) res))
    ) ;_ fin de foreach
  ) ;_ fin de foreach
  (setq res
         (cons (cons 94 (/ (length aristas) 2)) res)
  ) ;_ fin de setq
  (foreach extremo aristas
    (setq res (cons (cons 90 extremo) res))
  ) ;_ fin de foreach
  (setq res
         (cons (cons 95 (/ (length aristas) 2)) res)
  ) ;_ fin de setq
  (repeat (/ (length aristas) 2)
    (setq res (cons (cons 140 pliegue) res))
  ) ;_ fin de repeat
  (setq lista-ent (append (list
                            '(0
                              .
                              "MESH"
                             ) '(100
                                 .
                                 "AcDbEntity"
                                ) '(100
                                    .
                                    "AcDbSubDMesh"
                                   ) '(71 . 2) '(72 . 0)
                           ) ;_ fin de list
 ;_ fin de list
                          (reverse res)
                  ) ;_ fin de append
  ) ;_ fin de setq
  (if (entmake lista-ent)
    (vlax-ename->vla-object (entlast))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.3. Creacin de MESH mediante entmake.

(defun C:POLIEDRO-MALLA (/ mtrans clase centro radio nivel vertices
                         caras aristas pliegue obj
                        )
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-malla-poliedro)
  (op-poliedro clase)
  (setq caras (mapcar '(lambda (cara) (mapcar '1- cara)) caras))
  (setq aristas (aristas-cara caras))
  (setq obj (ent-mesh vertices caras aristas nivel pliegue))
  (if obj
    (progn (ax-escala obj (list radio radio radio))
           (if mtrans
             (vla-TransformBy obj mtrans)
           ) ;_ fin de if
           (ax-traslacion obj (trans centro 1 0 t))
           (ax-SWs)
    ) ;_ fin de progn
  ) ;_ fin de if
  (vla-EndUndoMark *aesl:dibujo*)
  (princ)
) ;_ fin de defun
;;;Listado 19.4. Funcin principal C:POLIEDRO-MALLA.

(defun datos-malla-subd (/ nombre str n-caras)
  (initget 1)
  (setq nombre
         (getstring
           "\nFuncin a emplear para la malla: "
         ) ;_ fin de getstring
  ) ;_ fin de setq
  (while (not (setq ecuacion
                     (car (atoms-family 0 (list nombre)))
              ) ;_ fin de setq
         ) ;_ fin de not
    (prompt
      (strcat
        "\nLa funcin "
        nombre
        " no se ha definido en el contexto actual."
      ) ;_ fin de strcat
    ) ;_ fin de prompt
    (initget 1)
    (setq nombre
           (getstring
             "\nIndique funcin a emplear para la malla: "
           ) ;_ fin de getstring
    ) ;_ fin de setq
  ) ;_ fin de while
  (initget 1)
  (setq centro (getpoint "\nCentro de la malla: "))
  (initget (+ 1 2 4))
  (setq dim-x
         (getdist centro
                  "\nDimensin en X de la malla: "
         ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq dim-y
         (getdist centro
                  "\nDimensin en Y de la malla: "
         ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq dim-z
         (getdist centro
                  "\nDimensin en Z de la malla: "
         ) ;_ fin de getdist
  ) ;_ fin de setq
  (if (< dim-x dim-y)
    (setq str "X")
    (setq str "Y")
  ) ;_ fin de if
  (initget (+ 1 2 4))
  (setq n-caras
         (getint
           (strcat
             "\nCantidad de caras en la dimensin "
             str
             ": "
           ) ;_ fin de strcat
         ) ;_ fin de getint
  ) ;_ fin de setq
  (if (> dim-x dim-y)
    (setq filas    n-caras
          columnas (fix (/ dim-x (/ dim-y n-caras)))
    ) ;_ fin de setq
    (setq filas    (fix (/ dim-y (/ dim-x n-caras)))
          columnas n-caras
    ) ;_ fin de setq
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.5. Funcin que solicita los datos para una malla de subdivisin.

(defun calc-vertices-malla (dim-x dim-y ecuacion dim-z filas
                            columnas / xmin ymin dx dy cara caras
                            x0 y0
                           )
  (setq xmin (- (/ dim-x 2))
        ymin (- (/ dim-y 2))
        dx   (/ dim-x columnas)
        dy   (/ dim-y filas)
  ) ;_ fin de setq
  (setq x0 xmin
        y0 ymin
  ) ;_ fin de setq
  (repeat columnas
    (setq y0 ymin)
    (repeat filas
      (setq
        cara (list (list xmin y0)
                   (list (+ xmin dx) y0)
                   (list (+ xmin dx) (+ y0 dy))
                   (list xmin (+ y0 dy))
             ) ;_ fin de list
      ) ;_ fin de setq
      (setq caras (cons cara caras)
            y0    (+ y0 dy)
      ) ;_ fin de setq
    ) ;_ fin de repeat
    (setq xmin (+ xmin dx))
  ) ;_ fin de repeat
  (coord-z (reverse caras) ecuacion dim-z)
) ;_ fin de defun
;;;Listado 19.6. Funcin que calcula las coordenadas de los vrtices para cada cara.

(defun coord-z
               (caras ecuacion dim-z / lst-z altura f-esc)
  (setq lst-z  (mapcar
                 '(lambda (cara)
                    (mapcar '(lambda (pto)
                               (apply ecuacion pto)
                             ) ;_ fin de lambda
                            cara
                    ) ;_ fin de mapcar
                  ) ;_ fin de lambda
                 caras
               ) ;_ fin de mapcar
        altura (- (apply 'max (apply 'append lst-z))
                  (apply 'min (apply 'append lst-z))
               ) ;_ fin de -
        f-esc  (/ dim-z altura)
        lst-z  (mapcar
                 '(lambda (lst-cara)
                    (mapcar '(lambda (z) (* z f-esc))
                            lst-cara
                    ) ;_ fin de mapcar
                  ) ;_ fin de lambda
                 lst-z
               ) ;_ fin de mapcar
  ) ;_ fin de setq
  (mapcar
    '(lambda (cara elev)
       (mapcar
         '(lambda (pto z) (append pto (list z)))
         cara
         elev
       ) ;_ fin de mapcar
     ) ;_ fin de lambda
    caras
    lst-z
  ) ;_ fin de mapcar
) ;_ fin de defun
;;;Listado 19.7. Funcin que calcula los valores de la coordenada Z.

(defun lista-vertices-unicos (caras / lista-vertices)
  (foreach cara caras
    (foreach vertice cara
      (if (not (member vertice lista-vertices))
        (setq lista-vertices (cons vertice lista-vertices))
      ) ;_ fin de if
    ) ;_ fin de foreach
  ) ;_ fin de foreach
  (reverse lista-vertices)
) ;_ fin de defun
;;;Listado 19.8. Funcin que genera la lista de vrtices sin duplicados.

(defun lista-indices-caras
       (caras vertices-unicos / indices indices-caras)
  (foreach cara caras
    (foreach vertice cara
      (setq indices
             (cons (vl-position vertice vertices-unicos) indices)
      ) ;_ fin de setq
    ) ;_ fin de foreach
    (setq indices-caras
           (cons indices indices-caras)
          indices nil
    ) ;_ fin de setq
  ) ;_ fin de foreach
  (reverse indices-caras)
) ;_ fin de defun
;;;Listado 19.9. Funcin que crea la lista de ndices de caras.


(defun lista-indices-aristas (indices-caras / i extremo-inicial
                              extremo-1 extremo-2 indices-arista
                             )
  (foreach cara indices-caras
    (setq i 0
          extremo-inicial
           (nth i cara)
    ) ;_ fin de setq
    (repeat (1- (length cara))
      (setq extremo-1 (nth i cara)
            i         (1+ i)
            extremo-2 (nth i cara)
      ) ;_ fin de setq
      (if
        (not
          (or (member (list extremo-1 extremo-2)
                      indices-arista
              ) ;_ fin de member
              (member (list extremo-2 extremo-1)
                      indices-arista
              ) ;_ fin de member
          ) ;_ fin de or
        ) ;_ fin de not
         (setq indices-arista
                (cons (list extremo-1
                            extremo-2
                      ) ;_ fin de list
                      indices-arista
                ) ;_ fin de cons
         ) ;_ fin de setq
      ) ;_ fin de if
    ) ;_ fin de repeat
    (if
      (not
        (or
          (member (list extremo-2 extremo-inicial)
                  indices-arista
          ) ;_ fin de member
          (member (list extremo-inicial extremo-2)
                  indices-arista
          ) ;_ fin de member
        ) ;_ fin de or
      ) ;_ fin de not
       (setq indices-arista
              (cons (list extremo-2
                          extremo-inicial
                    ) ;_ fin de list
                    indices-arista
              ) ;_ fin de cons
       ) ;_ fin de setq
    ) ;_ fin de if
  ) ;_ fin de foreach
(apply 'append (reverse indices-arista))) ;_ fin de defun
;;;Listado 19.11. Funcin que genera la lista de ndices de arista.

(defun subd-malla-rectangular
       (coord-vertices-caras / vertices caras aristas obj)
  (setq vertices (lista-vertices-unicos coord-vertices-caras)
        caras    (lista-indices-caras coord-vertices-caras vertices)
        aristas  (lista-indices-aristas caras)
        obj      (ent-mesh vertices caras aristas 0 -1)
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 19.12. Funcin que genera la estructura de datos y crea la malla.

(defun C:MALLA-SUBD (/ mtrans ecuacion centro dim-x dim-y dim-z filas columnas
                     n-caras coord-vertices obj
                    )
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-malla-subd)
  (setq coord-vertices (calc-vertices-malla
                         dim-x dim-y ecuacion dim-z filas columnas
                        ) ;_ fin de calc-vertices-malla
 ;_ fin de calc-vertices-malla
        obj            (subd-malla-rectangular coord-vertices)
  ) ;_ fin de setq
  (cond
    (obj
     (if mtrans
       (vla-TransformBy obj mtrans)
     ) ;_ fin de if
     (ax-traslacion obj (trans centro 1 0 t))
     (ax-SWs)
    )
    (t
     (prompt "\nError en la construccin de la malla.")
    )
  ) ;_ fin de cond
  (vla-EndUndoMark *aesl:dibujo*)
  (princ)
) ;_ fin de defun
;;;Listado 19.13. Funcin principal C:MALLA-SUBD.

(defun cmd-mesh-caja (centro n-div dim-x dim-y dim-z esquina / dims
                      d-inc divs e-fin obj
                     )
  (setq dims  (list dim-x dim-y dim-z)
        d-inc (/ (apply 'max (list dim-x dim-y dim-z))
                 n-div
              ) ;_ fin de /
        divs  (mapcar '(lambda (dim) (fix (/ dim d-inc)))
                      dims
              ) ;_ fin de mapcar
        divs  (mapcar '(lambda (div)
                         (if (< div 1)
                           1
                           div
                         ) ;_ fin de if
                       ) ;_ fin de lambda
                      divs
              ) ;_ fin de mapcar
  ) ;_ fin de setq
  (setvar "DIVMESHBOXLENGTH" (nth 0 divs))
  (setvar "DIVMESHBOXWIDTH" (nth 1 divs))
  (setvar "DIVMESHBOXHEIGHT" (nth 2 divs))
  (setvar "CMDECHO" 0)
  (if esquina
    (setq e-fin (mapcar '+ centro (list dim-x dim-y 0))
          obj   (vl-cmdf "._MESH" "_B" centro e-fin dim-z
                        ) ;_ fin de vl-cmdf
 ;_ fin de vl-cmdf
    ) ;_ fin de setq
    (setq obj (vl-cmdf "._MESH" "_B" "_C" centro "_L" dim-x dim-y
                       dim-z
                      ) ;_ fin de vl-cmdf
 ;_ fin de vl-cmdf
    ) ;_ fin de setq
  ) ;_ fin de if
  (setvar "CMDECHO" 1)
  (if obj
    (entlast)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.14. Creacin de una malla en forma de prisma rectangular.

(defun long-elipse (dim-x dim-y /)
  (*
    pi
    (+ dim-x dim-y)
    (+
      1
      (/
        (* 3.0 (expt (/ (- dim-x dim-y) (+ dim-x dim-y)) 2))
        (+
          10.0
          (sqrt
            (- 4.0
               (* 3.0
                  (expt (/ (- dim-x dim-y) (+ dim-x dim-y))
                        2.0
                  ) ;_ fin de expt
               ) ;_ fin de *
            ) ;_ fin de -
          ) ;_ fin de sqrt
        ) ;_ fin de +
      ) ;_ fin de /
    ) ;_ fin de +
  ) ;_ fin de *
) ;_ fin de defun
;;;Listado 19.15. Funcin que calcula la circunferencia de una elipse.

(defun cmd-mesh-cono (centro n-div dim-x dim-y dim-z / r-max dims
                      d-inc divs obj
                     )
  (setq r-max (apply 'max (list dim-x dim-y))
        dims  (list (long-elipse dim-x dim-y) r-max dim-z)
        d-inc (/ (apply 'max dims) n-div)
        divs  (mapcar '(lambda (dim) (fix (/ dim d-inc)))
                      dims
              ) ;_ fin de mapcar
        divs  (mapcar '(lambda (div)
                         (if (< div 1)
                           1
                           div
                         ) ;_ fin de if
                       ) ;_ fin de lambda
                      divs
              ) ;_ fin de mapcar
  ) ;_ fin de setq
  (setvar "DIVMESHCONEAXIS" (nth 0 divs))
  (setvar "DIVMESHCONEBASE" (nth 1 divs))
  (setvar "DIVMESHCONEHEIGHT" (nth 2 divs))
  (setvar "CMDECHO" 0)
  (setq
    obj (vl-cmdf "_MESH"
                 "_C"
                 "_E"
                 "_C"
                 centro
                 dim-x
                 (list (nth 0 centro)
                       (+ (nth 1 centro) dim-y)
                       (nth 2 centro)
                 ) ;_ fin de list
                 dim-z
        ) ;_ fin de vl-cmdf
  ) ;_ fin de setq
  (setvar "CMDECHO" 1)
  (if obj
    (entlast)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.16. Funcin que crea una malla en forma  de cono.

(defun cmd-mesh-cilindro
                         (centro n-div dim-x dim-y dim-z / r-max
                          dims d-inc divs obj
                         )
  (setq r-max (apply 'max (list dim-x dim-y))
        dims  (list (long-elipse dim-x dim-y) r-max dim-z)
        d-inc (/ (apply 'max dims) n-div)
        divs  (mapcar '(lambda (dim) (fix (/ dim d-inc))) dims)
        divs  (mapcar '(lambda (div)
                         (if (< div 1)
                           1
                           div
                         ) ;_ fin de if
                       ) ;_ fin de lambda
                      divs
              ) ;_ fin de mapcar
  ) ;_ fin de setq
  (setvar "DIVMESHCYLAXIS" (nth 0 divs)) ;division en torno al eje
  (setvar "DIVMESHCYLBASE" (nth 1 divs))
  (setvar "DIVMESHCYLHEIGHT" (nth 2 divs))
  (setvar "CMDECHO" 0)
  (setq obj (vl-cmdf "._MESH"
                     "_CY"
                     "_E"
                     "_C"
                     centro
                     dim-x
                     (list (nth 0 centro)
                           (+ (nth 1 centro) dim-y)
                           (nth 2 centro)
                     ) ;_ fin de list
                     (list (nth 0 centro)
                           (nth 1 centro)
                           (+ (nth 2 centro) dim-z)
                     ) ;_ fin de list
            ) ;_ fin de vl-cmdf
  ) ;_ fin de setq
  (setvar "CMDECHO" 1)
  (if obj
    (entlast)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.17. Funcin que crea una malla en forma de cilindro.

(defun cmd-mesh-esfera (centro n-div radio / dims d-inc divs obj)
  (setq dims  (list (* 2 pi radio) (* pi radio))
        d-inc (/ (car dims) n-div)
        divs  (mapcar '(lambda (dim) (fix (/ dim d-inc))) dims)
  ) ;_ fin de setq
  (if (> (nth 0 divs) 2)
    (setvar "DIVMESHSPHEREAXIS" (nth 0 divs))
    (setvar "DIVMESHSPHEREAXIS" 3)
  ) ;_ fin de if
  (if (> (nth 1 divs) 1)
    (setvar "DIVMESHSPHEREHEIGHT" (nth 1 divs))
    (setvar "DIVMESHSPHEREHEIGHT" 2)
  ) ;_ fin de if
  (setvar "CMDECHO" 0)
  (setq obj (vl-cmdf "._MESH" "_S" centro radio))
  (setvar "CMDECHO" 1)
  (if obj
    (entlast)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.18. Funcin que crea una malla en forma de esfera.

(defun entmod-mesh (ename ecuacion dim-z / lista-ent vertices lst-z
                    altura f-esc x y z vertices-mod nueva-lista
                   )
  (setq lista-ent (entget ename)
        vertices  (valores 10 lista-ent)
        lst-z     (mapcar
                    '(lambda (vert)
                       (apply
                         ecuacion
                         (list (nth 0 vert) (nth 1 vert))
                       ) ;_ fin de apply
                     ) ;_ fin de lambda
                    vertices
                  ) ;_ fin de mapcar
        altura    (- (apply 'max lst-z) (apply 'min lst-z))
        f-esc     (/ dim-z altura)
  ) ;_ fin de setq
  (foreach vertice vertices
    (setq x            (nth 0 vertice)
          y            (nth 1 vertice)
          z            (+ (nth 2 vertice)
                          (* f-esc (apply ecuacion (list x y)))
                       ) ;_ fin de +
          vertices-mod (cons (list x y z) vertices-mod)
    ) ;_ fin de setq
  ) ;_ fin de foreach
  (setq vertices-mod (reverse vertices-mod))
  (setq i 0
        nueva-lista
         lista-ent
  ) ;_ fin de setq
  (while (< i (length vertices))
    (setq nueva-lista
           (subst
             (cons 10 (nth i vertices-mod))
             (cons 10 (nth i vertices))
             nueva-lista
           ) ;_ fin de subst
          i (1+ i)
    ) ;_ fin de setq
  ) ;_ fin de while
  (entmod nueva-lista)
  (entupd ename)
) ;_ fin de defun
;;;Listado 19.19. Modificacin de una entidad MESH mediante entmod.

(defun ax-mod-mesh (obj ecuacion dim-z / nivel xyz lst-z i pt)
  (setq nivel (vla-get-Smoothness obj))
  (if (> nivel 0)
    (vla-put-Smoothness obj 0)
  ) ;_ fin de if
  (setq xyz   (vlax-safearray->list
                (vlax-variant-value (vla-get-Coordinates obj))
              ) ;_ fin de vlax-safearray->list
        lst-z (cal-z xyz ecuacion dim-z)
        i     0
  ) ;_ fin de setq
  (repeat (vla-get-VertexCount obj)
    (setq
      pt (vlax-safearray->list
           (vlax-variant-value (vla-get-Coordinate obj i))
         ) ;_ fin de vlax-safearray->list
    ) ;_ fin de setq
    (vla-put-Coordinate
      obj
      i
      (vlax-3d-point
        (list (nth 0 pt)
              (nth 1 pt)
              (+ (nth 2 pt) (nth i lst-z))
        ) ;_ fin de list
      ) ;_ fin de vlax-3d-point
    ) ;_ fin de vla-put-Coordinate
    (setq i (1+ i))
  ) ;_ fin de repeat
  (if (> nivel 0)
    (vla-put-Smoothness obj nivel)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.20. Modificacin de un objeto MESH mediante ActiveX.

(defun par-rev (x y /)
  (- (+ (expt x 2.0) (expt y 2.0)))
) ;_ fin de defun
;;;Listado 19.21. Funcin para el clculo de los vrtices de un paraboloide de revolucin.

(defun par-hip (x y) (- (expt x 2) (expt y 2)))
;;;Listado 19.22. Funcin para el clculo de los vrtices de un paraboloide hiperblico.

(defun SCP->U (/ tmp)
  (setq tmp (ax-scp "Univ"
                    '(0.0 0.0 0.0)
                    '(1.0 0.0 0.0)
                    '(0.0 1.0 0.0)
            ) ;_ fin de ax-scp
  ) ;_ fin de setq
  (vla-put-ActiveUCS *aesl:dibujo* tmp)
) ;_ fin de defun
;;;Listado 19.23. Funcin que establece como actual el sistema de coordenadas universal.

(defun datos-paraboloide (/)
  (initget 1 "Entmod Activex")
  (setq proc
         (getkword "Procedimiento: [Entmod/Activex]: ")
  ) ;_ fin de setq
  (initget 1 "Revolucin Hiperblico")
  (setq opcion
         (getkword
           "\nTipo Paraboloide [Revolucin/Hiperblico]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (cond
    ((= opcion "Revolucin") (setq opcion 'par-rev))
    ((= opcion "Hiperblico") (setq opcion 'par-hip))
  ) ;_ fin de cond
  (initget 1 "Rectngulo Crculo Elipse eSfera")
  (setq forma
         (getkword
           "\nForma [Rectngulo/Crculo/Elipse/eSfera]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (if (= forma "Rectngulo")
    (progn
      (initget "Centro Esquina")
      (if
        (not
          (setq origen
                 (getkword
                   "Origen Coordenadas [Centro/Esquina]<Centro>"
                 ) ;_ fin de getkword
          ) ;_ fin de setq
        ) ;_ fin de not
         (setq origen "Centro")
      ) ;_ fin de if
    ) ;_ fin de progn
  ) ;_ fin de if
  (initget 1)
  (setq posic   (getpoint "\nPosicin de la malla: ")
        z-malla (getdist posic "\nAltura de la malla: ")
  ) ;_ fin de setq
  (cond
    ((= forma "Rectngulo")
     (initget (+ 1 2 4))
     (setq dimX (getdist posic "\nDimensin en X: "))
     (initget (+ 1 2 4))
     (setq dimY (getdist posic "\nDimensin en Y: "))
     (initget (+ 1 2 4))
     (setq dimZ
            (getdist posic "\nGrosor de la malla: ")
     ) ;_ fin de setq
     (initget (+ 1 2 4))
     (setq prec (getint "\nResolucin de la malla: "))
     (while (not (< 1 prec 257))
       (prompt
         "\nEl valor de resolucin debe estar entre 2 y 256"
       ) ;_ fin de prompt
       (initget (+ 1 2 4))
       (setq prec
              (getint "\nResolucin de la malla: ")
       ) ;_ fin de setq
     ) ;_ fin de while
    )
    ((= forma "Crculo")
     (initget (+ 2 4))
     (if (not (setq prec
                     (getint
                       "\nSubdivisin permetro <20>: "
                     ) ;_ fin de getint
              ) ;_ fin de setq
         ) ;_ fin de not
       (setq prec 20)
     ) ;_ fin de if
     (initget (+ 1 2 4))
     (setq dimX (getdist posic "\nRadio de la malla: ")
           dimY dimX
     ) ;_ fin de setq
     (initget (+ 1 2 4))
     (setq dimZ
            (getdist posic "\nGrosor de la malla: ")
     ) ;_ fin de setq
    )
    ((= forma "Elipse")
     (initget (+ 2 4))
     (if (not (setq prec
                     (getint
                       "\nSubdivisin permetro <20>: "
                     ) ;_ fin de getint
              ) ;_ fin de setq
         ) ;_ fin de not
       (setq prec 20)
     ) ;_ fin de if
     (initget (+ 1 2 4))
     (setq dimX
            (getdist posic
                     "\nSemieje en X de la malla: "
            ) ;_ fin de getdist
     ) ;_ fin de setq
     (initget (+ 1 2 4))
     (setq dimY
            (getdist posic
                     "\nSemieje en Y de la malla: "
            ) ;_ fin de getdist
     ) ;_ fin de setq
     (initget (+ 1 2 4))
     (setq dimZ
            (getdist posic "\nGrosor de la malla: ")
     ) ;_ fin de setq
    )
    ((= forma "eSfera")
     (initget (+ 2 4))
     (if (not (setq prec
                     (getint
                       "\nSubdivisin ecuador <20>: "
                     ) ;_ fin de getint
              ) ;_ fin de setq
         ) ;_ fin de not
       (setq prec 20)
     ) ;_ fin de if
     (initget (+ 1 2 4))
     (setq dimX
            (getdist posic "\nRadio de la esfera: ")
     ) ;_ fin de setq
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 19.24. Funcin que solicita los datos para el paraboloide.

(defun C:PARABOLOIDE (/ *error* proc opcion forma origen posic
                      z-malla dimX dimY dimZ prec obj
                     )
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq scact  (ax-matriz-scp)
               mtrans (last scact)
         ) ;_ fin de setq
         (scp->u)
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (datos-paraboloide)
  (cmd-entrar)
  (cond ((= forma "Rectngulo")
         (if (= origen "Centro")
           (cmd-mesh-caja '(0 0 0) prec dimX dimY dimZ nil)
           (cmd-mesh-caja '(0 0 0) prec dimX dimY dimZ t)
         ) ;_ fin de if
        )
        ((= forma "Crculo")
         (cmd-mesh-cilindro '(0 0 0) prec dimX dimY dimZ)
        )
        ((= forma "Elipse")
         (cmd-mesh-cilindro '(0 0 0) prec dimX dimY dimZ)
        )
        ((= forma "eSfera")
         (cmd-mesh-esfera '(0 0 0) prec dimX)
        )
  ) ;_ fin de cond
  (setq obj (vlax-ename->vla-object (entlast)))
  (cmd-salir)
  (if (= (vla-get-ObjectName obj) "AcDbSubDMesh")
    (progn
      (ax-mod-mesh obj opcion z-malla)
      (if mtrans
        (progn (vla-put-ActiveUCS
                 *aesl:dibujo*
                 (vla-item
                   (vla-get-UserCoordinateSystems *aesl:dibujo*)
                   (car scact)
                 ) ;_ fin de vla-item
               ) ;_ fin de vla-put-ActiveUCS
               (vla-TransformBy obj mtrans)
        ) ;_ fin de progn
      ) ;_ fin de if
      (ax-traslacion obj posic)
      (ax-SWs)
    ) ;_ fin de progn
  ) ;_ fin de if
  (vla-EndUndoMark *aesl:dibujo*)
  (princ)
) ;_ fin de defun
;;;Listado 19.25. Funcin principal C:PARABOLOIDE.

(defun entmod-mesh-xyz (ename ecuacion dim-max eje / lista-ent
                        vertices i j k lst-mod altura f-esc v-i v-j
                        v-mod vertices-mod
                       )
  (setq lista-ent (entget ename)
        vertices  (valores 10 lista-ent)
  ) ;_ fin de setq
  (cond ((= eje 0)
         (setq i 1
               j 2
         ) ;_ fin de setq
        )
        ((= eje 1)
         (setq i 0
               j 2
         ) ;_ fin de setq
        )
        ((= eje 2)
         (setq i 0
               j 1
         ) ;_ fin de setq
        )
  ) ;_ fin de cond
  (setq lst-mod (mapcar '(lambda (vert)
                           (apply ecuacion
                                  (list (nth i vert)
                                        (nth j vert)
                                  ) ;_ fin de list
                           ) ;_ fin de apply
                         ) ;_ fin de lambda
                        vertices
                ) ;_ fin de mapcar
        altura  (- (apply 'max lst-mod)
                   (apply 'min lst-mod)
                ) ;_ fin de -
        f-esc   (/ dim-max altura)
  ) ;_ fin de setq
  (foreach vertice vertices
    (setq v-i   (nth i vertice)
          v-j   (nth j vertice)
          v-mod (+ (nth eje vertice)
                   (* f-esc
                      (apply ecuacion (list v-i v-j))
                   ) ;_ fin de *
                ) ;_ fin de +
    ) ;_ fin de setq
    (cond ((= eje 0)
           (setq vertices-mod
                  (cons (list v-mod v-i v-j)
                        vertices-mod
                  ) ;_ fin de cons
           ) ;_ fin de setq
          )
          ((= eje 1)
           (setq vertices-mod
                  (cons (list v-i v-mod v-j)
                        vertices-mod
                  ) ;_ fin de cons
           ) ;_ fin de setq
          )
          ((= eje 2)
           (setq vertices-mod
                  (cons (list v-i v-j v-mod)
                        vertices-mod
                  ) ;_ fin de cons
           ) ;_ fin de setq
          )
    ) ;_ fin de cond
  ) ;_ fin de foreach
  (setq vertices-mod (reverse vertices-mod))
  (setq k 0
        nueva-lista
         lista-ent
  ) ;_ fin de setq
  (while (< k (length vertices))
    (setq nueva-lista
           (subst
             (cons 10 (nth k vertices-mod))
             (cons 10 (nth k vertices))
             nueva-lista
           ) ;_ fin de subst
          k (1+ k)
    ) ;_ fin de setq
  ) ;_ fin de while
  (entmod nueva-lista)
  (entupd ename)
) ;_ fin de defun
;;;Listado 19.26. Funcin que deforma la malla en las direcciones X, Y o Z.

(defun C:DEFORMA (/ ent eje ecuacion var-dim)
  (prompt "\nDesigne objeto de malla a deformar: ")
  (while
    (not (setq ent (ssget "_:S" '((0 . "MESH")))))
     (prompt "\nDesigne objeto de malla a deformar: ")
  ) ;_ fin de while
  (initget (+ 2 4) "X Y Z")
  (setq eje
         (getkword
           "\nIndique eje para la transformacin [X/Y/Z]<Z>"
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (cond ((= eje "X") (setq eje 0))
        ((= eje "Y") (setq eje 1))
        (t (setq eje 2))
  ) ;_ fin de cond
  (setq ent (ssname ent 0))
  (initget 1)
  (setq ecuacion
         (getstring
           "\nIndique funcin a emplear para la deformacin: "
         ) ;_ fin de getstring
  ) ;_ fin de setq
  (cond
    ((car (atoms-family 0 (list ecuacion)))
     (initget (+ 1 2 4))
     (setq var-dim (getdist "\nAltura de la malla: "))
     (entmod-mesh-xyz ent (read ecuacion) var-dim eje)
    )
    (t
     (prompt
       (strcat
         "\nLa funcin "
         ecuacion
         " no se ha definido en el contexto actual."
       ) ;_ fin de strcat
     ) ;_ fin de prompt
    )
  ) ;_ fin de cond
  (princ)
) ;_ fin de defun
;;;Listado 19.27. Funcin principal C:DEFORMA.

(defun datos-hiperb (/)
  (initget 1)
  (setq cen-base (getpoint "\nCentro de la base: "))
  (initget (+ 1 2 4))
  (setq rad-base (getdist cen-base
                          "\nRadio de la base: "
                 ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq
    altura (getdist cen-base
                    "\nAltura del hiperboloide: "
           ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget 1)
  (setq cen-sup
         (getpoint
           "\nCentro de la circunferencia superior: "
         ) ;_ fin de getpoint
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq rad-sup
         (getdist
           cen-sup
           "\nRadio de la circunferencia superior: "
         ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (while
    (not (< 0.0
            (setq ang (getreal "\nngulo de giro: "))
            180.0
         ) ;_ fin de <
    ) ;_ fin de not
     (prompt
       "\nngulo debe ser mayor que 0 y menor que 180"
     ) ;_ fin de prompt
     (initget (+ 1 2 4))
  ) ;_ fin de while
  (setq ang (gar ang))
  (initget (+ 2 4))
  (if (not (setq n
                  (getint
                    "\nDivisiones de circunferencia <32>: "
                  ) ;_ fin de getint
           ) ;_ fin de setq
      ) ;_ fin de not
    (setq n 32)
  ) ;_ fin de if
  (initget (+ 2 4))
  (if
    (not
      (setq k (getint "\nNivel de suavizado <2>: "))
    ) ;_ fin de not
     (setq k 2)
  ) ;_ fin de if
  (setvar "SURFTAB1" n)
  (setvar "MESHTYPE" 1)
) ;_ fin de defun
;;;Listado 19.28. Funcin que solicita los datos para el hiperboloide.

(defun ent-polcirc (elev radio capa)
  (entmake
    (list '(0 . "LWPOLYLINE")
          '(100 . "AcDbEntity")
          (cons 8 capa)
          '(100 . "AcDbPolyline")
          '(90 . 2)
          '(70 . 1)
          (cons 38 elev)
          (cons 10 (list (- radio) 0.0))
          '(42 . 1.0)
          '(91 . 0)
          (cons 10 (list radio 0.0))
          '(42 . 1.0)
          '(91 . 0)
    ) ;_ fin de list
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 19.29. Funcin que crea los perfiles circulares.

(defun ax-tras-rot (obj vector ang)
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos ang) (- (sin ang)) 0.0 (nth 0 vector))
            (list (sin ang) (cos ang) 0.0 (nth 1 vector))
            (list 0.0 0.0 1.0 (nth 2 vector))
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 19.30. Funcin que traslada y gira un perfil.

(defun C:HIPERBOLOIDE (/ cen-base rad-base altura cen-sup rad-sup
                       ang perf-base perf-sup col-capas hiper
                      )
  (datos-hiperb)
  (cond
    ((ent-polcirc 0.0 rad-base "PERFILES")
     (setq perf-base (entlast))
     (ax-tras-rot
       (vlax-ename->vla-object perf-base)
       cen-base
       0.0
     ) ;_ fin de ax-tras-rot
    )
    (t
     (prompt "\nERROR en la creacin del perfil base.")
     (exit)
    )
  ) ;_ fin de cond
  (cond
    ((ent-polcirc altura rad-sup "PERFILES")
     (setq perf-sup (entlast))
     (ax-tras-rot
       (vlax-ename->vla-object perf-sup)
       cen-sup
       ang
     ) ;_ fin de ax-tras-rot
    )
    (t
     (prompt
       "\nERROR en la creacin del perfil superior."
     ) ;_ fin de prompt
     (exit)
    )
  ) ;_ fin de cond
  (setq col-capas (vla-get-layers *aesl:dibujo*))
  (vla-put-ActiveLayer
    *aesl:dibujo*
    (ax-capa col-capas "SUPERFICIE" "4" "Continuous")
  ) ;_ fin de vla-put-ActiveLayer
  (if (vl-cmdf "_rulesurf" perf-base perf-sup)
    (progn (setq hiper (entlast))
           (vla-put-smoothness
             (vlax-ename->vla-object hiper)
             k
           ) ;_ fin de vla-put-smoothness
           (ax-SWs)
    ) ;_ fin de progn
    (prompt "\nError en la creacin del hiperboloide.")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 19.31. Funcin principal C:HIPERBOLOIDE.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
