;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) 2012 Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 20.	Superficies de Procedimiento y NURBS.

(defun datos-nurbs (/ nombre)
  (initget 1)
  (setq nombre
         (getstring
           "\nFuncin para calcular la superficie: "
         ) ;_ fin de getstring
  ) ;_ fin de setq
  (while (not (setq ecuacion
                     (car (atoms-family 0 (list nombre)))
              ) ;_ fin de setq
         ) ;_ fin de not
    (prompt
      (strcat "\nLa funcin "
              nombre
              " no se ha definido en el contexto actual."
      ) ;_ fin de strcat
    ) ;_ fin de prompt
    (initget 1)
    (setq nombre
           (getstring
             "\nFuncin para calcular la superficie: "
           ) ;_ fin de getstring
    ) ;_ fin de setq
  ) ;_ fin de while
  (initget 1)
  (setq centro (getpoint "\nCentro de la superficie: "))
  (initget (+ 1 2 4))
  (setq
    dim-x (getdist
            centro
            "\nDimensin en X de la superficie: "
          ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq
    dim-y (getdist
            centro
            "\nDimensin en Y de la superficie: "
          ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq
    dim-z (getdist
            centro
            "\nDimensin en Z de la superficie: "
          ) ;_ fin de getdist
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq n-sec
         (getint
           (strcat
             "\nSecciones transversales en la direccin Y: "
           ) ;_ fin de strcat
         ) ;_ fin de getint
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq n-pts
         (getint
           (strcat
             "\nNmero de puntos de ajuste para cada seccin: "
           ) ;_ fin de strcat
         ) ;_ fin de getint
  ) ;_ fin de setq
  (initget 1 "Cuerda Raz Uniforme")
  (setq param
         (getkword
           "\nParametrizacin nudos [Cuerda/Raz cuadrada/Uniforme]: "
         ) ;_ fin de getkword
  ) ;_ fin de setq
  (cond
    ((= param "Cuerda") (setq param (+ 8 32 1024)))
    ((= param "Raz") (setq param (+ 8 64 1024)))
    ((= param "Uniforme")
     (setq param (+ 8 128 1024))
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 20.1. Solicitud de datos al usuario para la superficie NURBS por SOLEVADO.

(defun calc-secc (dim-x dim-y dim-z ecuacion n-sec n-pts / d-sec
                  d-pts x0 y0 seccion secciones
                 )
  (setq xmin  (- (/ dim-x 2))
        ymin  (- (/ dim-y 2))
        d-sec (/ dim-y (1- n-sec))
        d-pts (/ dim-x (1- n-pts))
        x0    xmin
        y0    ymin
  ) ;_ fin de setq
  (repeat n-sec
    (setq x0 xmin
          seccion nil
    ) ;_ fin de setq
    (repeat n-pts
      (setq seccion (cons (list x0 y0) seccion))
      (setq x0 (+ x0 d-pts))
    ) ;_ fin de repeat
    (setq secciones (cons (reverse seccion) secciones))
    (setq y0 (+ y0 d-sec))
  ) ;_ fin de repeat
  (coord-z (reverse secciones) ecuacion dim-z)
) ;_ fin de defun
;;;Listado 20.2. Funcin que calcula las coordenadas de los puntos de ajuste.

(defun ent-secc (lista-puntos cod70 vec-normal)
  (entmake
    (append (list '(0 . "SPLINE")
                  '(100 . "AcDbEntity")
                  '(100 . "AcDbSpline")
                  (cons 70 cod70)
                  '(71 . 3)
                  (cons 74 (length lista-puntos))
                  (cons 210 vec-normal)
            ) ;_ fin de list
            (mapcar '(lambda (x) (cons 11 x))
                    lista-puntos
            ) ;_ fin de mapcar
    ) ;_ fin de append
  ) ;_ fin de entmake
) ;_ fin de defun
;;;Listado 20.3. Funcin que crea la seccin transversal como SPLINE.

(defun C:SUP-NURBS (/ *error* mtrans dim-x dim-y ecuacion dim-z
                    n-sec n-pts param secciones c-secc obj i xmin
                    ymin
                   )
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cond ((= (getvar "WORLDUCS") 0)
         (setq mtrans (last (ax-matriz-scp)))
        )
        (t (setq mtrans nil))
  ) ;_ fin de cond
  (setvar "SURFACEMODELINGMODE" 1)
  (datos-nurbs)
  (setq secciones (calc-secc dim-x dim-y dim-z ecuacion n-sec n-pts
                            ) ;_ fin de calc-secc
        c-secc    (ssadd)
  ) ;_ fin de setq
  (foreach secc secciones
    (if (ent-secc secc param '(0.0 -1.0 0.0))
      (ssadd (entlast) c-secc)
    ) ;_ fin de if
  ) ;_ fin de foreach
  (cmd-entrar)
  (vl-cmdf "_loft" "_mode" "_surface" c-secc "" "")
  (if (= (vla-get-ObjectName
           (vlax-ename->vla-object (entlast))
         ) ;_ fin de vla-get-ObjectName
         "AcDbNurbSurface"
      ) ;_ fin de =
    (progn
      (setq obj (vlax-ename->vla-object (entlast)))
      (if (> (getvar "DELOBJ") 0)
        (progn (setq i 0)
               (repeat (sslength c-secc)
                 (entdel (ssname c-secc i))
                 (setq i (1+ i))
               ) ;_ fin de repeat
        ) ;_ fin de progn
      ) ;_ fin de if
      (if mtrans
        (vla-TransformBy obj mtrans)
      ) ;_ fin de if
      (ax-traslacion obj (trans centro 1 0 t))
      (vla-put-CvHullDisplay obj 1)
      (ax-SWs)
    ) ;_ fin de progn
    (progn
      (vla-ZoomWindow
        *aesl:acad*
        (vlax-3d-point (list xmin ymin))
        (vlax-3d-point
          (mapcar '- (list xmin ymin))
        ) ;_ fin de vlax-3d-point
      ) ;_ fin de vla-ZoomWindow
      (alert
        (strcat
          "\nError al construir la superficie."
          "\nProbablemente las secciones forman bucles"
        ) ;_ fin de strcat
      ) ;_ fin de alert
    ) ;_ fin de progn
  ) ;_ fin de if
  (cmd-salir)
) ;_ fin de defun
;;;Listado 20.4. Funcin principal C:SUP-NURBS.

(defun ent-perfil (elev alto ancho capa /)
  (setq x (/ ancho 2.0)
        y (- alto x)
  ) ;_ fin de setq
  (entmake (list '(0 . "LWPOLYLINE")
                 '(100 . "AcDbEntity")
                 '(100 . "AcDbPolyline")
                 (cons 8 capa)
                 (cons 38 elev)
                 '(90 . 4)              ;Nmero de vrtices
                 '(70 . 1)              ;Indica polilnea cerrada (1)
                 (list 10 x 0.0)        ;Coord. primer vrtice (SCO)
                 '(42 . 0.0)            ;Curvatura del segmento recto
                 '(91 . 0)              ;Fin de datos del vrtice 1
                 (list 10 x y)          ;Coord. segundo vrtice (SCO)
                 '(42 . 1.0)            ;Curvatura del segmento curvo
                 '(91 . 0)              ;Fin de datos del vrtice 2
                 (list 10 (- x) y)      ;Coord.tercer vrtice (SCO)
                 '(42 . 0.0)            ;Curvatura del segmento recto
                 '(91 . 0)              ;Fin de datos del vrtice 3
                 (list 10 (- x) 0.0)    ;Coord.cuarto vrtice (SCO)
                 '(42 . 0.0)            ;Curvatura del segmento recto
                 '(91 . 0)              ;Fin de datos vrtice 4
                 '(210 0.0 1.0 0.0)
           ) ;_ fin de list
  ) ;_ fin de entmake
)                                       ;Vector normal.
;;;Listado 20.5. Funcin que crea el perfil para la superficie de procedimiento.

(defun datos-sup-aso (/)
  (initget (+ 1 2 4))
  (setq n (getint "\nNmero de Perfiles: "))
  (initget (+ 1 2 4))
  (setq intervalo
         (getreal "\nDistancia entre perfiles: ")
  ) ;_ fin de setq
  (initget (+ 1 2 4))
  (setq alto (getreal "\nAltura del perfil: "))
  (initget (+ 1 2 4))
  (while
    (>= (setq
          ancho (getreal "\nAnchura del perfil: ")
        ) ;_ fin de setq
        (* alto 2)
    ) ;_ fin de >=
     (prompt
       (strcat "\nLa anchura debe ser menor que "
               (rtos (* alto 2.0) 2 2)
       ) ;_ fin de strcat
     ) ;_ fin de prompt
     (initget (+ 1 2 4))
  ) ;_ fin de while
  (initget 1)
  (setq id
         (getstring "\nID de la superficie: ")
  ) ;_ fin de setq
) ;_ fin de defun
;;;Listado 20.6. Solicitud de los datos para la creacin de los perfiles.

(defun ax-trans-scp (col-scp origen pt-ejex pt-ejey nombre / scp
                     actscp
                    )
  (setq scp (vl-catch-all-apply
              'vla-add
              (list col-scp
                    (vlax-3d-point origen)
                    (vlax-3d-point pt-ejex)
                    (vlax-3d-point pt-ejey)
                    nombre
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (cond
    ((vl-catch-all-error-p scp)
     (prompt (strcat "\nERROR: "
                     (vl-catch-all-error-message scp)
             ) ;_ fin de strcat
     ) ;_ fin de prompt
    )
    (t
     (setq actscp (vl-catch-all-apply
                    'vla-put-ActiveUCS
                    (list *aesl:dibujo* scp)
                  ) ;_ fin de vl-catch-all-apply
     ) ;_ fin de setq
     (if (vl-catch-all-error-p actscp)
       (prompt
         (strcat "\nERROR: "
                 (vl-catch-all-error-message scp)
         ) ;_ fin de strcat
       ) ;_ fin de prompt
     ) ;_ fin de if
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 20.7. Funcin que establece el SCP correcto antes de aplicar las restricciones.

(defun crea-perfiles (lista-origen alto ancho capa id / col-scp
                      pt-o pt-x pt-y nom i p1 p2
                     )
  (setq col-scp (vla-get-UserCoordinateSystems
                  *aesl:dibujo*
                ) ;_ fin de vla-get-UserCoordinateSystems
  ) ;_ fin de setq
  (foreach elev lista-origen
    (ent-perfil elev alto ancho capa)
    (setq perfiles (cons (entlast) perfiles))
  ) ;_ fin de foreach
  (setq perfiles (reverse perfiles))
  (ax-vista '(-1 0 0) t)
  (setvar "perspective" 0)
  (foreach perfil perfiles
    (setq pt-o (list 0.0 (nth i lista-origen) 0.0)
          pt-x (list 100.0 (nth i lista-origen) 0.0)
          pt-y (list 0 (nth i lista-origen) 100.0)
          nom  (strcat "SCP-" (itoa i))
          i    (1+ i)
    ) ;_ fin de setq
    (ax-trans-scp col-scp pt-o pt-x pt-y nom)
    (vl-cmdf "_AutoConstrain" perfil "")
    (vl-cmdf "_GcFix" (osnap '(0 0 0) "_mid"))
    (setq p1 (osnap (list 0.0 alto 0.0) "_qua")
          p2 (list 0.0 (/ alto 3.0) 0.0)
    ) ;_ fin de setq
    (vl-cmdf "_DcRadius" p1 p2 "")
    (setq resdim (vl-catch-all-apply
                   'vlax-ename->vla-object
                   (list (entlast))
                 ) ;_ fin de vl-catch-all-apply
    ) ;_ fin de setq
    (cond
      ((vl-catch-all-error-p resdim)
       (prompt (strcat "\nERROR: "
                       (vl-catch-all-error-message)
               ) ;_ fin de strcat
       ) ;_ fin de prompt
      )
      (t
       (vla-get-DimConstrName resdim)
       (vla-put-DimConstrName
         resdim
         (strcat id "_rad" (itoa i))
       ) ;_ fin de vla-put-DimConstrName
       (vla-put-DimConstrDesc
         resdim
         (strcat "Superficie "
                 id
                 "; Radio del perfil "
                 (itoa i)
         ) ;_ fin de strcat
       ) ;_ fin de vla-put-DimConstrDesc
      )
    ) ;_ fin de cond
  ) ;_ fin de foreach
) ;_ fin de defun
;;;Listado 20.8. Creacin de los perfiles y sus restricciones geomtricas y dimensionales.

(defun crea-sup-aso (perfiles /)
  (setvar "SURFACEASSOCIATIVITY" 1)
  (setvar "SURFACEMODELINGMODE" 0)
  (vla-put-ActiveLayer
    *aesl:dibujo*
    (ax-capa (vla-get-layers *aesl:dibujo*)
             (strcat id "_SUPERFICIE")
             "4"
             "Continuous"
    ) ;_ fin de ax-capa
  ) ;_ fin de vla-put-ActiveLayer
  (vl-cmdf "_LOFT" "_MOde" "_SUrface")
  (foreach perfil perfiles (vl-cmdf perfil))
  (vl-cmdf "" "_Cross")
) ;_ fin de defun
;;;Listado 20.9. Funcin que crea la superficie asociativa.

(defun val (i n /)
  (if (zerop (logand 1 i))
    (+ (/ n 2.0))
    (- (/ n 2.0))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 20.10. Funcin que calcula el valor a asignar a la restriccin dimensional.

(defun mod-rest
                (/ rdims i n rdim pos rdim valor-actual increm)
  (if (setq rdims
             (ssget
               "X"
               (list '(0 . "DIMENSION")
                     '(8 . "*ADSK_CONSTRAINTS")
                     (cons 1 (strcat id "_rad*"))
               ) ;_ fin de list
             ) ;_ fin de ssget
      ) ;_ fin de setq
    (progn (setq i 0
                 n (sslength rdims)
           ) ;_ fin de setq
           (repeat n
             (setq rdim         (ssname rdims i)
                   pos          (cdr (assoc 10 (entget rdim)))
                   rdim         (vlax-ename->vla-object rdim)
                   valor-actual (atof
                                  (vla-get-DimConstrValue
                                    rdim
                                  ) ;_ fin de vla-get-DimConstrValue
                                ) ;_ fin de atof
                   increm       (val i valor-actual)
                   i            (1+ i)
             ) ;_ fin de setq
             (vla-put-DimConstrExpression
               rdim
               (rtos (+ valor-actual increm))
             ) ;_ fin de vla-put-DimConstrExpression
           ) ;_ fin de repeat
    ) ;_ fin de progn
    (alert
      "\nError en las restricciones dimensionales."
    ) ;_ fin de alert
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 20.11. Funcin que modifica las restricciones dimensionales.

(defun C:SUP-ASO (/ *error* n intervalo alto ancho lista-origen
                  perfiles
                 )
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-StartUndoMark *aesl:dibujo*)
  (cmd-entrar)
  (if (= (getvar "WORLDUCS") 0)
    (vl-cmdf "_UCS" "_W")
  ) ;_ fin de if
  (datos-sup-aso)
  (setq i 0)
  (repeat n
    (setq lista-origen
           (cons (* intervalo i)
                 lista-origen
           ) ;_ fin de cons
          i (1+ i)
    ) ;_ fin de setq
  ) ;_ fin de repeat
  (crea-perfiles
    (reverse lista-origen)
    alto
    ancho
    (strcat id "_PERFIL")
    id
  ) ;_ fin de crea-perfiles
  (crea-sup-aso perfiles)
  (mod-rest)
  (cmd-salir)
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 20.12. Funcin principal C:SUP-ASO.

(defun cmd-conv-param (id / rdims i)
  (if (= (getvar "BLOCKEDITOR") 1)
    (progn (setq rdims (ssget
                         "X"
                         (list '(0 . "DIMENSION")
                               '(8 . "*ADSK_CONSTRAINTS")
                               (cons 1 (strcat id "_rad*"))
                         ) ;_ fin de list
                       ) ;_ fin de ssget
                 i     0
           ) ;_ fin de setq
           (repeat (sslength rdims)
             (vl-cmdf "_bcparameter"
                      "_Convert"
                      (ssname rdims i)
                      ""
             ) ;_ fin de vl-cmdf
             (setq i (1+ i))
           ) ;_ fin de repeat
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 20.13. Funcin que convierte las restricciones en parmetros si se trabaja en el Editor de Bloques.

(defun C:SUP-ASO (/ *error* n intervalo alto ancho lista-origen
                  perfiles
                 )
  (defun *error* ()
    (cmd-salir)
    (vla-EndUndoMark *aesl:dibujo*)
  ) ;_ fin de defun
  (vla-startundomark *aesl:dibujo*)
  (cmd-entrar)
  (if (= (getvar "WORLDUCS") 0)
    (vl-cmdf "_UCS" "_W")
  ) ;_ fin de if
  (datos-sup-aso)
  (setq i 0)
  (repeat n
    (setq lista-origen
           (cons (* intervalo i)
                 lista-origen
           ) ;_ fin de cons
          i (1+ i)
    ) ;_ fin de setq
  ) ;_ fin de repeat
  (crea-perfiles
    (reverse lista-origen)
    alto
    ancho
    (strcat id "_PERFIL")
    id
  ) ;_ fin de crea-perfiles
  (crea-sup-aso perfiles)
  (mod-rest)
  (cmd-conv-param id)
  (cmd-salir)
  (ax-SWs)
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 20.14. Cdigo de C:SUP-ASO con la conversin de restricciones a parmetros.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
