;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) 2012 Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 21.	Responder a Eventos: los Reactores.

(defun procesa-textos (obj-notif / sel-texto nueva-escala alt-texto
                       i texto
                      )
  (setq sel-texto    (ssget "X"
                            '((0 . "TEXT") (410 . "Model"))
                     ) ;_ fin de ssget
        nueva-escala (vla-get-CustomScale obj-notif)
        alt-texto    (* (getvar "textsize")
                        (/ 1 nueva-escala)
                     ) ;_ fin de *
        i            0
  ) ;_ fin de setq
  (while (setq texto (ssname sel-texto i))
    (setq texto (vlax-ename->vla-object texto))
    (if (and (vlax-write-enabled-p texto)
             (usar? (vla-get-Layer texto))
             (/= (vla-get-height texto) alt-texto)
        ) ;_ fin de and
      (vla-put-Height texto alt-texto)
    ) ;_ fin de if
    (setq i (1+ i))
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 21.1. Funcin que procesa los textos.

(defun respuesta-ventana (obj-notif obj-react lis-param)
  (if
    (and (not *ComandoActivo*)
         (vlax-read-enabled-p obj-notif)
         (= (getvar "ctab") (car (vlr-data obj-react)))
    ) ;_ fin de and
     (procesa-textos obj-notif)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 21.2. Funcin de respuesta a la modificacin de la ventana grfica.

(defun respuesta-inicio-comando (obj-react lis-param)
  (setq *ComandoActivo* t)
) ;_ fin de defun
;;;Listado 21.3. Funcin de respuesta al inicio de un comando.

(defun respuesta-fin-comando
                             (obj-react lis-param / obj-vgraf)
  (setq *ComandoActivo* nil
        obj-vgraf
                        (vlax-ename->vla-object
                          (handent (cadr (vlr-data obj-react)))
                        ) ;_ fin de vlax-ename->vla-object
  ) ;_ fin de setq
  (if (and (= (car lis-param) "PSPACE")
           (vlax-read-enabled-p obj-vgraf)
           (/= (getvar "CTAB") "Model")
      ) ;_ fin de and
    (procesa-textos obj-vgraf)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 21.4. Funcin de respuesta al fin de un comando.

(defun reactor-ventana (ventana)
  (vlr-pers
    (VLR-Object-Reactor
      (list ventana)
      (list (getvar "CTAB") (vla-get-Handle ventana))
      '((:VLR-objectClosed . respuesta-ventana))
    ) ;_ fin de VLR-Object-Reactor
  ) ;_ fin de vlr-pers
) ;_ fin de defun
;;;Listado 21.5. Creacin del reactor de Objeto.

(defun reactor-comando (ventana)
  (vlr-pers
    (vlr-command-reactor
      (list (getvar "ctab") (vla-get-handle ventana))
      '((:vlr-commandWillStart . respuesta-inicio-comando)
        (:vlr-commandEnded . respuesta-fin-comando)
       )
    ) ;_ fin de vlr-command-reactor
  ) ;_ fin de vlr-pers
) ;_ fin de defun
;;;Listado 21.6. Creacin del reactor de Comando.

(defun C:AUTO-ESCALA (/ vgraf obj-vgraf)
  (cond
    ((/= (getvar "ctab") "Model")
     (prompt
       "\nDesigne ventana para auto-escala de textos:\n"
     ) ;_ fin de prompt
     (while (not (setq vgraf
                        (ssget "_:S" '((0 . "VIEWPORT")))
                 ) ;_ fin de setq
            ) ;_ fin de not
     ) ;_ fin de while
     (setq obj-vgraf (vlax-ename->vla-object
                       (ssname vgraf 0)
                     ) ;_ fin de vlax-ename->vla-object
     ) ;_ fin de setq
     (if (not (propietario? obj-vgraf))
       (progn (reactor-ventana obj-vgraf)
              (reactor-comando obj-vgraf)
       ) ;_ fin de progn
       (alert "Esta ventana ya tiene reactores.")
     ) ;_ fin de if
    )
    (t
     (alert
       "Este comando vlido slo\nen Espacio Papel."
     ) ;_ fin de alert
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 21.7. C:AUTO-ESCALA. Interfaz de usuario para creacin de los reactores.

(defun propietario? (objeto / reactores)
  (if (setq
        reactores (mapcar 'cadr
                          (vlr-reactors
                            :VLR-Object-Reactor
                          ) ;_ fin de vlr-reactors
                  ) ;_ fin de mapcar
      ) ;_ fin de setq
    (apply 'or
           (mapcar
             '(lambda (x)
                (if (member objeto (vlr-owners x))
                  t
                ) ;_ fin de if
              ) ;_ fin de lambda
             reactores
           ) ;_ fin de mapcar
    ) ;_ fin de apply
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 21.8. Funcin que determina si un objeto tiene vinculados reactores.

(vl-load-com)
(load "funciones-reactor"
      "No se encuentra FUNCIONES-REACTOR"
) ;_ fin de load
(autoload "escala-texto" '("AUTO-ESCALA"))
;;;Listado 21.9. Cdigo a incluir en ACADDOC.LSP.

;|
(defun reaccion (reactor params)
  (princ (vlr-data reactor))
  (princ " | ")
  (princ (vlr-current-reaction-name))
  (princ " | ")
  (princ params)
  (princ "\n"))
;;;Listado 21.10. Funcin de respuesta que informa de los eventos.


(defun reaccion-obj (obj reactor params)
  (princ (vlr-data reactor))
  (princ " | ")
  (princ (vlr-current-reaction-name))
  (princ " | ")
  (princ params)
  (princ "\n"))
;;;Listado 21.11. Funcin de respuesta para el reactor de objeto.

(vlr-acdb-reactor "BaseDatos"
                  '((:vlr-objectAppended . reaccion)
                    (:vlr-objectUnAppended . reaccion)
                    (:vlr-objectReAppended . reaccion)
                    (:vlr-objectOpenedForModify . reaccion)
                    (:vlr-objectModified . reaccion)
                    (:vlr-objectErased . reaccion)
                    (:vlr-objectUnErased . reaccion)))
Listado 21.12. Creacin del reactor de la Base de Datos


(vlr-editor-reactor
  "Editor"
  '((:vlr-beginClose . reaccion)
    (:vlr-beginDxfIn . reaccion)
    (:vlr-abortDxfIn . reaccion)
    (:vlr-dxfInComplete . reaccion)
    (:vlr-beginDxfOut . reaccion)
    (:vlr-abortDxfOut . reaccion)
    (:vlr-dxfOutComplete . reaccion)
    (:vlr-databaseToBeDestroyed . reaccion)
    (:vlr-unknownCommand . reaccion)
    (:vlr-commandWillStart . reaccion)
    (:vlr-commandCancelled . reaccion)
    (:vlr-commandEnded . reaccion)
    (:vlr-commandFailed . reaccion)
    (:vlr-lispWillStart . reaccion)
    (:vlr-lispEnded . reaccion)
    (:vlr-lispCancelled . reaccion)
    (:vlr-beginDwgOpen . reaccion)
    (:vlr-endDwgOpen . reaccion)
    (:vlr-dwgFileOpened . reaccion)
    (:vlr-beginSave . reaccion)
    (:vlr-sysVarWillChange . reaccion)
    (:vlr-sysVarChanged . reaccion)))
;;;Listado 21.13. Creacin del reactor del Editor.

(defun c:vlr-objeto  ()
  (setq obj (vlax-ename->vla-object (car (entsel))))
  (vlr-object-reactor (list obj)
                      "Objeto"
                      '((:vlr-cancelled . reaccion-obj)
                        (:vlr-copied . reaccion-obj)
                        (:vlr-erased . reaccion-obj)
                        (:vlr-unerased . reaccion-obj)
                        (:vlr-goodbye . reaccion-obj)
                        (:vlr-openedForModify . reaccion-obj)
                        (:vlr-modified . reaccion-obj)
                        (:vlr-subObjModified . reaccion-obj)
                        (:vlr-modifyUndone . reaccion-obj)
                        (:vlr-modifiedXData . reaccion-obj)
                        (:vlr-unappended . reaccion-obj)
                        (:vlr-reappended . reaccion-obj)
                        (:vlr-objectClosed . reaccion-obj))))
;;;Listado 21.14. Funcin para vincular un reactor informativo de Objeto.

|;
;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
