;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) 2012 Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 24.	Tablas.

(defun ins-tabla
                 (/ pt altura-fila ancho-col nfilas ncols)
  (setq pt          (getpoint
                      "\nEsquina superior izquierda de la tabla: "
                    ) ;_ fin de getpoint
        altura-fila (getdist pt
                             "\nAltura para las filas: "
                    ) ;_ fin de getdist
        ancho-col   (getdist pt
                             "\nAnchura para las columnas: "
                    ) ;_ fin de getdist
        nfilas      (getint "\nNmero de filas: ")
        ncols       (getint "\nNmero de columnas: ")
  ) ;_ fin de setq
  (vla-AddTable
    (espacio-actual *aesl:dibujo*)
    (vlax-3d-point pt)
    nfilas
    ncols
    altura-fila
    ancho-col
  ) ;_ fin de vla-AddTable
) ;_ fin de defun
;;;Listado 24.1. Funcin que inserta una tabla en el dibujo.

(defun altura-fila (obj-tabla i-fila altura-texto /)
  (setq i 0)
  (repeat (vla-get-Columns obj-tabla)
    (vla-setTextHeight2 obj-tabla i-fila i 0 altura-texto)
    (setq i (1+ i))
  ) ;_ fin de repeat
) ;_ fin de defun
;;;Listado 24.2. Funcin que cambia la altura de texto de una fila.

(defun altura-columna (obj-tabla i-col altura-texto /)
  (setq i 0)
  (repeat (vla-get-Rows obj-tabla)
    (vla-setTextHeight2 obj-tabla i i-col 0 altura-texto)
    (setq i (1+ i))
  ) ;_ fin de repeat
) ;_ fin de defun
;;;Listado 24.3. Funcin que cambia la altura de texto para una columna.

(defun sel-bloque
                  (nombre / ss cont ename datos lista-datos)
  (setq cont 0)
  (if (setq ss
             (ssget
               "X"
               (list (cons 0 "INSERT") (cons 2 nombre))
             ) ;_ fin de ssget
      ) ;_ fin de setq
    (while (setq ename (ssname ss cont))
      (setq obj (vlax-ename->vla-object ename))
      (setq datos
             (list
               (cons "CAPA" (vla-get-layer obj))
               (cons
                 "COORDS"
                 (vlax-safearray->list
                   (vlax-variant-value
                     (vla-get-insertionpoint
                       obj
                     ) ;_ fin de vla-get-insertionpoint
                   ) ;_ fin de vlax-variant-value
                 ) ;_ fin de vlax-safearray->list
               ) ;_ fin de cons
             ) ;_ fin de list
      ) ;_ fin de setq
      (setq lista-datos
             (cons
               (append
                 datos
                 (ax-lee-atributos ename)
               ) ;_ fin de append
               lista-datos
             ) ;_ fin de cons
      ) ;_ fin de setq
      (setq cont (1+ cont))
    ) ;_ fin de while
  ) ;_ fin de if
  lista-datos
) ;_ fin de defun
;;;Listado 24.4. Seleccin del bloque a procesar y extraccin de sus atributos a una lista.

(defun C:TABLA-ATRIBUTOS (/ *error* pres-act bloque nombre
                          atributos nfilas ncols pt-ins pt-esq
                          altura-fila ancho-col obj-tabla i j txt
                         )
  (vla-StartUndoMark *aesl:dibujo*)
  (defun *error* (msg)
    (vla-EndUndoMark *aesl:dibujo*)
    (vl-cmdf "_U")
    (prompt msg)
  ) ;_ fin de defun
  (if (/= (setq pres-act (getvar "CTAB")) "Model")
    (setvar "TILEMODE" 1)
  ) ;_ fin de if
  (while
    (not (and (setq bloque
                     (ssget "_:S" '((0 . "INSERT")))
              ) ;_ fin de setq
              (equal (vla-get-HasAttributes
                       (vlax-ename->vla-object
                         (ssname bloque 0)
                       ) ;_ fin de vlax-ename->vla-object
                     ) ;_ fin de vla-get-HasAttributes
                     :vlax-true
              ) ;_ fin de equal
         ) ;_ fin de and
    ) ;_ fin de not
     (prompt
       "\nDebe seleccionar un bloque con atributos: "
     ) ;_ fin de prompt
  ) ;_ fin de while
  (setq nombre    (cdr (assoc 2 (entget (ssname bloque 0))))
        atributos (sel-bloque nombre)
        nfilas    (length atributos)
        ncols     (length (car atributos))
  ) ;_ fin de setq
  (setvar "CTAB" pres-act)
  (initget 1)
  (setq pt-ins
         (getpoint
           "\nIndique punto de insercin de la tabla: "
         ) ;_ fin de getpoint
  ) ;_ fin de setq
  (initget (+ 1 32))
  (setq pt-esq      (getcorner pt-ins
                               "\nIndique tamao de la tabla: "
                    ) ;_ fin de getcorner
        altura-fila (/ (abs (- (nth 1 pt-ins)
                               (nth 1 pt-esq)
                            ) ;_ fin de -
                       ) ;_ fin de abs
                       nfilas
                    ) ;_ fin de /
        ancho-col   (/ (abs
                         (- (nth 0 pt-esq) (nth 0 pt-ins))
                       ) ;_ fin de abs
                       ncols
                    ) ;_ fin de /
        obj-tabla                       ; Creacin de Tabla
                    (vla-AddTable
                      (espacio-actual *aesl:dibujo*)
                      (vlax-3d-point pt-ins)
                      (1+ nfilas)
                      ncols
                      altura-fila
                      ancho-col
                    ) ;_ fin de vla-AddTable
  ) ;_ fin de setq
  (vla-put-RegenerateTableSuppressed
    obj-tabla
    :vlax-true
  ) ;_ fin de vla-put-RegenerateTableSuppressed
  (vla-SetRowHeight obj-tabla 0 (* altura-fila 1.5))
  (vla-SetTextHeight
    obj-tabla
    acTitleRow
    (* altura-fila 0.75)
  ) ;_ fin de vla-SetTextHeight
  (vla-SetTextHeight
    obj-tabla
    acHeaderRow
    (* altura-fila 0.60)
  ) ;_ fin de vla-SetTextHeight
  (vla-SetTextHeight
    obj-tabla
    acDataRow
    (* altura-fila 0.50)
  ) ;_ fin de vla-SetTextHeight
  (vla-SetText obj-tabla 0 0 nombre)    ; Ttulo
  (setq i 0)
  (foreach dato (car atributos)
    (vla-SetText obj-tabla 1 i (car dato)) ; Encabezamiento
    (setq i (1+ i))
  ) ;_ fin de foreach
  (setq i 0)
  (repeat (1- nfilas)                   ; Filas de datos
    (setq j 0)
    (repeat ncols
      (vla-SetCellAlignment
        obj-tabla
        (+ i 2)
        j
        acMiddleCenter
      ) ;_ fin de vla-SetCellAlignment
      (setq
        txt (vl-princ-to-string
              (cdr (nth j (nth i atributos)))
            ) ;_ fin de vl-princ-to-string
      ) ;_ fin de setq
      (vla-SetText obj-tabla (+ i 2) j txt)
      (setq j (1+ j))
    ) ;_ fin de repeat
    (setq i (1+ i))
  ) ;_ fin de repeat
  (vla-put-RegenerateTableSuppressed
    obj-tabla
    :vlax-false
  ) ;_ fin de vla-put-RegenerateTableSuppressed
  (vla-EndUndoMark *aesl:dibujo*)
) ;_ fin de defun
;;;Listado 24.5. Funcin principal C:TABLA-DATOS.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
