;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) 2012 Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 25.	Visual LISP como cliente ActiveX.

(defun conecta-excel ()
  (setq apl-excel        (vlax-get-or-create-object
                           "excel.application"
                         ) ;_ fin de vlax-get-or-create-object
        coleccion-libros (vlax-get-property
                           apl-excel
                           "workbooks"
                         ) ;_ fin de vlax-get-property
        nuevo-libro      (vlax-invoke-method
                           coleccion-libros
                           "add"
                         ) ;_ fin de vlax-invoke-method
        coleccion-hojas  (vlax-get-property
                           nuevo-libro
                           "sheets"
                         ) ;_ fin de vlax-get-property
        hoja-1           (vlax-get-property
                           coleccion-hojas
                           "item"
                           1
                         ) ;_ fin de vlax-get-property
        celdas-excel     (vlax-get-property hoja-1 "cells")
  ) ;_ fin de setq
  (vla-put-visible apl-excel :vlax-true)
) ;_ fin de defun
;;;Listado 25.1. Funcin CONECTA-EXCEL.

(defun desconecta-excel ()
  (vlax-release-object apl-excel)
  (gc)
) ;_ fin de defun
;;;Listado 25.2. Funcin DESCONECTA-EXCEL.

(defun apl-err (msg) (desconecta-excel) (prompt msg))
;;;Listado 25.3. Funcin APL-ERR.

(defun lista->excel (nombre lista / *error* celdas-excel hoja-1
                     coleccion-hojas nuevo-libro coleccion-libros
                     apl-excel
                    )
  (setq *error* apl-err)
  (vl-load-com)
  (conecta-excel)
  (vlax-put-property hoja-1 "Name" nombre)
  (procesa-tabla lista)
  (desconecta-excel)
) ;_ fin de defun
;;;Listado 25.4. Funcin LISTA->EXCEL.

(defun procesa-tabla (lista / numfila numcol)
  (setq numfila 1
        numcol 0
  ) ;_ fin de setq
  (foreach campo (car lista)
    (dato->celda
      numfila
      (setq numcol (1+ numcol))
      (car campo)
    ) ;_ fin de dato->celda
  ) ;_ fin de foreach
  (while (setq fila (car lista))
    (setq numfila (1+ numfila)
          lista   (cdr lista)
    ) ;_ fin de setq
    (procesa-fila fila numfila)
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 25.5. Funcin PROCESA-TABLA.

(defun procesa-fila (fila numfila / numcol)
  (setq numcol 0)
  (foreach campo fila
    (dato->celda
      numfila
      (setq numcol (1+ numcol))
      (cdr campo)
    ) ;_ fin de dato->celda
  ) ;_ fin de foreach
) ;_ fin de defun
;;;Listado 25.6. Funcin procesa-fila.

(defun dato->celda (fila col valor)
  (vlax-put-property
    celdas-excel
    "Item"
    fila
    col
    (vl-princ-to-string valor)
  ) ;_ fin de vlax-put-property
) ;_ fin de defun
;;;Listado 25.7. Funcin Auxiliar DATO->CELDA.

(defun C:EXCEL-ATRIBUTOS (/ lista-bloques)
  (if (inicia-dialogo
        "atributos"
        "./dcl/atributos.dcl"
      ) ;_ fin de inicia-dialogo
    (progn
      (if (setq lista-bloques (lee-bloques))
        (llena-lista "lista_bloques" lista-bloques)
        (set_tile
          "error"
          "No hay bloques en el Dibujo Actual"
        ) ;_ fin de set_tile
      ) ;_ fin de if
      (action_tile
        "lista_bloques"
        "(comprueba-atributos $value)"
      ) ;_ fin de action_tile
      (action_tile
        "accept"
        "(extrae (get_tile \"lista_bloques\"))"
      ) ;_ fin de action_tile
      (action_tile "cancel" "(done_dialog 0)")
      (start_dialog)
      (unload_dialog dcl_id)
      (princ)
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 25.9. Funcin principal C:EXCEL-ATRIBUTOS.

(defun inicia-dialogo (nombredialogo archivodcl / ok)
  (if (= -1 (setq dcl_id (load_dialog archivodcl)))
    (alert "No se encuentra el archivo DCL")
    (if (not (new_dialog nombredialogo dcl_id))
      (alert "No se encuentra la definicin del dilogo")
      (setq ok t)
    ) ;_ fin de if
  ) ;_ fin de if
  ok
) ;_ fin de defun
;;;Listado 25.10. Funcin auxiliar INICIA-DIALOGO.

(defun llena-lista (componente-lista lista-textos)
  (start_list componente-lista)
  (mapcar
    'add_list
    (mapcar '(lambda (term) (strcat (car term) "\t" (cdr term)))
            lista-textos
    ) ;_ fin de mapcar
  ) ;_ fin de mapcar
  (end_list)
) ;_ fin de defun
;;;Listado 25.11. Funcin LLENA-LISTA.

(defun comprueba-atributos (valor)
  (if (not
        (equal (cdr (nth (atoi valor) lista-bloques)) "ATRIB")
      ) ;_ fin de not
    (set_tile "error"
              "El bloque seleccionado no posee atributos"
    ) ;_ fin de set_tile
    (set_tile "error" "")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 25.12. Funcin COMPRUEBA-ATRIBUTOS.

(defun extrae (valor / seleccin nombre)
  (if lista-bloques
    (setq seleccin (nth (atoi valor) lista-bloques))
  ) ;_ fin de if
  (if (= (cdr seleccin) "ATRIB")
    (progn (set_tile "error" "Espere, por favor...")
           (setq nombre (car seleccin))
           (lista->excel nombre (sel-bloque nombre))
           (done_dialog)
    ) ;_ fin de progn
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 25.13. Funcin EXTRAE.

(defun lee-bloques (/ lista-blq)
  (vlax-for obj (vla-get-blocks *aesl:dibujo*)
    (if (and (not (wcmatch (vla-get-name obj) "`**,*|*"))
             (equal (vla-get-IsXref obj) :vlax-false)
        ) ;_ fin de and
      (setq lista-blq
             (cons (cons (vla-get-Name obj)
                         (if (tiene-atributos? obj)
                           "ATRIB"
                           ""
                         ) ;_ fin de if
                   ) ;_ fin de cons
                   lista-blq
             ) ;_ fin de cons
      ) ;_ fin de setq
    ) ;_ fin de if
  ) ;_ fin de vlax-for
  (setq lista-blq
         (vl-sort lista-blq
                  '(lambda (n1 n2) (< (car n1) (car n2)))
         ) ;_ fin de vl-sort
  ) ;_ fin de setq
  lista-blq
) ;_ fin de defun
;;;Listado 25.14. Funcin LEE-BLOQUES.

(defun tiene-atributos? (obj-defbloque / resultado)
  (vlax-for obj obj-defbloque
    (if (equal (vla-get-ObjectName obj)
               "AcDbAttributeDefinition"
        ) ;_ fin de equal
      (setq resultado t)
    ) ;_ fin de if
  ) ;_ fin de vlax-for
  resultado
) ;_ fin de defun
;;;Listado 25.15. Funcin TIENE-ATRIBUTOS?.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
