;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 6.	Estructuras y Datos ActiveX--------------------------

;;;6.1	Matrices (Arrays)

(defun ax-cuadrada? (matriz / dim tmp)
  (setq dim 1)
  (repeat (vlax-safearray-get-dim matriz)
    (setq tmp (cons
                (- (vlax-safearray-get-u-bound matriz dim)
                   (vlax-safearray-get-l-bound matriz dim)
                ) ;_ fin de -
                tmp
              ) ;_ fin de cons
          dim (1+ dim)
    ) ;_ fin de setq
  ) ;_ fin de repeat
  (apply '= tmp)
) ;_ fin de defun
;;;Listado 6.1. Funcin AX CUADRADA?.

(defun ax-safearrayp (dato)
  (eq (type dato) 'safearray)
) ;_ fin de defun
;;;Listado 6.2. Predicado AX-SAFEARRAYP.

(defun ax-matriz->lista (s-arr)
  (if (ax-safearrayp s-arr)
    (vlax safearray >list s-arr)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 6.3. Funcin AX-MATRIZ->LISTA.

(defun ax-tipo-dato (lista)
  (if
    (apply 'and
           (mapcar '(lambda (x y) (eq (type x) (type y)))
                   lista
                   (cdr lista)
           ) ;_ fin de mapcar
    ) ;_ fin de apply
     (ax-tipo (car lista))
     vlax-vbVariant
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 6.4. Funcin AX TIPO-DATO.

(defun ax-tipo (dato)
  (setq dato (type dato))
  (cond ((eq dato 'INT) vlax-vbLong)
        ((eq dato 'REAL) vlax-vbDouble)
        ((eq dato 'STR) vlax-vbString)
        ((eq dato 'VLA-OBJECT) vlax-vbObject)
        (t vlax-vbVariant)
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 6.5. Funcin AX-TIPO.

(defun ax-lista->matriz (lista)
  (vlax-safearray-fill
    (vlax-make-safearray
      (ax-tipo-dato lista)
      (cons 0 (1- (length lista)))
    ) ;_ fin de vlax-make-safearray
    lista
  ) ;_ fin de vlax-safearray-fill
) ;_ fin de defun
;;;Listado 6.6. Funcin AX-LISTA->MATRIZ.


;;;6.6	Procesamiento de Colecciones.--------------------------------

(defun ax-act-des (dibujo)
  (vlax-map-collection
    (vla-get-layers dibujo)
    '(lambda (x)
       (if (equal (vla-get-LayerOn x) :vlax-true)
         (vla-put-LayerOn x :vlax-false)
         (vla-put-LayerOn x :vlax-true)
       ) ;_ fin de if
     ) ;_ fin de lambda
  ) ;_ fin de vlax-map-collection
) ;_ fin de defun
;;;Listado 6.7. Funcin para activar/desactivar capas mediante ActiveX.

(defun ax-lista-capas (dibujo / capas)
  (vlax-for capa (vla-get-layers dibujo)
    (setq capas
           (cons (vla-get-name capa) capas)
    ) ;_ fin de setq
  ) ;_ fin de vlax-for
  (acad_strlsort capas)
) ;_ fin de defun
;;;Listado 6.8. Obtencin de una lista de las capas del dibujo mediante ActiveX.

(defun ax-lista-nombres (dibujo nombre / nombres)
  (setq coleccion (vlax-get-property dibujo nombre))
  (vlax-for obj coleccion
    (setq nombres
           (cons (vla-get-name obj) nombres)
    ) ;_ fin de setq
  ) ;_ fin de vlax-for
  (acad_strlsort nombres)
) ;_ fin de defun
;;;Listado 6.9. Funcin genrica para obtener los nombres de los objetos de una coleccin.

(defun tan~ (ang) (/ (sin ang) (cos ang)))
;;;Listado 6.10. Clculo de tangente sin prever la divisin por cero.

(defun tan (ang / coseno)
  (if (zerop (setq coseno (cos ang)))
    1.8E+308
    (/ (sin ang) coseno)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 6.11. Clculo de la tangente previendo la divisin por cero.

(defun ax-existe? (elemento coleccin)
  (not (vl-catch-all-error-p
         (vl-catch-all-apply
           'vla-item
           (list coleccin elemento)
         ) ;_ fin de vl-catch-all-apply
       ) ;_ fin de vl-catch-all-error-p
  ) ;_ fin de not
) ;_ fin de defun
;;;Listado 6.12. Comprobar si existe un elemento en una coleccin.

(defun ax-existe? (elemento coleccin / resultado)
  (if (not (vl-catch-all-error-p
             (setq resultado (vl-catch-all-apply
                               'vla-item
                               (list coleccin elemento)
                             ) ;_ fin de vl-catch-all-apply
             ) ;_ fin de setq
           ) ;_ fin de vl-catch-all-error-p
      ) ;_ fin de not
    resultado
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 6.13. Funcin AX-EXISTE? que devuelve el objeto-VLA.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
