;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Captulo 7.	Entrada de datos.

(defun valor-por-defecto (funcin mensaje valor / tmp)
  (if (setq tmp
             (apply
               funcin
               (list (strcat mensaje
                             "<"
                             (vl-princ-to-string valor)
                             ">: "
                     ) ;_ fin de strcat
               ) ;_ fin de list
             ) ;_ fin de apply
      ) ;_ fin de setq
    tmp
    valor
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.1. Solicitud de datos incluyendo valor predeterminado.

(defun cadena-por-defecto (mensaje valor / tmp)
  (setq
    tmp (apply 'getstring
               (list (strcat mensaje "<" valor ">: "))
        ) ;_ fin de apply
  ) ;_ fin de setq
  (if (/= tmp "")
    tmp
    valor
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.2. Solicitud de cadena con valor predeterminado.

(defun valor-con-opciones
                          (funcion mensaje opciones / tmp)
  (initget opciones)
  (if (setq tmp
             (apply
               funcion
               (list
                 (strcat mensaje
                         " ["
                         (sustituye "/" " " opciones)
                         "]: "
                 ) ;_ fin de strcat
               ) ;_ fin de list
             ) ;_ fin de apply
      ) ;_ fin de setq
    tmp
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.3. Solicitud de datos incluyendo opciones.

(defun getfixnum (mensaje / tmp)
  (initget 1)
  (setq tmp (getreal mensaje))
  (while
    (and tmp (not (< -2147483647.0 tmp 2147483647.0)))
     (prompt
       "Requiere un n entero entre -2147483647 y 2147483647..\n"
     ) ;_ fin de prompt
     (initget 1)
     (setq tmp (getreal mensaje))
  ) ;_ fin de while
  (if (numberp tmp)
    (fix tmp)
    tmp
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.4. Funcin para obtener nmeros enteros largos.

(defun id-bits (valor /)
  (vl-remove 0
             (mapcar '(lambda (i) (logand i valor))
                     '(1 2 4 8 16 32 64 128 256 512 1024 2048 4096
                       8192 16384 32768 65536
                      )
             ) ;_ fin de mapcar
  ) ;_ fin de vl-remove
) ;_ fin de defun
;;;Listado 7.5. Funcin para detectar los bits activados.


(defun bits-activos? (bits valor)
  (= bits (logand bits valor))
) ;_ fin de defun

(defun activa-bits (bits valor)
  (logior bits valor)
) ;_ fin de defun

(defun anula-bits (bits valor)
  (logand (~ bits) valor)
) ;_ fin de defun
;;;Listado 7.6. Funciones para comprobar, activar o anular bits.

(defun conmuta-varsis (varsis bits)
  (setvar varsis (boole 6 (getvar varsis) bits))
) ;_ fin de defun
;;;Listado 7.7. Conmutador de variables.

(defun cmd-entrar (/ 3dosm)
  (if (and (setq 3dosm (getvar "3DOSMODE"))
           (not (bits-activos? (lsh 1 0) 3dosm))
      ) ;_ fin de and
    (conmuta-varsis "3DOSMODE" (lsh 1 0))
  ) ;_ fin de if
  (if (not (bits-activos? (lsh 1 14) (getvar "OSMODE")))
    (conmuta-varsis "OSMODE" (lsh 1 14))
  ) ;_ fin de if
  (if (bits-activos? (lsh 1 0) (getvar "CMDECHO"))
    (conmuta-varsis "CMDECHO" (lsh 1 0))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.8. Ajuste de las variables de sistema antes de entrar al comando.

(defun cmd-salir (/ 3dosm)
  (if (and (setq 3dosm (getvar "3DOSMODE"))
           (bits-activos? (lsh 1 0) 3dosm)
      ) ;_ fin de and
    (conmuta-varsis "3DOSMODE" (lsh 1 0))
  ) ;_ fin de if
  (if (bits-activos? (lsh 1 14) (getvar "OSMODE"))
    (conmuta-varsis "OSMODE" (lsh 1 14))
  ) ;_ fin de if
  (if (not (bits-activos? (lsh 1 0) (getvar "CMDECHO")))
    (conmuta-varsis "CMDECHO" (lsh 1 0))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.9. Restitucin de los valores originales al salir del comando.

;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
