;;;Cdigo fuente del libro "Experto AutoCAD con Visual LISP"
;;; (c) Ediciones ARTUAL, S.L. Barcelona, Espaa.
;;; (c) 2012 Reinaldo Togores. Todos los derechos reservados
;;; Se permite su uso mencionando la obra y su autor.

;;;Funciones de uso general.

(defun sustituye (nuevo viejo cadena)
  (while (vl-string-search viejo cadena)
    (setq cadena (vl-string-subst nuevo viejo cadena))
  ) ;_ fin de while
) ;_ fin de defun
;;;Listado 5.12. Sustitucin de caracteres en una cadena.

(defun valor-por-defecto (funcin mensaje valor / tmp)
  (if (setq tmp
             (apply
               funcin
               (list (strcat mensaje
                             "<"
                             (vl-princ-to-string valor)
                             ">: "
                     ) ;_ fin de strcat
               ) ;_ fin de list
             ) ;_ fin de apply
      ) ;_ fin de setq
    tmp
    valor
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.1. Solicitud de datos incluyendo valor predeterminado.

(defun cadena-por-defecto (mensaje valor / tmp)
  (setq
    tmp (apply 'getstring
               (list (strcat mensaje "<" valor ">: "))
        ) ;_ fin de apply
  ) ;_ fin de setq
  (if (/= tmp "")
    tmp
    valor
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.2. Solicitud de cadena con valor predeterminado.

(defun valor-con-opciones
                          (funcion mensaje opciones / tmp)
  (initget opciones)
  (if (setq tmp
             (apply
               funcion
               (list
                 (strcat mensaje
                         " ["
                         (sustituye "/" " " opciones)
                         "]: "
                 ) ;_ fin de strcat
               ) ;_ fin de list
             ) ;_ fin de apply
      ) ;_ fin de setq
    tmp
    valor
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.3. Solicitud de datos incluyendo opciones.


(defun bits-activos? (bits valor)
  (= bits (logand bits valor))
) ;_ fin de defun

(defun activa-bits (bits valor)
  (logior bits valor)
) ;_ fin de defun

(defun anula-bits (bits valor)
  (logand (~ bits) valor)
) ;_ fin de defun
;;;Listado 7.6. Funciones para comprobar, activar o anular bits.

(defun conmuta-varsis (varsis bits)
  (setvar varsis (boole 6 (getvar varsis) bits))
) ;_ fin de defun
;;;Listado 7.7. Conmutador de variables.

(defun cmd-entrar (/ 3dosm)
  (if (and (setq 3dosm (getvar "3DOSMODE"))
           (not (bits-activos? (lsh 1 0) 3dosm))
      ) ;_ fin de and
    (conmuta-varsis "3DOSMODE" (lsh 1 0))
  ) ;_ fin de if
  (if (not (bits-activos? (lsh 1 14) (getvar "OSMODE")))
    (conmuta-varsis "OSMODE" (lsh 1 14))
  ) ;_ fin de if
  (if (bits-activos? (lsh 1 0) (getvar "CMDECHO"))
    (conmuta-varsis "CMDECHO" (lsh 1 0))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.8. Ajuste de las variables de sistema antes de entrar al comando.

(defun cmd-salir (/ 3dosm)
  (if (and (setq 3dosm (getvar "3DOSMODE"))
           (bits-activos? (lsh 1 0) 3dosm)
      ) ;_ fin de and
    (conmuta-varsis "3DOSMODE" (lsh 1 0))
  ) ;_ fin de if
  (if (bits-activos? (lsh 1 14) (getvar "OSMODE"))
    (conmuta-varsis "OSMODE" (lsh 1 14))
  ) ;_ fin de if
  (if (not (bits-activos? (lsh 1 0) (getvar "CMDECHO")))
    (conmuta-varsis "CMDECHO" (lsh 1 0))
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 7.9. Restitucin de los valores originales al salir del comando.

(defun valor-cod (clave ename)
  (cdr (assoc clave (entget ename)))
) ;_ fin de defun
;;;Listado 10.9. Extraccin del valor asociado a un cdigo DXF.

(defun valores (clave lista / sublista resultado)
  (while (setq sublista (assoc clave lista))
    (setq resultado (cons (cdr sublista) resultado)
          lista     (cdr (member sublista lista))
    ) ;_ fin de setq
  ) ;_ fin de while
  (reverse resultado)
) ;_ fin de defun
;;;Listado 10.12. Extraccin de valores mltiples de una lista de asociacin.


(defun espacio-actual (dibujo /)
  (vla-get-block (vla-get-ActiveLayout dibujo))
) ;_ fin de defun
;;;Listado 10.26. Funcin que obtiene el espacio actual para la creacin de nueva geometra.

(defun 3d->2d (punto)
  (list (car punto) (cadr punto))
) ;_ fin de defun
;;;Listado 10.27. Funcin auxiliar 3d->2d.

(defun usar? (capa)
  (zerop
    (logand (cdr (assoc 70 (tblsearch "layer" capa)))
            (+ 1 4)
    ) ;_ fin de logand
  ) ;_ fin de zerop
) ;_ fin de defun
;;;Listado 10.36. Comprueba si una capa no est inutilizada ni bloqueada.

(defun ax-capa (col-capas nombre color tipolinea / capa)
  (setq capa (vla-add col-capas nombre))
  (vlax-put-property capa "color" color)
  (vlax-put-property capa "linetype" tipolinea)
  capa
) ;_ fin de defun
;;;Listado 10.39. Creacin de capa con ActiveX.


;;; Vector A->B
;;; Argumentos: A, B, listas de tres nmeros reales. 
(defun vec (A B) (mapcar '- B A))

;;; Suma de vectores
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun v+v (v1 v2) (mapcar '+ v1 v2))

;;; Producto ESCALAR (dot product)
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun x-esc (v1 v2) (apply '+ (mapcar '* v1 v2)))

;;; Longitud del vector (mdulo)
;;; Argumento: v, lista de tres nmeros reales.
(defun m-vec (v) (sqrt (apply '+ (mapcar '* v v))))

;;; Vector unitario
;;; Argumento: v, lista de tres nmeros reales.
(defun v-unit (v / m)
  (cond ((zerop (setq m (m-vec v))) nil)
        (t (mapcar '(lambda (n) (/ n m)) v))
  ) ;_ fin de cond
) ;_ fin de defun

;;; Producto VECTORIAL (cross product)
;;; Argumentos: v1, v2, listas de tres nmeros reales.
(defun prod-vec (v1 v2)
  (list (- (* (cadr v1) (caddr v2)) (* (cadr v2) (caddr v1)))
        (- (* (car v2) (caddr v1)) (* (car v1) (caddr v2)))
        (- (* (car v1) (cadr v2)) (* (car v2) (cadr v1)))
  ) ;_ fin de list
) ;_ fin de defun
;;;Listado 13.3. Operaciones con vectores.

(defun ax-traslacion (obj vector)
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list 1.0 0.0 0.0 (nth 0 vector))
            (list 0.0 1.0 0.0 (nth 1 vector))
            (list 0.0 0.0 1.0 (nth 2 vector))
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.4. Funcin para desplazamiento.

;;; Grados a Radianes
(defun gar (g) (/ (* g pi) 180.0))
;;; Radianes a Grados
(defun rag (r) (* (/ r pi) 180.0))
;;;Listado 13.5. Conversin entre Grados y Radianes.

(defun ax-rot-x (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list 1.0 0.0 0.0 0.0)
            (list 0.0 (cos a) (sin a) 0.0)
            (list 0.0 (- (sin a)) (cos a) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.6. Rotacin en torno a X.

(defun ax-rot-y (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos a) 0.0 (sin a) 0.0)
            (list 0.0 1.0 0.0 0.0)
            (list (- (sin a)) 0.0 (cos a) 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.7. Rotacin en torno a Y.

(defun ax-rot-z (obj a)
  (setq a (gar a))
  (vla-TransformBy
    obj
    (vlax-tmatrix
      (list (list (cos a) (- (sin a)) 0.0 0.0)
            (list (sin a) (cos a) 0.0 0.0)
            (list 0.0 0.0 1.0 0.0)
            (list 0.0 0.0 0.0 1.0)
      ) ;_ fin de list
    ) ;_ fin de vlax-tmatrix
  ) ;_ fin de vla-TransformBy
) ;_ fin de defun
;;;Listado 13.8. Rotacin en torno a Z.

(defun ax-escala (obj vector / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (list (list (nth 0 vector) 0.0 0.0 0.0)
                            (list 0.0 (nth 1 vector) 0.0 0.0)
                            (list 0.0 0.0 (nth 2 vector) 0.0)
                            (list 0.0 0.0 0.0 1.0)
                      ) ;_ fin de list
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.9. Funcin para transformacin de escala en XYZ.

(defun ax-ciz-x (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (list (list 1.0 factor 0.0 0.0)
                            (list 0.0 1.0 0.0 0.0)
                            (list 0.0 0.0 1.0 0.0)
                            (list 0.0 0.0 0.0 1.0)
                      ) ;_ fin de list
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.10. Cizalladura en X.

(defun ax-ciz-y (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (vlax-tmatrix
                        (list (list 1.0 0.0 0.0 0.0)
                              (list factor 1.0 0.0 0.0)
                              (list 0.0 0.0 1.0 0.0)
                              (list 0.0 0.0 0.0 1.0)
                        ) ;_ fin de list
                      ) ;_ fin de vlax-tmatrix
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.11. Cizalladura en Y.

(defun ax-ciz-z (obj factor / res)
  (setq res (vl-catch-all-apply
              'vla-TransformBy
              (list obj
                    (vlax-tmatrix
                      (vlax-tmatrix
                        (list (list 1.0 0.0 0.0 0.0)
                              (list 0.0 1.0 0.0 0.0)
                              (list factor factor 1.0 0.0)
                              (list 0.0 0.0 0.0 1.0)
                        ) ;_ fin de list
                      ) ;_ fin de vlax-tmatrix
                    ) ;_ fin de vlax-tmatrix
              ) ;_ fin de list
            ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p res)
    (prompt "Este objeto no puede ser deformado!")
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.12. Cizalladura en Z.

(defun ax-scp (nombre origen dirx diry / tmp)
  (setq
    tmp (vla-Add (vla-get-UserCoordinateSystems *aesl:dibujo*)
                 (vlax-3d-point '(0 0 0))
                 (vlax-3d-point dirx)
                 (vlax-3d-point diry)
                 nombre
        ) ;_ fin de vla-Add
  ) ;_ fin de setq
  (vla-put-Origin tmp (vlax-3d-point origen))
  tmp
) ;_ fin de defun
;;;Listado 13.14. Funcin que aade un nuevo SCP al documento actual.

(defun ax-matriz-scp (/ nombre num-scp nuevo-scp)
  (setq nombre (getvar "UCSNAME"))
  (cond
    ((or (equal nombre "")
         (and (vl-string-search "*" nombre 0)
              (vl-string-search "*" nombre (1- (strlen nombre)))
         ) ;_ fin de and
     ) ;_ fin de or
     (setq num-scp (vla-get-Count
                     (vla-get-UserCoordinateSystems *aesl:dibujo*)
                   ) ;_ fin de vla-get-Count
           nombre  (strcat "SCP_" (itoa num-scp))
     ) ;_ fin de setq
     (setq nuevo-scp (ax-scp nombre
                             (getvar "UCSORG")
                             (getvar "UCSXDIR")
                             (getvar "UCSYDIR")
                     ) ;_ fin de ax-scp
     ) ;_ fin de setq
     (vla-put-ActiveUCS *aesl:dibujo* nuevo-scp)
     (list nombre (vla-GetUCSMatrix nuevo-scp))
    )
    (t
     (list (vla-get-name (vla-get-ActiveUCS *aesl:dibujo*))
           (vla-GetUCSMatrix (vla-get-ActiveUCS *aesl:dibujo*))
     ) ;_ fin de list
    )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 13.15. Funcin que devuelve la Matriz de Transformacin del SCP actual.


(defun var-vis ()
  (if (= (getvar "BLOCKEDITOR") 0)
    (PROGN (setvar "VSMONOCOLOR" " RGB:211,76,3")
           (setvar "VSFACECOLORMODE" 1)
           (setvar "VSEDGES" 0)
           (setvar "VSEDGECOLOR" "RGB:255,212,82")
           (setvar "VSISOONTOP" 0)
           (setvar "VSFACESTYLE" 2)
           (setvar "VSOBSCUREDEDGES" 0)
           (setvar "VSOCCLUDEDEDGES" 0)
           (setvar "VSFACEOPACITY" 99)
           (setvar "PERSPECTIVE" 1)
    ) ;_ fin de PROGN
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 13.16. Funcin que establece un estilo visual personalizado.

(defun ax-vista (direccion zoom / vport)
  (setq vport (vla-get-ActiveViewport *aesl:dibujo*))
  (vla-put-Direction vport (vlax-3d-point direccion))
  (vla-put-ActiveViewport *aesl:dibujo* vport)
  (vlax-release-object vport)
  (if zoom
    (vla-ZoomExtents *aesl:acad*)
  ) ;_ fin de if
  (var-vis)
  (princ)
) ;_ fin de defun
;;;Listado 13.17. Funcin que establece la direccin de la vista y el estilo visual.

(defun ax-sup ()
  (ax-vista '(0 0 1) t)
) ;_ fin de defun

(defun ax-der ()
  (ax-vista '(1 0 0) t)
) ;_ fin de defun

(defun ax-fre ()
  (ax-vista '(0 -1 0) t)
) ;_ fin de defun

(defun ax-NEs ()
  (ax-vista '(1 1 1) t)
) ;_ fin de defun

(defun ax-NWs ()
  (ax-vista '(-1 1 1) t)
) ;_ fin de defun

(defun ax-SWs ()
  (ax-vista '(-1 -1 1) t)
) ;_ fin de defun

(defun ax-SEs ()
  (ax-vista '(1 -1 1) t)
) ;_ fin de defun

(defun ax-inf ()
  (ax-vista '(0 0 -1) t)
) ;_ fin de defun

(defun ax-izq ()
  (ax-vista '(-1 0 0) t)
) ;_ fin de defun

(defun ax-tra ()
  (ax-vista '(0 1 0) t)
) ;_ fin de defun

(defun ax-NEi ()
  (ax-vista '(1 1 -1) t)
) ;_ fin de defun

(defun ax-NWi ()
  (ax-vista '(-1 1 -1) t)
) ;_ fin de defun

(defun ax-SWi ()
  (ax-vista '(-1 -1 -1) t)
) ;_ fin de defun

(defun ax-SEi ()
  (ax-vista '(1 -1 -1) t)
) ;_ fin de defun

;;;Listado 13.18. Funciones para ajustar la vista isomtrica 3D.

;;; funcin f1
(defun f1 (x y /) (cos (sqrt (+ (* x x 2) (* y y)))))

;;; funcin f2
(defun f2 (x y /) (sqrt (abs (* x y))))

;;; funcin f3
(defun f3 (x y /) (/ (* x y) 10))
;;;Listado 16.8. Funciones para el clculo de distintas formas de superficie.


(defun op-poliedro (clase /)
  (cond ((= clase "Tetraedro")
         (setq vertices '((0 0 1)
                          (0 0.94280904 -0.33333333)
                          (-0.81649658 -0.47140452 -0.33333333)
                          (0.81649658 -0.47140452 -0.33333333)
                         )
               caras    '((1 2 3) (1 3 4) (1 4 2) (2 4 3))
         ) ;_ fin de setq
        )
        ((= clase "Hexaedro")
         (setq vertices '((-0.57735 -0.57735 -0.57735)
                          (-0.57735 0.57735 -0.57735)
                          (0.57735 0.57735 -0.57735)
                          (0.57735 -0.57735 -0.57735)
                          (-0.57735 -0.57735 0.57735)
                          (-0.57735 0.57735 0.57735)
                          (0.57735 0.57735 0.57735)
                          (0.57735 -0.57735 0.57735)
                         )
               caras    '((1 2 3 4)
                          (5 6 2 1)
                          (6 7 3 2)
                          (7 8 4 3)
                          (8 5 1 4)
                          (8 7 6 5)
                         )
         ) ;_ fin de setq
        )
        ((= clase "Dodecaedro")
         (setq vertices '((0.57735 -0.187592 0.794654)
                          (0.356822 0.491123 0.794654)
                          (-0.356822 0.491123 0.794654)
                          (-0.57735 -0.187592 0.794654)
                          (-1.11022e-016 -0.607062 0.794654)
                          (0.934172 -0.303531 0.187592)
                          (0.934172 0.303531 -0.187592)
                          (0.57735 0.794654 0.187592)
                          (-5.55112e-017 0.982247 -0.187592)
                          (-0.57735 0.794654 0.187592)
                          (-0.934172 0.303531 -0.187592)
                          (-0.934172 -0.303531 0.187592)
                          (-0.57735 -0.794654 -0.187592)
                          (5.55112e-017 -0.982247 0.187592)
                          (0.57735 -0.794654 -0.187592)
                          (0.356822 -0.491123 -0.794654)
                          (0.57735 0.187592 -0.794654)
                          (-1.33234e-016 0.607062 -0.794654)
                          (-0.57735 0.187592 -0.794654)
                          (-0.356822 -0.491123 -0.794654)
                         )
               caras    '((1 2 3 4 5)
                          (1 6 7 8 2)
                          (2 8 9 10 3)
                          (3 10 11 12 4)
                          (4 12 13 14 5)
                          (5 14 15 6 1)
                          (6 15 16 17 7)
                          (8 7 17 18 9)
                          (10 9 18 19 11)
                          (12 11 19 20 13)
                          (14 13 20 16 15)
                          (16 20 19 18 17)
                         )
         ) ;_ fin de setq
        )
  ) ;_ fin de cond
) ;_ fin de defun
;;;Listado 16.20. Carga de datos de vrtices y caras para el poliedro.

(defun cal-z (xyz ecuacion dim-z / z h f-esc)
  (while xyz
    (setq z   (cons (apply ecuacion
                           (list (nth 0 xyz) (nth 1 xyz))
                    ) ;_ fin de apply
                    z
              ) ;_ fin de cons
          xyz (cdddr xyz)
    ) ;_ fin de setq
  ) ;_ fin de while
  (setq h     (- (apply 'max z) (apply 'min z))
        f-esc (/ dim-z h)
  ) ;_ fin de setq
  (reverse (mapcar '(lambda (n) (* n f-esc)) z))
) ;_ fin de defun
;;;Listado 16.23. Funcin para el clculo del valor de la coordenada Z.


(defun sol-hist (solido)
  (if (= (getvar "SOLIDHIST") 1)
    (vla-put-History solido :vlax-true)
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.8. Activar la propiedad de Registro en un Slido 3D.

(defun ax-props (obj / props valor pmin pmax bbox)
  (setq props
         (vl-remove-if
           'null
           (mapcar
             '(lambda (prop)
                (if (vlax-property-available-p obj prop)
                  (progn
                    (setq valor (vl-catch-all-apply
                                  'vlax-get-property
                                  (list obj prop)
                                ) ;_ fin de vl-catch-all-apply
                    ) ;_ fin de setq
                    (if (not (vl-catch-all-error-p valor))
                      (if (= (type valor) 'variant)
                        (cond
                          ((>= (vlax-variant-type valor)
                               8192
                           ) ;_ fin de >=
                           (cons prop
                                 (vlax-safearray->list
                                   (vlax-variant-value valor)
                                 ) ;_ fin de vlax-safearray->list
                           ) ;_ fin de cons
                          )
                          ((t
                             (cons prop
                                   (vlax-variant-value valor)
                             ) ;_ fin de cons
                           ) ;_ fin de t
                          )
                        ) ;_ fin de cond
                        (cons prop valor)
                      ) ;_ fin de if
                    ) ;_ fin de if
                  ) ;_ fin de progn
                ) ;_ fin de if
              ) ;_ fin de lambda
             '("Centroid" "MomentOfInertia" "PrincipalDirections"
               "PrincipalMoments" "ProductOfInertia"
               "RadiiOfGyration" "Volume" "Area" "Circumference"
               "Radius" "Center" "Normal" "Perimeter" "Coordinates"
               "FaceCount" "VertexCount" "Smoothness" "Elevation"
               "ArcLength" "EndAngle" "EndPoint" "StartAngle"
               "StartPoint" "TotalAngle" "Angle" "Delta"
               "Thickness" "BasePoint" "DirectionVector"
               "SecondPoint" "BaseRadius" "Height" "Position"
               "TopRadius" "TotalLength" "TurnHeight" "Turns"
               "TurnSlope" "Twist" "Direction" "TaperAngle"
               "EndDraftAngle" "EndDraftMagnitude"
               "NumCrossSections" "NumGuidePaths" "StartDraftAngle"
               "StartDraftMagnitude" "SurfaceNormals" "SurfaceType"
               "RevolutionAngle" "AxisPosition" "AxisDirection"
               "ProfileRotation" "Bank" "Length" "ProfileRotation"
               "scale"
              )
           ) ;_ fin de mapcar
         ) ;_ fin de vl-remove-if
  ) ;_ fin de setq
  (setq bbox (vl-catch-all-apply
               'vla-GetBoundingBox
               (list obj 'pmin 'pmax)
             ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (vl-catch-all-error-p bbox)
    props
    (setq props (cons (list "BoundingBox"
                            (vlax-safearray->list pmin)
                            (vlax-safearray->list pmax)
                      ) ;_ fin de list
                      props
                ) ;_ fin de cons
    ) ;_ fin de setq
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 17.20. Funcin que extrae propiedades fsicas y geomtricas de los objetos.

(defun coord-z
               (caras ecuacion dim-z / lst-z altura f-esc)
  (setq lst-z  (mapcar
                 '(lambda (cara)
                    (mapcar '(lambda (pto)
                               (apply ecuacion pto)
                             ) ;_ fin de lambda
                            cara
                    ) ;_ fin de mapcar
                  ) ;_ fin de lambda
                 caras
               ) ;_ fin de mapcar
        altura (- (apply 'max (apply 'append lst-z))
                  (apply 'min (apply 'append lst-z))
               ) ;_ fin de -
        f-esc  (/ dim-z altura)
        lst-z  (mapcar
                 '(lambda (lst-cara)
                    (mapcar '(lambda (z) (* z f-esc))
                            lst-cara
                    ) ;_ fin de mapcar
                  ) ;_ fin de lambda
                 lst-z
               ) ;_ fin de mapcar
  ) ;_ fin de setq
  (mapcar
    '(lambda (cara elev)
       (mapcar
         '(lambda (pto z) (append pto (list z)))
         cara
         elev
       ) ;_ fin de mapcar
     ) ;_ fin de lambda
    caras
    lst-z
  ) ;_ fin de mapcar
) ;_ fin de defun
;;;Listado 19.7. Funcin que calcula los valores de la coordenada Z.


(defun par-rev (x y /)
  (- (+ (expt x 2.0) (expt y 2.0)))
) ;_ fin de defun
;;;Listado 19.21. Funcin para el clculo de los vrtices de un paraboloide de revolucin.

(defun par-hip (x y) (- (expt x 2) (expt y 2)))
;;;Listado 19.22. Funcin para el clculo de los vrtices de un paraboloide hiperblico.

(defun SCP->U (/ tmp)
  (setq tmp (ax-scp "Univ"
                    '(0.0 0.0 0.0)
                    '(1.0 0.0 0.0)
                    '(0.0 1.0 0.0)
            ) ;_ fin de ax-scp
  ) ;_ fin de setq
  (vla-put-ActiveUCS *aesl:dibujo* tmp)
) ;_ fin de defun
;;;Listado 19.23. Funcin que establece como actual el sistema de coordenadas universal.

(defun ax-extrae-atrib
                       (bloque constante / atributos lista)
  (setq
    atributos (vlax-variant-value
                (if constante
                  (vla-getconstantattributes
                    bloque
                  ) ;_ fin de vla-getconstantattributes
                  (vla-getattributes bloque)
                ) ;_ fin de if
              ) ;_ fin de vlax-variant-value
  ) ;_ fin de setq
  (if
    (>= (vlax-safearray-get-u-bound atributos 1) 0)
     (foreach atrib
                    (vlax-safearray->list atributos)
       (setq lista
              (cons
                (cons
                  (vlax-get-property
                    atrib
                    "TagString"
                  ) ;_ fin de vlax-get-property
                  (if
                    (= (vla-get-MTextAttribute
                         atrib
                       ) ;_ fin de vla-get-MTextAttribute
                       :vlax-true
                    ) ;_ fin de =
                     (sustituye
                       ""
                       "\\P"
                       (vlax-get-property
                         atrib
                         "TextString"
                       ) ;_ fin de vlax-get-property
                     ) ;_ fin de sustituye
                     (vlax-get-property
                       atrib
                       "TextString"
                     ) ;_ fin de vlax-get-property
                  ) ;_ fin de if
                ) ;_ fin de cons
                lista
              ) ;_ fin de cons
       ) ;_ fin de setq
     ) ;_ fin de foreach
  ) ;_ fin de if
  lista
) ;_ fin de defun
;;;Listado 23.2. Funcin estndar para extraer valores de atributos.

(defun ax-lee-atributos
       (noment / bloque resultado atributos lista)
  (setq bloque (vlax-ename->vla-object noment))
  (setq resultado (vl-catch-all-apply
                    'vla-get-HasAttributes
                    (list bloque)
                  ) ;_ fin de vl-catch-all-apply
  ) ;_ fin de setq
  (if (eq resultado :vlax-true)
    (append (reverse (ax-extrae-atrib bloque t))
            (reverse (ax-extrae-atrib bloque nil))
    ) ;_ fin de append
  ) ;_ fin de if
) ;_ fin de defun
;;;Listado 23.3. Procesamiento de un bloque para extraer una lista con sus atributos.

(defun sel-bloque
                  (nombre / ss cont ename datos lista-datos)
  (setq cont 0)
  (if (setq ss
             (ssget
               "X"
               (list (cons 0 "INSERT") (cons 2 nombre))
             ) ;_ fin de ssget
      ) ;_ fin de setq
    (while (setq ename (ssname ss cont))
      (setq obj (vlax-ename->vla-object ename))
      (setq datos
             (list
               (cons "CAPA" (vla-get-layer obj))
               (cons
                 "COORDS"
                 (vlax-safearray->list
                   (vlax-variant-value
                     (vla-get-insertionpoint
                       obj
                     ) ;_ fin de vla-get-insertionpoint
                   ) ;_ fin de vlax-variant-value
                 ) ;_ fin de vlax-safearray->list
               ) ;_ fin de cons
             ) ;_ fin de list
      ) ;_ fin de setq
      (setq lista-datos
             (cons
               (append
                 datos
                 (ax-lee-atributos ename)
               ) ;_ fin de append
               lista-datos
             ) ;_ fin de cons
      ) ;_ fin de setq
      (setq cont (1+ cont))
    ) ;_ fin de while
  ) ;_ fin de if
  lista-datos
) ;_ fin de defun
;;;Listado 24.4. Seleccin del bloque a procesar y extraccin de sus atributos a una lista.
;|Visual LISP Format Options
(67 2 40 2 T "fin de " 55 9 0 0 1 nil T nil T)
;*** NO aadir texto a continuacin del comentario! ***|;
